#undef COM_SYM_DEF
#include "PSgraf_p.h"
void dArr0(struct dvec X0,struct dvec A,struct dvec O);
void dArr1(struct dvec X0,struct dvec A,struct dvec O);
void dArr10(struct dvec X0,struct dvec A,struct dvec O);
void dArr11(struct dvec X0,struct dvec A,struct dvec O);
/*--------------------------------------------------------------------------------------
sArrow: set length of head, widths of tail and head, and root
--------------------------------------------------------------------------------------*/
void sArrow(double lhead,double wtail,double whead,char root)
{
  arr_root=toupper(root);
  if (arr_root!='T' && arr_root!='C' && arr_root!='H') {
    PSwarning("sArrow", "unknown root; set to 'T'");
    arr_root='T';
  }

  arr_lhead=lhead; arr_wtail=wtail; arr_whead=whead;

  return;
}
/*--------------------------------------------------------------------------------------
sAArrow: set length of head, widths of tail and head, root, and tilting of annotation
--------------------------------------------------------------------------------------*/
void sArrow(double lhead,double wtail,double whead,char root,int tilt)
{
  arr_root=toupper(root);
  if (arr_root!='T' && arr_root!='C' && arr_root!='H') {
    PSwarning("sArrow", "unknown root; set to 'T'");
    arr_root='T';
  }

  arr_lhead=lhead; arr_wtail=wtail; arr_whead=whead;

  arr_tilt=tilt;

  return;
}
/*--------------------------------------------------------------------------------------
dArrow: draw arrow in coordinate system CS at origin with direction X1
        its plane is set to be orthogonal to the viewing direction;
	the dimensions of the arrow are in [mm]

HISTORY:
  v3.0.0									031122kr
--------------------------------------------------------------------------------------*/
void dArrow(char CS,int type,struct dvec X1)
{
  int dim;
  struct dvec O,X0;

  dim=X1.dim;

  if (dim==2) {
    O.dim=2;  O.x=X1.y; O.y=-X1.x;
    X0.dim=2; X0.x=0;   X0.y=0;
    dArrow(CS,type,X0,X1,O);
    return;
  }

  if (dim==3) {
    O=vecprod(R32_view,X1);
    X0.dim=3; X0.x=0; X0.y=0; X0.z=0;
    dArrow(CS,type,X0,X1,O);
    return;
  }

  PSwarning("dArrow",
	    "dimensions different from 2 or 3: operator ignored");

  return;
}
/*--------------------------------------------------------------------------------------
dArrow: draw arrow in 2d coordinate system CS at origin with direction (x1,y1)

HISTORY:
  v3.0.3									041227kr
--------------------------------------------------------------------------------------*/
void dArrow(char CS,int type,double x1,double y1)
{
  struct dvec O,X0,X1;

  X0.dim=X1.dim=O.dim=2;

  X0.x=0;  X0.y=0;
  X1.x=x1; X1.y=y1;
  O.x=X1.y; O.y=-X1.x;
  dArrow(CS,type,X0,X1,O);

  return;
}
/*--------------------------------------------------------------------------------------
dArrow: draw arrow in coordinate system CS at X0 with direction X1
        its plane is set to be orthogonal to the viewing direction;
	the dimensions of the arrow are in [mm]

HISTORY:
  v3.0.0									031122kr
--------------------------------------------------------------------------------------*/
void dArrow(char CS,int type,struct dvec X0,struct dvec X1)
{
  int dim;
  struct dvec O;

  dim=X0.dim;
  if (X1.dim!=dim) {
    PSwarning("dArrow",
	      "dimensions different: operator ignored");
    return;
  }

  if (dim==2) {
    O.dim=2; O.x=X1.y; O.y=-X1.x;
    dArrow(CS,type,X0,X1,O);
    return;
  }

  if (dim==3) {
    O=vecprod(R32_view,X1);
    dArrow(CS,type,X0,X1,O);
    return;
  }

  PSwarning("dArrow",
	    "dimensions different from 2 or 3: operator ignored");

  return;
}
/*--------------------------------------------------------------------------------------
dArrow: draw arrow in 2d coordinate system CS at (x0,y0) with direction (x1,y1)

HISTORY:
  v3.0.3									041227kr
--------------------------------------------------------------------------------------*/
void dArrow(char CS,int type,double x0,double y0,double x1,double y1)
{
  struct dvec O,X0,X1;

  X0.dim=X1.dim=O.dim=2;

  X0.x=x0; X0.y=y0;
  X1.x=x1; X1.y=y1;
  O.x=X1.y; O.y=-X1.x;
  dArrow(CS,type,X0,X1,O);

  return;
}
/*--------------------------------------------------------------------------------------
dArrow: draw arrow in coordinate system CS at X0 with direction X1
        with its plane given by X1 and O;
	the dimensions of the arrow are in [mm]

HISTORY:
  v3.0.0									031122kr
--------------------------------------------------------------------------------------*/
void dArrow(char CS,int type,struct dvec X0,struct dvec X1,struct dvec O)
{
  int dim;
  struct dvec P0;

  CS=toupper(CS);
  if (CS!='P' && CS!='W') {
    PSwarning("dArrow",
	      "requested coordinate system not implemented: operator ignored");
    return;
  }
  if (Pthick<=0 && (ColorSpace!=0 && F_c0==NONE)) {
    PSwarning("dArrow",
	      "filling pattern=NONE and pen thickness=0: operator ignored");
    return;
  }
  dim=X0.dim;
  if (X1.dim!=dim) {
    PSwarning("dArrow",
	      "dimensions different: operator ignored");
    return;
  }
  if (dim!=2 && dim!=3) {
    PSwarning("dArrow",
	      "dimensions different from 2 or 3: operator ignored");
    return;
  }
  if (norm(vecprod(X1,O))/(norm(X1)*norm(O))<0.001) {
    PSwarning("dArrow",
	      "vectors X1 and O practically parallel: operator ignored");
    return;
  }

  if (arr_root=='C') X0=subvec(X0,scalmult(0.5,X1));
  if (arr_root=='H') X0=subvec(X0,X1);

  if (CS=='W') {
    pWP(X0,&X0); pWP(X1,&X1); pWP(O,&O);
    P0.dim=X0.dim; P0.x=P0.y=P0.z=0; pWP(P0,&P0);
    X1=subvec(X1,P0); O=subvec(O,P0);
  }

  if (type==0)  {dArr0(X0,X1,O);  return;}
  if (type==1)  {dArr1(X0,X1,O);  return;}

  if (type==10) {dArr10(X0,X1,O); return;}
  if (type==11) {dArr11(X0,X1,O); return;}

  PSwarning("dArrow","arrow type not implemented: operator ignored");
  return;
}
/*--------------------------------------------------------------------------------------
dAArrow: draw annotated arrow in coordinate system CS at origin with direction X1
        its plane is set to be orthogonal to the viewing direction;
	the dimensions of the arrow are in [mm]

HISTORY:
  v3.0.0									031122kr
--------------------------------------------------------------------------------------*/
void dAArrow(char CS,int type,struct dvec X1,const char*text)
{
  double angle,old_angle;
  struct dvec dx,x12;

  dx=normalize(vecprod(R32_view,X1)); 	// check if R32_view is defined for 2d plots
  dx=scalmult(0.8*arr_wtail+1,dx);	// 0.8: larger distance for thick arrows
  dx=addvec(dx,scalmult(0.5,X1));

  pWP(X1,&x12);
  angle=atan2(x12.y,x12.x)/Deg_to_Rad;

  if (arr_tilt) {
    old_angle=TextRotation;
    sTextRotation(angle);
    movea('P',dx); dText(text);
    TextRotation=old_angle;
  }

  dArrow(CS,type,X1);

  return;
}
/*--------------------------------------------------------------------------------------
dAArrow: draw annotated arrow in coordinate system CS at X0 with direction X1
        its plane is set to be orthogonal to the viewing direction;
	the dimensions of the arrow are in [mm]

HISTORY:
  v3.0.0									031122kr
--------------------------------------------------------------------------------------*/
void dAArrow(char CS,int type,struct dvec X0,struct dvec X1,const char *text)
{
  int dim;

  PSwarning("dAArrow(X0,X1)",
	    "not implemented yet: operator ignored");
  return;

  dim=X0.dim;
  if (X1.dim!=dim) {
    PSwarning("dAArrow",
	      "dimensions different: operator ignored");
    return;
  }


  return;
}
/*--------------------------------------------------------------------------------------
dAArrow: draw annotated arrow in coordinate system CS at X0 with direction X1
        with its plane given by X1 and O;
	the dimensions of the arrow are in [mm]

HISTORY:
  v3.0.0									031122kr
--------------------------------------------------------------------------------------*/
void dAArrow(char CS,int type,struct dvec X0,struct dvec X1,struct dvec O,const char *text)
{
  int dim;

  PSwarning("dAArrow(X0,X1,O)",
	    "not implemented yet: operator ignored");
  return;


  CS=toupper(CS);
  if (CS!='P' && CS!='W') {
    PSwarning("dAArrow",
	      "requested coordinate system not implemented: operator ignored");
    return;
  }
  if (Pthick<=0 && 
      (ColorSpace!=0 && F_c0==NONE)) {
      //      ((ColorSpace==0 && F_gry==NONE) || (ColorSpace!=0 && F_c0==NONE))) {
    PSwarning("dAArrow",
	      "filling pattern=NONE and pen thickness=0: operator ignored");
    return;
  }
  dim=X0.dim;
  if (X1.dim!=dim) {
    PSwarning("dAArrow",
	      "dimensions different: operator ignored");
    return;
  }
  if (dim!=2 && dim!=3) {
    PSwarning("dAArrow",
	      "dimensions different from 2 or 3: operator ignored");
    return;
  }
  if (norm(vecprod(X1,O))/(norm(X1)*norm(O))<0.001) {
    PSwarning("dAArrow",
	      "vectors X1 and O practically parallel: operator ignored");
    return;
  }

  return;
}
/*--------------------------------------------------------------------------------------
dArr0: draw arrow 0 (single headed) defined by
         X0: tail end
         A:  arrow vector
         O:  orientation of arrow plane (O and A non-parallel)
       in paper coordinates
--------------------------------------------------------------------------------------*/
void dArr0(struct dvec X0,struct dvec A,struct dvec O)
{
  double l;
  struct dvec ot,oh,at,onorm,x[8];

  l=norm(A);
  at=scalmult(1-arr_lhead/l,A);
  onorm=normalize(subvec(O,scalmult(dotprod(O,A)/pow(l,2),A)));
  ot=scalmult(arr_wtail/2,onorm);
  oh=scalmult((arr_whead-arr_wtail)/2,onorm);

  x[0]=subvec(X0,ot);
  x[1]=addvec(x[0],at);
  x[2]=subvec(x[1],oh);
  x[3]=addvec(X0,A);
  x[6]=addvec(X0,ot);
  x[5]=addvec(x[6],at);
  x[4]=addvec(x[5],oh);
  x[7]=x[0];

  dPolygon('P',x,8);
  
  return;
}
/*--------------------------------------------------------------------------------------
dArr1: draw arrow 1 (lines, single headed) defined by
         X0: tail end
         A:  arrow vector
         O:  orientation of arrow plane (O and A non-parallel)
       in paper coordinates
--------------------------------------------------------------------------------------*/
void dArr1(struct dvec X0,struct dvec A,struct dvec O)
{
  double l;
  struct dvec oh,at,buf;

  l=norm(A);
  at=scalmult(1-arr_lhead/l,A);
  buf=normalize(subvec(O,scalmult(dotprod(O,A)/pow(l,2),A)));
  oh=scalmult(arr_whead/2,buf);

  buf=addvec(X0,A);
  dLine('P',X0,buf);
  dLine('P',buf,addvec(addvec(X0,at),oh));
  dLine('P',buf,subvec(addvec(X0,at),oh));
  
  return;
}
/*--------------------------------------------------------------------------------------
dArr10: draw arrow 10 (double headed) defined by
         X0: tail end
         A:  arrow vector
         O:  orientation of arrow plane (O and A non-parallel)
       in paper coordinates
--------------------------------------------------------------------------------------*/
void dArr10(struct dvec X0,struct dvec A,struct dvec O)
{
  double l;
  struct dvec ot,oh,oth,at,ats,buf,x[11];

  l=norm(A);
  at =scalmult(1-arr_lhead/l,A);
  ats=scalmult(arr_lhead/l,A);
  buf=normalize(subvec(O,scalmult(dotprod(O,A)/pow(l,2),A)));
  ot =scalmult(arr_wtail/2,buf);
  oth=scalmult(arr_whead/2,buf);
  oh =scalmult((arr_whead-arr_wtail)/2,buf);

  x[0]=X0;
  buf=addvec(x[0],ats);
  x[1]=subvec(buf,oth); x[2]=subvec(buf,ot);
  x[9]=addvec(buf,oth); x[8]=addvec(buf,ot);
  buf=addvec(x[0],at);
  x[4]=subvec(buf,oth); x[3]=subvec(buf,ot);
  x[6]=addvec(buf,oth); x[7]=addvec(buf,ot);
  x[5]=addvec(x[0],A);
  x[10]=x[0];

  dPolygon('P',x,11);
  
  return;
}
/*--------------------------------------------------------------------------------------
dArr11: draw arrow 11 (lines, double headed) defined by
         X0: tail end
         A:  arrow vector
         O:  orientation of arrow plane (O and A non-parallel)
       in paper coordinates
--------------------------------------------------------------------------------------*/
void dArr11(struct dvec X0,struct dvec A,struct dvec O)
{
  double l;
  struct dvec oh,at,ats,buf;

  l=norm(A);
  at =scalmult(1-arr_lhead/l,A);
  ats=scalmult(arr_lhead/l,A);
  buf=normalize(subvec(O,scalmult(dotprod(O,A)/pow(l,2),A)));
  oh=scalmult(arr_whead/2,buf);

  buf=addvec(X0,A);
  dLine('P',X0,buf);
  dLine('P',buf,addvec(addvec(X0,at),oh));
  dLine('P',buf,subvec(addvec(X0,at),oh));
  dLine('P',X0,addvec(addvec(X0,ats),oh));
  dLine('P',X0,subvec(addvec(X0,ats),oh));
  
  return;
}
