#undef COM_SYM_DEF
#include "PSgraf_p.h"
/*--------------------------------------------------------------------------------------
sCBlock: set parameters for contoured block

HISTORY:
  v3.0.0									031226kr
--------------------------------------------------------------------------------------*/
void sCBlock(double x0,double dx,int Nx,int il,int iu,
	     double y0,double dy,int Ny,int jl,int ju,
	     double z0,double dz,int Nz,int kl,int ku,int outer,struct dvec illum)
{
  if (il<0 || il>=iu || iu>=Nx) {
    PSwarning("sCBlock","interval in x-direction not regular: set to full");
    il=0; iu=Nx-1;
  }
  if (jl<0 || jl>=ju || ju>=Ny) {
    PSwarning("sCBlock","interval in y-direction not regular: set to full");
    jl=0; ju=Ny-1;
  }
  if (kl<0 || kl>=ku || ku>=Nz) {
    PSwarning("sCBlock","interval in z-direction not regular: set to full");
    kl=0; ku=Nz-1;
  }

  CB_Nx=Nx; CB_il=il; CB_iu=iu; CB_x0=x0; CB_dx=dx;
  CB_Ny=Ny; CB_jl=jl; CB_ju=ju; CB_y0=y0; CB_dy=dy;
  CB_Nz=Nz; CB_kl=kl; CB_ku=ku; CB_z0=z0; CB_dz=dz;
 
  CB_outer=outer;
  if (illum.dim!=3)
    CB_illum.dim=0;
  else
    CB_illum=normalize(illum);

  return;
}
/*--------------------------------------------------------------------------------------
dCBlock: draw contoured block from data f[NX][NY][NZ]

HISTORY:
  v3.0.0									031226kr
--------------------------------------------------------------------------------------*/
void dim_ctr(struct color*orig,double dim,struct color*dimmed);
void dCBlock(double***f,int filled,int framed)
{
  int LOC,oldN,i,j,k;
  double *z,*oldctr,*CB_ctr;
  struct dvec X0,X1,X2;
  struct color *oldfill,*CB_col;

  if (L_!=NULL || B_!=NULL) {
    PSwarning("dCBlock","previously existing grid deleted");
    deleteGrid();
  }

  if (CB_illum.dim==3) {				// get ctr and fills
    oldN=N_Ctr;
    oldctr=C_ctr;
    oldfill=C_col-1;
    if ((CB_ctr=(double*)malloc(oldN*sizeof(double)))==NULL)
      PSerror("dCBlock","memory overflow");
    if ((CB_col=(struct color*)malloc((oldN+1)*sizeof(struct color)))==NULL)
      PSerror("dCBlock","memory overflow");
  } else {
    oldN=0; oldctr=CB_ctr=NULL; oldfill=CB_col=NULL;
  }

  if (CB_Nx>CB_Ny) {
    if (CB_Ny>CB_Nz) i=CB_Nx*CB_Ny; else i=CB_Nx*CB_Nz;
  } else {
    if (CB_Nx>CB_Nz) i=CB_Nx*CB_Ny; else i=CB_Ny*CB_Nz;
  }
  if ((z=(double*)malloc(i*sizeof(double)))==NULL) PSerror("dCBlock","memory overflow");

  X0.dim=X1.dim=X2.dim=3;

  if (fabs(R32_view.x)>0.001) {				// face perpendicular to x
    if (CB_illum.dim==3) {
      dim_ctr(oldfill,-CB_illum.x,CB_col);
      sContours(oldctr,oldN,CB_col);
    }
    LOC=((R32_view.x<0 && CB_outer) || (R32_view.x>0 && !CB_outer))?CB_iu:CB_il;
    X0.x=CB_x0+LOC*CB_dx; X0.y=CB_y0+CB_jl*CB_dy; X0.z=CB_z0+CB_kl*CB_dz;
    X1.x=0;                 X1.y=CB_dy;             X1.z=0;
    X2.x=0;                 X2.y=0;                 X2.z=CB_dz;
    gRGrid(X0,X1,X2,CB_ju-CB_jl+1,CB_ku-CB_kl+1);
    for (j=0;j<=CB_ju-CB_jl;j++)
      for (k=0;k<=CB_ku-CB_kl;k++)
	z[j+(CB_ju-CB_jl+1)*k]=f[LOC][j+CB_jl][k+CB_kl];
    dContours(z,filled,framed);
    dGridBoundary(0,0);
    deleteGrid();
  }

  if (fabs(R32_view.y)>0.001) {				// face perpendicular to y
    if (CB_illum.dim==3) {
      dim_ctr(oldfill,-CB_illum.y,CB_col);
      sContours(oldctr,oldN,CB_col);
    }
    LOC=((R32_view.y<0 && CB_outer) || (R32_view.y>0 && !CB_outer))?CB_ju:CB_jl;
    X0.x=CB_x0+CB_il*CB_dx; X0.y=CB_y0+LOC*CB_dy; X0.z=CB_z0+CB_kl*CB_dz;
    X1.x=CB_dx;             X1.y=0;                 X1.z=0;
    X2.x=0;                 X2.y=0;                 X2.z=CB_dz;
    gRGrid(X0,X1,X2,CB_iu-CB_il+1,CB_ku-CB_kl+1);
    for (i=0;i<=CB_iu-CB_il;i++)
      for (k=0;k<=CB_ku-CB_kl;k++)
	z[i+(CB_iu-CB_il+1)*k]=f[i+CB_il][LOC][k+CB_kl];
    dContours(z,filled,framed);
    dGridBoundary(0,0);
    deleteGrid();
  }

  if (fabs(R32_view.z)>0.001) {				// face perpendicular to z
    if (CB_illum.dim==3) {
      dim_ctr(oldfill,-CB_illum.z,CB_col);
      sContours(oldctr,oldN,CB_col);
    }
    LOC=((R32_view.z<0 && CB_outer) || (R32_view.z>0 && !CB_outer))?CB_ku:CB_kl;
    X0.x=CB_x0+CB_il*CB_dx; X0.y=CB_y0+CB_jl*CB_dy; X0.z=CB_z0+LOC*CB_dz;
    X1.x=CB_dx;             X1.y=0;                 X1.z=0;
    X2.x=0;                 X2.y=CB_dy;             X2.z=0;
    gRGrid(X0,X1,X2,CB_iu-CB_il+1,CB_ju-CB_jl+1);
    for (i=0;i<=CB_iu-CB_il;i++)
      for (j=0;j<=CB_ju-CB_jl;j++)
	z[i+(CB_iu-CB_il+1)*j]=f[i+CB_il][j+CB_jl][LOC];
    dContours(z,filled,framed);
    dGridBoundary(0,0);
    deleteGrid();
  }
  
  if (CB_illum.dim==3) {				// restore saved ctr and fills
    sContours(oldctr,oldN,oldfill);
    free((void*)CB_ctr); free((void*)CB_col);
  }
  free((void*)z);

  return;
}
/*------------------------------------------------------------------------------------*/
void dim_ctr(struct color*orig,double dim,struct color*dimmed)
{
  int i;

  if (dim<0) dim=0;

  if ((orig[0]).CS==0)
    for (i=0;i<=N_Ctr;i++) {
      dimmed[i].CS=orig[i].CS;
      dimmed[i].c0=dim*orig[i].c0;
    }
  if ((orig[0]).CS==2)
    for (i=0;i<=N_Ctr;i++) {
      dimmed[i].CS=orig[i].CS;
      dimmed[i].c0=dim*orig[i].c0;
      dimmed[i].c1=dim*orig[i].c1;
      dimmed[i].c2=dim*orig[i].c2;
    }  
  if ((orig[0]).CS==4)
    for (i=0;i<=N_Ctr;i++) {
      dimmed[i].CS=orig[i].CS;
      dimmed[i].c0=orig[i].c0;
      dimmed[i].c1=orig[i].c1;
      dimmed[i].c2=dim*orig[i].c2;
    }
  if ((orig[0]).CS==8)
    for (i=0;i<=N_Ctr;i++) {
      dimmed[i].CS=orig[i].CS;
      dimmed[i].c0=dim*orig[i].c0;
      dimmed[i].c1=dim*orig[i].c1;
      dimmed[i].c2=dim*orig[i].c2;
      dimmed[i].c3=orig[i].c3;
    }  

  return;
}
