#undef COM_SYM_DEF
#include "PSgraf_p.h"
/*--------------------------------------------------------------------------------------
sDataPoint: set symbol (-1 --> no symbol) for data point and
            length of end bars on error [mm]

HISTORY:
  v3.0.0									031129kr
--------------------------------------------------------------------------------------*/
void sDataPoint(int symbol,double rsymbol,double barlength)
{
  dp_sym=symbol;
  dp_rsym=rsymbol;
  dp_bar=barlength;

  return;
}
/*--------------------------------------------------------------------------------------
dDataPoint: draw data point together with error bars

HISTORY:
  v3.0.0									031129kr
--------------------------------------------------------------------------------------*/
void dDataPoint(char CS,struct dvec X,struct dvec dX)
{
  struct dvec x;

  if (Pthick<=0) {
    PSwarning("dDataPoint",
	      "filling pattern=NONE and pen thickness=0: operator ignored");
    return;
  }
    
  CS=toupper(CS);
  if (CS!='W' && CS!='P') {
    PSwarning("dDataPoint",
	      "requested coordinate system not implemented: operator ignored");
    return;
  }
  if (X.dim!=2) {
    PSwarning("dDataPoint","only implemented for two-dimensional data: operator ignored");
    return;
  }
  if (X.dim!=dX.dim) {
    PSwarning("dDataPoint",
	      "dimensions of data point and errors different operator ignored");
    return;
  }


  fprintf(PSout, "%%--dDataPoint\n");
  fprintf(PSout,"gsave\n");
  fprintf(PSout,"newpath\n");
  if (dX.x>0) {				// x-error
    x=identity(X);
    x.x-=dX.x; movea(CS,x);
    if (PWx*dX.x>1.2*dp_rsym) {mover('P',0,-dp_bar/2); liner('P',0,dp_bar);}
    movea(CS,x); x.x+=2*dX.x; linea(CS,x);
    if (PWx*dX.x>1.2*dp_rsym) {mover('P',0,-dp_bar/2); liner('P',0,dp_bar);}
  }
  if (dX.y>0) {				// y-error
    x=identity(X);
    x.y-=dX.y; movea(CS,x);
    if (PWy*dX.y>1.2*dp_rsym) {mover('P',-dp_bar/2,0); liner('P',dp_bar,0);}
    movea(CS,x); x.y+=2*dX.y; linea(CS,x);
    if (PWy*dX.y>1.2*dp_rsym) {mover('P',-dp_bar/2,0); liner('P',dp_bar,0);}
  }
  fprintf(PSout,"closepath\n");
  if (F_c0!=NONE) {
    if (Pthick>0) fprintf(PSout,"gsave ");
    if (ColorSpace==0)
      fprintf(PSout,"%f setgray fill ",1-F_c0);
    else if (ColorSpace==2)
      fprintf(PSout,"%f %f %f setrgbcolor fill ",F_c0,F_c1,F_c2);
    else if (ColorSpace==4)
      fprintf(PSout,"%f %f %f sethsbcolor fill ",F_c0,F_c1,F_c2);
    else if (ColorSpace==8)
      fprintf(PSout,"%f %f %f %f setcmykcolor fill ",F_c0,F_c1,F_c2,F_c3);
    if (Pthick>0) fprintf(PSout,"grestore ");
  }
  if (Pthick>0) fprintf(PSout,"stroke\n"); else fprintf(PSout,"\n");
  fprintf(PSout,"grestore\n");

  if (dp_sym>=0) dSymbol(CS,X,dp_rsym,dp_sym);

  return;
}
/*--------------------------------------------------------------------------------------
dDataPoint: draw data point (x,y) together with error bars (dx,dy)
--------------------------------------------------------------------------------------*/
void dDataPoint(char CS,double x,double dx,double y,double dy)
{
  struct dvec X,dX;

  X.dim=dX.dim=2;
  X.x=x; X.y=y;
  dX.x=dx; dX.y=dy;

  dDataPoint(CS,X,dX);

  return;
}
