/*---------------------------------------------------------------------------------------
VERSION 3.0.1, COPYRIGHT 1991,1992,1995,1997,2000,2003 K. ROTH

PSgraf IS LICENSED FREE OF CHARGE. THEREFORE THIS FILE, AND ALL THE ACCOMPANYING FILES
WHICH IN THEIR ENTIRETY CONSTITUTE PSgraf, ARE PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
KIND, WHETHER EXPRESSED OR IMPLIED. YOU ARE RESPONSIBLE FOR ASCERTAINING THE FITNESS OF
PSgraf FOR ANY SPECIFIC USE, AND CONSEQUENTLY YOU ASSUME ALL THE RESPONSIBILITIES AND
COST THAT MAY ARISE FROM USING IT.
---------------------------------------------------------------------------------------*/

/*
PSgraf3.h: header file for PSgraf

history:
  debug dSymbol, added pPW (3.0.1)	       					040623kr
  simple 3d objects added; introduce struct dvec; C++ (3.0.0)                   031115kr
  encapsulated bitmap preview (2.4.1)                                           011226kr
  arbitrary triangular grids (2.4.0a)                                           001216kr
  added bitmaps and TIFF                                                        971121kr
  delete magnification and sPaperCoord                                          950330kr
  make all coordinates positive	(compatibility with other OS)                   950330kr
  variable argument lists for gray/color                                        920819kr
  color added (2.0)                                                             920523kr
  created based on SciGraf for the Macintosh (1.0)                              910608kr
  
modifications:
*/

/*-------------------------------------------------------------- begin include watcher */
#ifndef PSgrafVersion


/*------------------------------------------------------------------------ definitions */
#define PSgrafVersion 303
#define NONE -1

struct dvec {                   /* vector in space */
  int dim;                              /* dimension */
  double x,y,z;                         /* coordinates */
};

struct color {                  /* color */
  int CS;                               /* color space (G=0, RGB=2, HSB=4, CMYK=8) */
  double c0,c1,c2,c3;                   /* coordinates */
};

/*-------------------------------------------------- vector operations (vectorops.cpp) */
struct dvec identity(struct dvec x);
struct dvec negvec(struct dvec x);
double      norm(struct dvec x);
void        normalize(struct dvec*x);
struct dvec normalize(struct dvec x);
struct dvec addvec(struct dvec a,struct dvec b);
struct dvec subvec(struct dvec a,struct dvec b);
struct dvec scalmult(double a,struct dvec b);
double      dotprod(struct dvec a,struct dvec b);
void        vecprod(struct dvec a,struct dvec b,struct dvec*c);
struct dvec vecprod(struct dvec a,struct dvec b);

/*-------------------------------------------------------- projectors (projectors.cpp) */
struct dvec pPI(struct dvec Xp);
void        pPI(struct dvec Xp,struct dvec*Xi);
void        pPI(double xp,double yp,double zp,double*xi,double*yi);
struct dvec pPW(struct dvec Xp);
void        pPW(struct dvec Xp,struct dvec *Xw);
void        pPW(double xp,double yp,double*xw,double*yw);
void        pPW(double xp,double yp,double zp,double*xw,double*yw,double*zw);
struct dvec pWI(struct dvec Xw);
void        pWI(struct dvec Xw,struct dvec*Xi);
void        pWI(double xw,double yw,double zw,double*xi,double*yi);
struct dvec pWP(struct dvec Xw);
void        pWP(struct dvec Xw,struct dvec*Xp);
void        pWP(double xw,double yw,double*xp,double*yp);
void        pWP(double xw,double yw,double zw,double*xp,double*yp,double*zp);

/*------------------------------------------------------------------------- functions */
void dArrow(char CS,int type,struct dvec X1);
void dArrow(char CS,int type,double x1,double y1);
void dArrow(char CS,int type,struct dvec X0,struct dvec X1);
void dArrow(char CS,int type,double x0,double y0,double x1,double y1);
void dArrow(char CS,int type,struct dvec X0,struct dvec X1,struct dvec O);
void dAArrow(char CS,int type,struct dvec X1,const char*text);
void dAArrow(char CS,int type,struct dvec X0,struct dvec X1,const char*text);
void dAArrow(char CS,int type,struct dvec X0,struct dvec X1,struct dvec O,const char*text);
void dBars(char CS,double *x,double *f,int N);
void dBitMap(double x0,double y0,double dpx,double dpy,int Nx,int Ny,
	     double*gry);
void dBitMap(double x0,double y0,double dpx,double dpy,int Nx,int Ny,
	     double*c0,double*c1,double*c2);
void dBitMap(double x0,double y0,double dpx,double dpy,int Nx,int Ny,
	     double*c0,double*c1,double*c2,double*c3);
void dCBlock(double***f,int filled,int framed);
void dContours(double*Z,int filled,int framed);
void dDataPoint(char CS,struct dvec X,struct dvec dX);
void dDataPoint(char CS,double x,double dx,double y,double dy);
void dDensity(char CS,double x0,double y0,double dx,double dy,int Nx,int Ny,
	      double*zd,double misval);
void deleteGrid(void);
void dGrid(int Node_numbers,int Line_numbers);
void dGridBoundary(int Node_numbers,int Line_numbers);
void dLegend(double x0,double y0,double dx,double dy,int horiz,int nth);
void dLine(char CS,struct dvec x0,struct dvec x1);
void dLine(char CS,double x0,double y0,double x1,double y1);
void dLine(char CS,double x0,double y0,double z0,double x1,double y1,double z1);
void dMissing(int*missing,struct color misscol);
void dMissing(int*missing);
void dNumber(double thenumber);
void dParallel(char CS,struct dvec x0,struct dvec s0,struct dvec s1);
void dParallel(char CS,struct dvec x0,struct dvec s0,struct dvec s1,struct dvec s2);
void dPolygon(char CS,struct dvec*x,int N);
void dPolygon(char CS,double*x,double*y,int N);
void dPolygon(char CS,double*x,double*y,double*z,int N);
void dRectangle(char CS,double llx,double lly,double urx,double ury,double theta);
void dRectangle(char CS,double llx,double lly,double urx,double ury);
void dSymbol(char CS,struct dvec x,double R,int SY);
void dSymbol(char CS,double x,double y,double R,int SY);
void dSymbol(char CS,double x,double y,double z,double R,int SY);
void dText(const char*text);
void dTIFF(double x0,double y0,double*dpx,double*dpy,const char*fn);
void dXAxis(struct dvec X0,double x1,int below);
void dXAxis(double x0,double y0,double x1,int below);
void dXAxis(double x0,double y0,double z0,double x1,int below);
void dYAxis(struct dvec X0,double y1,int atleft);
void dYAxis(double x0,double y0,double y1,int atleft);
void dYAxis(double x0,double y0,double z0,double y1,int atleft);
void dZAxis(struct dvec X0,double z1,int below);
void dZAxis(double x0,double y0,double z1,int below);
void dZAxis(double x0,double y0,double z0,double z1,int below);
void endPS(void);
int gColorSpace();
void gContours(double*ctr,int*N,struct color*c);
void gDRGrid(double*x,double*y,int Nx,int Ny);
void gDRGrid(double*x,double*y,double*z,int Nx,int Ny);
struct color gFill();
void gRGrid(struct dvec x0,struct dvec x1,struct dvec x2,int N1,int N2);
void gRGrid(double x0,double dx,int Nx,double y0,double dy,int Ny);
void gSRGrid(struct dvec x0,struct dvec x1,int N1,struct dvec x2,int N2);
void gSRGrid(double*x,int Nx,double*y,int Ny);
struct color gStroke();
void gTGrid(double*x,double*y,int NN,int*b,int*e,int NL);
void gPaper(char*name);
void gPreview(void);
int  gTCover();
int  interpolate(double Z[],int missing[]);
void iTCover(int NT,int**nodes,double*cgx,double*cgy);
void MinMax(double*X,int N,double*xmin,double*xmax);
void movea(char CS,struct dvec x);
void movea(char CS,double x,double y);
void movea(char CS,double x,double y,double z);
void mover(char CS,struct dvec x);
void mover(char CS,double x,double y);
void mover(char CS,double x,double y,double z);
void PSerror(const char*where,const char*what);
void PSwarning(const char*where,const char*what);
void sArrow(double lhead,double wtail,double whead,char root);
void sCBlock(double x0,double dx,int Nx,int il,int iu,
	     double y0,double dy,int Ny,int jl,int ju,
	     double z0,double dz,int Nz,int kl,int ku,int outer,struct dvec illum);
void sClipping(char CS,double xmin,double xmax,double ymin,double ymax);
void sClipping();
int  sColorSpace(int cspace);
int  sColorSpace(const char*cspace);
void sContours(double*ctr,int N,struct color*c);
void sContours(double*ctr,int N,double*c0);
void sContours(double*ctr,int N,double*c0,double*c1,double*c2);
void sContours(double*ctr,int N,double*c0,double*c1,double*c2,double*c3);
void sDash(int ip);
void sDash(const char*dpat);
void sDataPoint(int symbol,double rsymbol,double barlength);
void sDensity(double*val,int N,struct color*c);
void sDensity(double*val,int N,double*c0);
void sDensity(double*val,int N,double*c0,double*c1,double*c2);
void sDensity(double*val,int N,double*c0,double*c1,double*c2,double*c3);
void sFill();
void sFill(struct color c);
void sFill(double c0);
void sFill(double c0,double c1,double c2);
void sFill(double c0,double c1,double c2,double c3);
void sIPlane(struct dvec x1,struct dvec x2);
void sIPlane(struct dvec x0,struct dvec x1,struct dvec x2);
void sMinPixel(double pixelsize);
void sNumber(char format,int prec);
void sStroke(struct color c);
void sStroke(double c0);
void sStroke(double c0,double c1,double c2);
void sStroke(double c0,double c1,double c2,double c3);
void sTeXStyle(const char*texsty,int layered);
void sText(const char*font,double size,char hadjust,char vadjust);
void sTextRotation(double theta);
void sThickness(double thick);
void sView(struct dvec view,struct dvec right);
void sView(struct dvec x0,struct dvec view,struct dvec right);
void sXIntervals(double interval0,double interval1,double interval2,int precision);
void sXTicks(double tick0,double tick1,double tick2);
void sXWorldCoord(double wleft,double wright,double pleft,double pright);
void sYIntervals(double interval0,double interval1,double interval2,int precision);
void sYTicks(double tick0,double tick1,double tick2);
void sYWorldCoord(double wlow,double whigh,double plow,double phigh);
void sZIntervals(double interval0,double interval1,double interval2,int precision);
void sZTicks(double tick0,double tick1,double tick2);
void sZWorldCoord(double wlow,double whigh,double plow,double phigh);
void sZWorldCoord();

/*---------------------------------------------------------------- end include watcher */
#endif
