#undef COM_SYM_DEF
#include "PSgraf_p.h"

void put_label(double T0x,double T0y,double T1x,double T1y,double thenumber);
/*============================================================================ x-axis */
/*--------------------------------------------------------------------------------------
dXAxis: draw x-axis from x0 to x=x1, if (below) draw ticks below axis

HISTORY:
  v3.0.0									031115kr
  change format of labels from f to g						940619kr
  v2.0										920524kr
  label end-points if axis is too short and issue warning			920205kr
--------------------------------------------------------------------------------------*/

void dXAxis(struct dvec X0,double x1,int below)
{
  char oldhadj,oldvadj,oldform;
  int prec,oldprec,nl[3],nh[3],n0,ndigit,dim,i,j;
  double dx[3],dx0,delta,deltaP=0,xl,xh,y;
  struct dvec X1,b0,b1,xe,ye;
  struct color oldfill;
  
  dim=X0.dim;

  if (z_set && dim!=3) {
    PSwarning("dXAxis","origin must be defined in three dimensions: operator ignored");
    return;
  }

  if (dim!=2 && dim!=3)
    PSerror("dXAxis","dimension different from 2 or 3");

  if (x1==X0.x) {
    PSwarning("dXAxis","length of axis is 0");
    return;
  }

  oldhadj=hadj; oldvadj=vadj;			/* save previous attributes */
  oldform=nformat; oldprec=precision;
  hadj='C';
  if (xauto) {
    delta=fabs(x1-X0.x);				/* estimate 0-order interval */
    b0=pWI(X0);
    X1=identity(X0); X1.x=x1;
    b1=pWI(X1);
    deltaP=sqrt(pow(b1.x-b0.x,2)+pow(b1.y-b0.y,2));
//     b0.dim=dim;
//     b0.x=0; b0.y=0;
//     if (dim==3) {b0.z=0; pWI(b0,&b1);} else pWP(b0,&b1);
//     P0=sqrt(pow(b1.x,2)+pow(b1.y,2));
//     b0.x=delta;     
//     if (dim==3) pWI(b0,&b1); else pWP(b0,&b1);
//     deltaP=sqrt(pow(b1.x,2)+pow(b1.y,2))-P0;

    if ((ndigit=(int)(log10(0.1*delta)))<0) ndigit--;
    if (ndigit>4 || ndigit<-4) {
      nformat='g'; prec=1;
    } else {
      //hj  nformat='f'; prec=(ndigit<0)?-ndigit-1:0;
      nformat='f'; prec=(ndigit<0)?-ndigit:0;
    }
    dx0=pow(10,ndigit);
    ndigit=(ndigit>0)?ndigit+1:-ndigit+2;
    for (;;) {					/* search through one decade */
      if ((n0=(int)(delta/dx0))==0) goto autofail;
      if (n0<10 && n0*ndigit*curFSize*X_stretch/deltaP<0.3) {dx[1]=dx0/2; break;}
      dx0 *= 2; if ((n0=(int)(delta/dx0))==0) goto autofail;
      if (n0<10 && n0*ndigit*curFSize*X_stretch/deltaP<0.3) {dx[1]=dx0/2; break;}
      dx0 *= 2.5; if ((n0=(int)(delta/dx0))==0) goto autofail;
      if (n0<10 && n0*ndigit*curFSize*X_stretch/deltaP<0.3) {dx[1]=dx0/5; break;}
      dx0 *= 2; ndigit=((ndigit=(int)(log10(dx0)))>0)?ndigit+1:-ndigit+2;
    }
    dx[0]=dx0; dx[2]=0.1*dx0;
  }
  else {prec=xprecision; for (i=0; i<3; i++) dx[i]=xinterval[i];}
  
  xl=(X0.x<x1)?X0.x:x1; xh=(X0.x>x1)?X0.x:x1;
  if (dx[0]>0 && (fabs(xl/dx[0])>(double)INT_MAX || fabs(xh/dx[0])>(double)INT_MAX))
    goto autofail;
  for (i=0; i<3; i++) 
    if (dx[i]>0) {
      nl[i]=(int)(xl/dx[i]); nh[i]=(int)(xh/dx[i]);
      if (xl>0) nl[i]++; if (xh<0) nh[i]--;
    }
  fprintf(PSout,"%%---------- dXAxis\n");
  b0.dim=dim; b0.x=X0.x; b0.y=X0.y; if (dim==3) b0.z=X0.z;
  b1.dim=dim; b1.x=x1;   b1.y=X0.y; if (dim==3) b1.z=X0.z;
  dLine('W',b0,b1);

  for (i=0; i<3; i++)				/* ticks without labeling */
    if (dx[i]>0) {
      y=(below)?-xTiMaLe[i]:xTiMaLe[i];
      fprintf(PSout,"%%   %i order tick marks\n",i);
      fprintf(PSout,"newpath\n");
      for (j=nl[i]; j <= nh[i]; j++) {
	b0.x=j*dx[i]; b0.y=X0.y; if (dim==3) b0.z=X0.z;
	movea('W',b0);
	b0.x=0;       b0.y=y;    if (dim==3) b0.z=0;
	liner('P',b0);
      }
      fprintf(PSout,"  closepath\n");
      fprintf(PSout,"stroke\n");
    }

  if (dx[0]>0 && prec>=0) {			/* labeling 0-order ticks */
    precision=prec;
    y=(below)?-xTiMaLe[0]:xTiMaLe[0];
    for (j=nl[0]; j<=nh[0]; j++) {
      b1.x=j*dx[0]; b1.y=X0.y;
      if (dim==3) {b1.z=X0.z; xe=pWI(b1);} else xe=pWP(b1);
      X1=pWP(b1); X1.y+=y; if (dim==3) ye=pPI(X1); else ye=identity(X1);
      put_label(xe.x,xe.y,ye.x,ye.y,j*dx[0]);
    }
  }
  hadj=oldhadj; vadj=oldvadj;			/* restore attributes */
  nformat=oldform; precision=oldprec;
    
  return;
  
  autofail:
  PSwarning("dXAxis","axis too short for automatic labeling");

  X1=identity(X0); X1.x=x1;
  dLine('W',X0,X1);

  if (2*curFSize*Y_stretch<deltaP) {		// label end points
    oldhadj=hadj; hadj='L';

    b0=b1=pWI(X0); b1.y+=xTiMaLe[0]; dLine('P',b0,b1);
    put_label(b0.x,b0.y,b1.x,b1.y,X0.x);

    b0=b1=pWI(X1); b1.y+=xTiMaLe[0]; dLine('P',b0,b1);
    put_label(b0.x,b0.y,b1.x,b1.y,X1.x);

    hadj=oldhadj;				/* restore attributes */
  } else {					// draw arrow point
    xe.dim=ye.dim=3; xe.x=1; xe.y=0; xe.z=0; ye.x=0; ye.y=1; ye.z=0;
    if ((x1-X0.x)*PWx*dotprod(R32_view,vecprod(xe,ye))>0) {
      oldfill=gFill();
      sFill();
      dSymbol('W',X0,1,0);
      sFill(gStroke());
      dSymbol('W',X1,0.3,0);
      sFill(oldfill);
    } else {
      dSymbol('W',X0,1,11);
      dSymbol('W',X1,1,0);
    }
  }

  return;
}
/*--------------------------------------------------------------------------------------
dXAxis: draw x-axis from x0 to x=x1, if (below) draw ticks below axis

HISTORY:
  v3.0.0									031115kr
--------------------------------------------------------------------------------------*/
void dXAxis(double x0,double y0,double x1,int below)
{
  struct dvec X0;
  
  X0.dim=2; X0.x=x0; X0.y=y0;
  dXAxis(X0,x1,below);

  return;
}
/*--------------------------------------------------------------------------------------
dXAxis: draw x-axis from x0 to x=x1, if (below) draw ticks below axis

HISTORY:
  v3.0.0									031115kr
--------------------------------------------------------------------------------------*/
void dXAxis(double x0,double y0,double z0,double x1,int below)
{
  struct dvec X0;
  
  X0.dim=3; X0.x=x0; X0.y=y0; X0.z=z0;
  dXAxis(X0,x1,below);

  return;
}


/*============================================================================ y-axis */
/*--------------------------------------------------------------------------------------
dYAxis: draw y-axis from x0 to y=y1, if (atleft) draw ticks left of axis

HISTORY:
  v3.0.0									031115kr
  change format of labels from f to g						940619kr
  v2.0										920524kr
  label end-points if axis is too short and issue warning			920205kr
--------------------------------------------------------------------------------------*/
void dYAxis(struct dvec X0,double y1,int atleft)
{
  char oldhadj,oldvadj,oldform;
  int nl[3],nh[3],n0,ndigit,prec,oldprec,dim,i,j;
  double dy[3],dy0,delta,deltaP=0,yl,yh,x;
  struct dvec X1,b0,b1,xe,ye;
  struct color oldfill;
  
  dim=X0.dim;

  if (z_set && dim!=3) {
    PSwarning("dYAxis","origin must be defined in three dimensions: operator ignored");
    return;
  }

  if (dim!=2 && dim!=3)
    PSerror("dYAxis","dimension different from 2 or 3");

  if (y1==X0.y) {
    PSwarning("dYAxis","length of axis is 0");
    return;
  }

  oldhadj=hadj; oldvadj=vadj;			/* save previous attributes */
  oldform=nformat; oldprec=precision;
  vadj='C';
  if (yauto) {
    delta=fabs(y1-X0.y);		       	/* estimate 0-order interval */
    b0=pWI(X0);
    X1=identity(X0); X1.y=y1;
    b1=pWI(X1);
    deltaP=sqrt(pow(b1.x-b0.x,2)+pow(b1.y-b0.y,2));
    if ((ndigit=(int)log10(delta/10))<0) ndigit--;
    if (ndigit>4 || ndigit<-4) {
      nformat='g'; prec=1;
    } else {
      //hj     nformat='f'; prec=(ndigit<0)?-ndigit-1:0;
      nformat='f'; prec=(ndigit<0)?-ndigit:0;
    }
    dy0=pow(10,ndigit);
    ndigit=(ndigit>0)?ndigit+1:-ndigit+2;
    for (;;) {					/* search through one decade */
      if ((n0=(int)(delta/dy0))==0) goto autofail;
      if (n0<10 && n0*curFSize*Y_stretch/deltaP<0.4) {dy[1]=dy0/2; break;}
      dy0*=2; if ((n0=(int)(delta/dy0))==0) goto autofail;
      if (n0<10 && n0*curFSize*Y_stretch/deltaP<0.4) {dy[1]=dy0/2; break;}
      dy0*=2.5; if ((n0=(int)(delta/dy0))==0) goto autofail;
      if (n0<10 && n0*curFSize*Y_stretch/deltaP<0.4) {dy[1]=dy0/5; break;}
      dy0*=2; ndigit=((ndigit=(int)(log10(dy0)))>0)?ndigit+1:-ndigit+2;
    }
    dy[0]=dy0; dy[2]=dy0/10.;
  }
  else {prec=yprecision; for (i=0;i<3;i++) dy[i]=yinterval[i];}

  yl=(X0.y<y1)?X0.y:y1; yh=(X0.y>y1)?X0.y:y1;
  if (dy[0]>0 && (fabs(yl/dy[0])>(double)INT_MAX || fabs(yh/dy[0])>(double)INT_MAX))
    goto autofail;
  for (i=0;i<3;i++)
    if (dy[i]>0) {
      nl[i]=(int)(yl/dy[i]); nh[i]=(int)(yh/dy[i]);
      if (yl>0) nl[i]++; if (yh<0) nh[i]--;
    }
  fprintf(PSout,"%%---------- dYAxis\n");
  b0.dim=dim; b0.x=X0.x; b0.y=X0.y; if (dim==3) b0.z=X0.z;
  b1.dim=dim; b1.x=X0.x; b1.y=y1;   if (dim==3) b1.z=X0.z;
  dLine('W',b0,b1);

  for (i=0;i<3;i++)				/* ticks without labeling */
    if (dy[i]>0) {
      x=(atleft)?-yTiMaLe[i]:yTiMaLe[i];
      fprintf(PSout,"%%   %i order tick marks\n",i);
      fprintf(PSout,"newpath\n");
      for (j=nl[i];j<=nh[i];j++) {
	b0.x=X0.x; b0.y=j*dy[i]; if (dim==3) b0.z=X0.z;
	movea('W',b0);
	b0.x=x;       b0.y=0;    if (dim==3) b0.z=0;
	liner('P',b0);
      }
      fprintf(PSout,"  closepath\n");
      fprintf(PSout,"stroke\n");
    }

  if (dy[0]>0 && prec>=0) {			/* labeling on 0-order ticks */
    precision=prec;
    x=(atleft)?-yTiMaLe[0]:yTiMaLe[0];
    for (j=nl[0]; j<=nh[0]; j++) {
      b1.x=X0.x; b1.y=j*dy[0];
      if (dim==3) {b1.z=X0.z; xe=pWI(b1);} else xe=pWP(b1);
      X1=pWP(b1); X1.x+=x; if (dim==3) ye=pPI(X1); else ye=identity(X1);
      put_label(xe.x,xe.y,ye.x,ye.y,j*dy[0]);
    }
  }
  hadj=oldhadj; vadj=oldvadj;			/* restore attributes */
  nformat=oldform; precision=oldprec;
    
  return;
  
  autofail:
    PSwarning("dYAxis","axis too short for automatic labeling");

  X1=identity(X0); X1.y=y1;
  dLine('W',X0,X1);

  if (2*curFSize*Y_stretch<deltaP) {		// label end points
    oldhadj=hadj; hadj='L';

    b0=b1=pWI(X0); b1.x+=yTiMaLe[0]; dLine('P',b0,b1);
    put_label(b0.x,b0.y,b1.x,b1.y,X0.y);

    b0=b1=pWI(X1); b1.x+=yTiMaLe[0]; dLine('P',b0,b1);
    put_label(b0.x,b0.y,b1.x,b1.y,X1.y);

    hadj=oldhadj;				/* restore attributes */
  } else {					// draw arrow point
    xe.dim=ye.dim=3; xe.x=1; xe.y=0; xe.z=0; ye.x=0; ye.y=1; ye.z=0;
    if ((y1-X0.y)*PWx*dotprod(R32_view,vecprod(xe,ye))>0) {
      oldfill=gFill();
      sFill();
      dSymbol('W',X0,1,0);
      sFill(gStroke());
      dSymbol('W',X1,0.3,0);
      sFill(oldfill);
    } else {
      dSymbol('W',X0,1,11);
      dSymbol('W',X1,1,0);
    }
  }

  return;
}
/*--------------------------------------------------------------------------------------
dYAxis: draw y-axis from x0 to y=y1, if (atleft) draw ticks at left of axis

HISTORY:
  v3.0.0									031115kr
--------------------------------------------------------------------------------------*/
void dYAxis(double x0,double y0,double y1,int atleft)
{
  struct dvec X0;
  
  X0.dim=2; X0.x=x0; X0.y=y0;
  dYAxis(X0,y1,atleft);

  return;
}
/*--------------------------------------------------------------------------------------
dYAxis: draw y-axis from x0 to y=y1, if (atleft) draw ticks at left of axis

HISTORY:
  v3.0.0									031115kr
--------------------------------------------------------------------------------------*/
void dYAxis(double x0,double y0,double z0,double y1,int atleft)
{
  struct dvec X0;
  
  X0.dim=3; X0.x=x0; X0.y=y0; X0.y=z0;
  dYAxis(X0,y1,atleft);

  return;
}


/*============================================================================ z-axis */
/*--------------------------------------------------------------------------------------
dZAxis: draw z-axis from x0 to z=z1, if (below) draw ticks below axis

HISTORY:
  v3.0.0									031115kr
--------------------------------------------------------------------------------------*/
void dZAxis(struct dvec X0,double z1,int below)
{
  char oldhadj,oldvadj,oldform;
  int prec,oldprec,nl[3],nh[3],n0,ndigit,dim,i,j;
  double dz[3],dz0,delta,deltaP=0,zl,zh,z;
  struct dvec X1,b0,b1,xe,ye;
  struct color oldfill;
  
  dim=X0.dim;
  if (dim!=3) {
    PSwarning("dZAxis","dimension must be 3 for z-axis to be defined: operator ignored");
    return;
  }

  if (z1==X0.z) {
    PSwarning("dZAxis","zero-length axis requested: operator ignored");
    return;
  }

  oldhadj=hadj; oldvadj=vadj;			/* save previous attributes */
  oldform=nformat; oldprec=precision;

  hadj='C';
  if (zauto) {
    delta=fabs(z1-X0.z);				/* estimate 0-order interval */
    b0=pWI(X0);
    X1=identity(X0); X1.z=z1;
    b1=pWI(X1);
    deltaP=sqrt(pow(b1.x-b0.x,2)+pow(b1.y-b0.y,2));
//     b0.dim=dim;
//     b0.x=0; b0.y=0; b0.z=0; b1=pWI(b0); P0=sqrt(pow(b1.x,2)+pow(b1.y,2));
//     b0.z=delta;     	    b0=pWI(b0); deltaP=sqrt(pow(b1.x-b0.x,2)+pow(b1.y-b0.y,2));
    if ((ndigit=(int)(log10(0.1*delta)))<0) ndigit--;
    if (ndigit>4 || ndigit<-4) {
      nformat='g'; prec=1;
    } else {
      nformat='f'; prec=(ndigit<0)?-ndigit-1:0;
    }
    dz0=pow(10,ndigit);
    ndigit=(ndigit>0)?ndigit+1:-ndigit+2;
    for (;;) {					/* search through one decade */
      if ((n0=(int)(delta/dz0))==0) goto autofail;
      if (n0<10 && n0*ndigit*curFSize*X_stretch/deltaP<0.3) {dz[1]=dz0/2; break;}
      dz0 *= 2; if ((n0=(int)(delta/dz0))==0) goto autofail;
      if (n0<10 && n0*ndigit*curFSize*X_stretch/deltaP<0.3) {dz[1]=dz0/2; break;}
      dz0 *= 2.5; if ((n0=(int)(delta/dz0))==0) goto autofail;
      if (n0<10 && n0*ndigit*curFSize*X_stretch/deltaP<0.3) {dz[1]=dz0/5; break;}
      dz0 *= 2; ndigit=((ndigit=(int)(log10(dz0)))>0)?ndigit+1:-ndigit+2;
    }
    dz[0]=dz0; dz[2]=0.1*dz0;
  }
  else {prec=zprecision;for (i=0; i<3; i++) dz[i]=zinterval[i];}
  
  zl=(X0.z<z1)?X0.z:z1; zh=(X0.z>z1)?X0.z:z1;
  if (dz[0]>0 && (fabs(zl/dz[0])>(double)INT_MAX || fabs(zh/dz[0])>(double)INT_MAX))
    goto autofail;
  for (i=0; i<3; i++) 
    if (dz[i]>0) {nl[i]=(int)(zl/dz[i]); nh[i]=(int)(zh/dz[i]);
    if (zl>0) nl[i]++; if (zh<0) nh[i]--;}

  fprintf(PSout,"%%---------- dZAxis\n");
  b0.dim=dim; b0.x=X0.x; b0.y=X0.y; b0.z=X0.z;
  b1.dim=dim; b1.x=X0.x; b1.y=X0.y; b1.z=z1;
  dLine('W',b0,b1);

  for (i=0; i<3; i++)				/* ticks without labeling */
    if (dz[i]>0) {
      z=(below)?-zTiMaLe[i]:zTiMaLe[i];
      fprintf(PSout,"%%   %i order tick marks\n",i);
      fprintf(PSout,"newpath\n");
      for (j=nl[i]; j <= nh[i]; j++) {
	b0.x=X0.x; b0.y=X0.y; b0.z=j*dz[i];
	movea('W',b0);
	b0.x=0;       b0.y=z;    b0.z=0;
	liner('P',b0);
      }
      fprintf(PSout,"  closepath\n");
      fprintf(PSout,"stroke\n");
    }

  if (dz[0]>0 && prec>=0) {			/* labeling 0-order ticks */
    precision=prec;
    z=(below)?-zTiMaLe[0]:zTiMaLe[0];
    for (j=nl[0]; j<=nh[0]; j++) {
      b1.x=X0.x; b1.y=X0.y; b1.z=j*dz[0]; xe=pWI(b1); X1=pWP(b1);
      X1.y+=z; ye=pPI(X1);
      put_label(xe.x,xe.y,ye.x,ye.y,j*dz[0]);
    }
  }
  hadj=oldhadj; vadj=oldvadj;			/* restore attributes */
  nformat=oldform; precision=oldprec;
    
  return;
  

  autofail:
  PSwarning("dZAxis","axis too short for automatic labeling");

  X1=identity(X0); X1.z=z1;
  dLine('W',X0,X1);

  if (2*curFSize*Y_stretch<deltaP) {		// label end points
    oldhadj=hadj; hadj='L';

    b0=b1=pWI(X0); b1.x+=zTiMaLe[0]; dLine('P',b0,b1);
    put_label(b0.x,b0.y,b1.x,b1.y,X0.z);

    b0=b1=pWI(X1); b1.x+=zTiMaLe[0]; dLine('P',b0,b1);
    put_label(b0.x,b0.y,b1.x,b1.y,X1.z);

    hadj=oldhadj;				/* restore attributes */
  } else {					// draw arrow point
    xe.dim=ye.dim=3; xe.x=1; xe.y=0; xe.z=0; ye.x=0; ye.y=1; ye.z=0;
    if ((z1-X0.z)*PWz*dotprod(R32_view,vecprod(xe,ye))>0) {
      oldfill=gFill();
      sFill();
      dSymbol('W',X0,1,0);
      sFill(gStroke());
      dSymbol('W',X1,0.3,0);
      sFill(oldfill);
    } else {
      dSymbol('W',X0,1,11);
      dSymbol('W',X1,1,0);
    }
  }

  return;
}
/*--------------------------------------------------------------------------------------
dZAxis: draw z-axis from x0 to z=z1, if (below) draw ticks below axis

HISTORY:
  v3.0.0									031115kr
--------------------------------------------------------------------------------------*/
void dZAxis(double x0,double y0,double z1,int below)
{
  struct dvec X0;
  
  X0.dim=2; X0.x=x0; X0.y=y0;
  dZAxis(X0,z1,below);

  return;
}
/*--------------------------------------------------------------------------------------
dZAxis: draw y-axis from x0 to z=z1, if (below) draw ticks below axis

HISTORY:
  v3.0.0									031115kr
--------------------------------------------------------------------------------------*/
void dZAxis(double x0,double y0,double z0,double z1,int below)
{
  struct dvec X0;
  
  X0.dim=3; X0.x=x0; X0.y=y0; X0.y=z0;
  dZAxis(X0,z1,below);

  return;
}
/*========================================================================= auxiliary */
/*--------------------------------------------------------------------------------------
put_label: 

HISTORY:
  v3.0.0									031206kr
--------------------------------------------------------------------------------------*/
void put_label(double T0x,double T0y,double T1x,double T1y,double thenumber)
{
  int exponent,positive,prec;
  double L,H,d,dTx,dTy,phi,num,buf;
  char thisformat,cbuf[64];
  struct dvec T0,T1;

  dTx=T1x-T0x; dTy=T1y-T0y;
  if (100*fabs(dTx)<fabs(dTy)) {			// vertical tick
    hadj='C'; vadj=(dTy>0)?'B':'T';
    movea('P',T0x,T0y+1.5*dTy);
    dNumber(thenumber);
    return;
  }
  if (100*fabs(dTy)<fabs(dTx)) {			// horizontal tick
    hadj=(dTx>0)?'L':'R'; vadj='C';
    movea('P',T0x+1.5*dTx,T0y);
    dNumber(thenumber);
    return;
  }

  if (thenumber==0) {					// tilted tick
    sprintf(cbuf,"0");
  } else {
    prec=(precision>0)?precision:-precision;
    if (!(positive=thenumber>0)) thenumber=-thenumber;
    buf=log10(thenumber);
    if (nformat=='g')
      thisformat=(fabs(buf)>prec+1)?'e':'f';
    else
      thisformat=nformat;
    if (thisformat=='f') {
      if (positive) sprintf(cbuf,"%1.*f",prec,thenumber);
      else {
	if (TeXfonts==1)
	  sprintf(cbuf,"$-%1.*f$",prec,thenumber);
	else
	  sprintf(cbuf,"-%1.*f",prec,thenumber);
      }
    } else {
      exponent=(int)buf; if (buf<0) exponent--;
      num=((positive)?1:-1)*thenumber/pow(10,exponent);
      buf=num*pow(10,prec);
      if (pow(10,prec+1)-buf<0.5) {
	exponent++; num/=10;
      }
      if (TeXfonts==1) {
	sprintf(cbuf,"{$%1.*f\\cdot10^{%1i}$}",prec,num,exponent);
      } else {
	sprintf(cbuf,"%1.*fe%+1i",prec,num,exponent);
      }
    }
  }


  hadj='C'; vadj='C';
  phi=atan2(dTy,dTx);
  L=strlen(cbuf)*curFSize*X_stretch; H=curFSize*Y_stretch; d=zTiMaLe[0]/2;
  T0.dim=T1.dim=2; T0.x=T0x; T0.y=T0y; T1.x=T1x; T1.y=T1y;
  if (L*fabs(dTy)>H*fabs(dTx)) {		       	// region I
    d=1.5+fabs(H/(2*sin(phi)*zTiMaLe[0]));
    T1=addvec(T0,scalmult(d,subvec(T1,T0)));
    movea('P',T1);
  } else {		       				// region II
    d=1.5+fabs(L/(2*cos(phi)*zTiMaLe[0]));
    T1=addvec(T0,scalmult(d,subvec(T1,T0)));
    movea('P',T1);
  }
  dText(cbuf);

  
  return;
}
