#undef COM_SYM_DEF
#include "PSgraf_p.h"
/*--------------------------------------------------------------------------------------
dBitMap: Draw bitmap in PaperCoordinates with lower left corner (x0,y0) and upper right
	corner (x0+dpx,y0+dpy). The source of the bitmap contains Nx data points in
	x-direction and Ny in y-direction which are stored in c0 for gray scale images
	and in c0, c1, c2, [c3] for color images. First value is at lower left corner
	successive values proceed in x-direction. Values must be in the range 0...1. They
	are transformed to 8 bit values leading to a 24 bit rendering of the images.

HISTORY:
  v3.0.0									031221kr
  check if data correspond to ColorSpace					021009kr
  created	       								971121kr
--------------------------------------------------------------------------------------*/
void d2hex(char* hex,double d);
void hsb2rgb(double *r,double *g,double *b,double *hue,double *sat,double *bri,int N);
void dBitMap(double x0,double y0,double dpx,double dpy,int Nx,int Ny,
	     double *gry)
{
  int i,flag=0,N=Nx*Ny,ic=0;
  char g[3],cbuf[127];

  if (gry==NULL) PSerror("dBitMap","NULL vector for gray component");
  for (i=0;i<N;i++) {
    if (gry[i]<0) {flag++; gry[i]=0;}
    if (gry[i]>1) {flag++; gry[i]=1;}
  }
  if (flag>0) {
    if (flag==1) {
      sprintf(cbuf,"1 value out of range [0...1]; has been set to limit");
    } else {
      sprintf(cbuf,"%i values out of range [0...1]; have been set to limit",flag);
    }
    PSwarning("dBitMap (G)",cbuf);
  }  
  	  
  fprintf(PSout, "%%-- dBitMap (G)\n");				// draw bitmap
  fprintf(PSout,"gsave\n");
  fprintf(PSout,"/DeviceGray setcolorspace\n");
  fprintf(PSout,"%f %f translate\n%f %f scale\n",x0,y0,dpx,dpy);
  fprintf(PSout,"<<\n");
  fprintf(PSout,"  /ImageType 1\n");
  fprintf(PSout,"  /Width %i /Height %i\n",Nx,Ny);
  fprintf(PSout,"  /BitsPerComponent 8\n");
  fprintf(PSout,"  /Decode [0 1]\n");
  fprintf(PSout,"  /ImageMatrix [%i 0 0 %i 0 0]\n",Nx,Ny);
  fprintf(PSout,"  /DataSource currentfile /ASCIIHexDecode filter\n");
  fprintf(PSout,"  /Interpolate true\n");
  fprintf(PSout,">>\n");
  fprintf(PSout,"image\n");
  for (i=0;i<N;i++) {
    d2hex(g,gry[i]);
    ic+=2; if (ic>80) {fprintf(PSout,"\n"); ic=2;}
    fprintf(PSout,"%2s",g);
  }
  fprintf(PSout,">");

  fprintf(PSout,"\ngrestore\n");
  updateBB(x0,y0);
  updateBB(x0+dpx,y0+dpy);

return;
}
/*------------------------------------------------------------------------------------*/
void dBitMap(double x0,double y0,double dpx,double dpy,int Nx,int Ny,
	     double *c0,double *c1,double *c2)
{
  int i,flag=0,N=Nx*Ny,ic=0;
  char r[3],g[3],b[3],cbuf[127];
  double *d0,*d1,*d2;

  if (ColorSpace!=2 && ColorSpace!=4) {
    PSwarning("dBitMap(double,double,double)",
	      "color space not RGB or HSB: operator ignored");
    return;
  }

  for (i=0;i<N;i++) {
    if (c0[i]<0) {flag++; c0[i]=0;}
    if (c0[i]>1) {flag++; c0[i]=1;}
    if (c1[i]<0) {flag++; c1[i]=0;}
    if (c1[i]>1) {flag++; c1[i]=1;}
    if (c2[i]<0) {flag++; c2[i]=0;}
    if (c2[i]>1) {flag++; c2[i]=1;}
  }
  if (flag>0) {
    if (flag==1)
      sprintf(cbuf,"1 value out of range [0...1]; has been set to limit");
    else
      sprintf(cbuf,"%i values out of range [0...1]; have been set to limit",flag);
    PSwarning("dBitMap",cbuf);
  }


  if (ColorSpace==2) {						// RGB
    if (c0==NULL) PSerror("dBitMap","NULL vector for R-component");
    if (c1==NULL) PSerror("dBitMap","NULL vector for G-component");
    if (c2==NULL) PSerror("dBitMap","NULL vector for B-component");

    d0=c0; d1=c1; d2=c2;
  } else {							// HSB
    if (c0==NULL) PSerror("dBitMap","NULL vector for H-component");
    if (c1==NULL) PSerror("dBitMap","NULL vector for S-component");
    if (c2==NULL) PSerror("dBitMap","NULL vector for B-component");

    if ((d0=(double*)malloc(N*sizeof(double)))==NULL)
      PSerror("dBitMap","memory overflow");
    if ((d1=(double*)malloc(N*sizeof(double)))==NULL)
      PSerror("dBitMap","memory overflow");
    if ((d2=(double*)malloc(N*sizeof(double)))==NULL)
      PSerror("dBitMap","memory overflow");
    hsb2rgb(d0,d1,d2,c0,c1,c2,N);
  }
  	  
  fprintf(PSout, "%%-- dBitMap \n");				// draw bitmap
  fprintf(PSout,"gsave\n");
  fprintf(PSout,"/DeviceRGB setcolorspace\n");
  fprintf(PSout,"%f %f translate\n%f %f scale\n",x0,y0,dpx,dpy);
  fprintf(PSout,"<<\n");
  fprintf(PSout,"  /ImageType 1\n");
  fprintf(PSout,"  /Width %i /Height %i\n",Nx,Ny);
  fprintf(PSout,"  /BitsPerComponent 8\n");
  fprintf(PSout,"  /Decode [0 1 0 1 0 1]\n");
  fprintf(PSout,"  /ImageMatrix [%i 0 0 %i 0 0]\n",Nx,Ny);
  fprintf(PSout,"  /DataSource currentfile /ASCIIHexDecode filter\n");
  fprintf(PSout,"  /Interpolate true\n");
  fprintf(PSout,">>\n");
  fprintf(PSout,"image\n");
  for (i=0;i<N;i++) {
    d2hex(r,d0[i]); d2hex(g,d1[i]); d2hex(b,d2[i]);
    ic+=6; if (ic>80) {fprintf(PSout,"\n"); ic=6;}
    fprintf(PSout,"%2s%2s%2s",r,g,b);
  }
  fprintf(PSout,">");
  fprintf(PSout,"\ngrestore\n");
  updateBB(x0,y0);
  updateBB(x0+dpx,y0+dpy);
  
  if (ColorSpace==4) {free((void*)d0); free((void*)d1); free((void*)d2);}

return;
}
/*------------------------------------------------------------------------------------*/
void dBitMap(double x0,double y0,double dpx,double dpy,int Nx,int Ny,
	     double *c0,double *c1,double *c2,double *c3)
{
  int i,flag=0,N=Nx*Ny,ic=0;
  char c[3],m[3],y[3],k[3],cbuf[127];

  if (c0==NULL) PSerror("dBitMap","NULL vector for C-component");
  if (c1==NULL) PSerror("dBitMap","NULL vector for M-component");
  if (c2==NULL) PSerror("dBitMap","NULL vector for Y-component");
  if (c3==NULL) PSerror("dBitMap","NULL vector for K-component");

  for (i=0;i<N;i++) {
    if (c0[i]<0) {flag++; c0[i]=0;}
    if (c0[i]>1) {flag++; c0[i]=1;}
    if (c1[i]<0) {flag++; c1[i]=0;}
    if (c1[i]>1) {flag++; c1[i]=1;}
    if (c2[i]<0) {flag++; c2[i]=0;}
    if (c2[i]>1) {flag++; c2[i]=1;}
    if (c3[i]<0) {flag++; c3[i]=0;}
    if (c3[i]>1) {flag++; c3[i]=1;}
  }
  if (flag>0) {
    if (flag==1)
      sprintf(cbuf,"1 value out of range [0...1]; has been set to limit");
    else
      sprintf(cbuf,"%i values out of range [0...1]; have been set to limit",flag);
    PSwarning("dBitMap",cbuf);
  }  
  	  
  fprintf(PSout, "%%-- dBitMap \n");				// draw bitmap
  fprintf(PSout,"gsave\n");
  fprintf(PSout,"/DeviceCMYK setcolorspace\n");
  fprintf(PSout,"%f %f translate\n%f %f scale\n",x0,y0,dpx,dpy);
  fprintf(PSout,"<<\n");
  fprintf(PSout,"  /ImageType 1\n");
  fprintf(PSout,"  /Width %i /Height %i\n",Nx,Ny);
  fprintf(PSout,"  /BitsPerComponent 8\n");
  fprintf(PSout,"  /Decode [0 1 0 1 0 1 0 1]\n");
  fprintf(PSout,"  /ImageMatrix [%i 0 0 %i 0 0]\n",Nx,Ny);
  fprintf(PSout,"  /DataSource currentfile /ASCIIHexDecode filter\n");
  fprintf(PSout,"  /Interpolate true\n");
  fprintf(PSout,">>\n");
  fprintf(PSout,"image\n");
  for (i=0;i<N;i++) {
    d2hex(c,c0[i]); d2hex(m,c1[i]); d2hex(y,c2[i]); d2hex(k,c3[i]);
    ic+=8; if (ic>80) {fprintf(PSout,"\n"); ic=8;}
    fprintf(PSout,"%2s%2s%2s%2s",c,m,y,k);
  }
  fprintf(PSout,">");
  fprintf(PSout,"\ngrestore\n");
  updateBB(x0,y0);
  updateBB(x0+dpx,y0+dpy);

return;
}
/*--------------------------------------------------------------------------------------
d2hex:	transform double 0...1 to integer 0...255 represented as hexadecimal 00...ff;
	Notice: range of d is not check in this function; hex must be length 3 or more
--------------------------------------------------------------------------------------*/
void d2hex(char* hex,double d)
{
  int i,a,b;

  i=(int)(256*d); if (i==256) i--;
  a=(int)(i/16); b=i-16*a;
  if (a>9) a+='0'+'a'-'9'-1; else a+='0';
  if (b>9) b+='0'+'a'-'9'-1; else b+='0';
  sprintf(hex,"%c%c",a,b);

  return;
}
/*--------------------------------------------------------------------------------------
hsb2rgb: transform HSB to RGB
--------------------------------------------------------------------------------------*/
void hsb2rgb(double *r,double *g,double *b,double *hue,double *sat,double *bri,int N)
{
  int i,cl,flag=0;
  char cbuf[127];
  double h0,r0,g0,b0,buf;

  r0=g0=b0=0;

  for (i=0;i<N;i++) {				/* check input */
    if (hue[i]<0) {flag++; hue[i]=0;}
    if (hue[i]>1) {flag++; hue[i]=1;}
    if (sat[i]<0) {flag++; sat[i]=0;}
    if (sat[i]>1) {flag++; sat[i]=1;}
    if (bri[i]<0) {flag++; bri[i]=0;}
    if (bri[i]>1) {flag++; bri[i]=1;}
  }
  if (flag>0) {
    if (flag==1) {
      sprintf(cbuf,"1 HSB-value out of range [0...1]; has been set to limit");
    } else {
      sprintf(cbuf,"%i HSB-values out of range [0...1]; have been set to limit",flag);
    }
    PSwarning("dBitMap:hsb2rgb",cbuf);
  }
  
  for (i=0;i<N;i++) {				/* transform */
    h0=modf(6*hue[i],&buf); cl=(int)buf;
    switch (cl) {
      case 0: case 6:
	r0=1; g0=h0; b0=0;
	break;
      case 1:
	r0=1-h0; g0=1; b0=0;
	break;
      case 2:
	r0=0; g0=1; b0=h0;
	break;
      case 3:
	r0=0; g0=1-h0; b0=1;
	break;
      case 4:
	r0=h0; g0=0; b0=1;
	break;
      case 5:
	r0=1; g0=0; b0=1-h0;
	break;
    }
    r[i]=bri[i]*(1-sat[i]*(1-r0));
    g[i]=bri[i]*(1-sat[i]*(1-g0));
    b[i]=bri[i]*(1-sat[i]*(1-b0));
  }

  return;
}
