#undef COM_SYM_DEF
#include "PSgraf_p.h"
/*--------------------------------------------------------------------------------------
dContours: draw contourlines

HISTORY:
  v3.0.0 operate in projection plane (Paper Coordinates)			031115kr
  distinguish grid types							001217kr
  minimal pixel size								940526kr
  remove sorting of ctr (required to be given sorted)				920821kr
  v2.0										920524kr
  replaced fwhite by fback							920203kr
--------------------------------------------------------------------------------------*/
int gCrossPoints(double *Z,double ctr);
void get_p(struct line *begin,double *Z,int iCTR,int no_frame_onB_);
struct line *follow_bdr(double *Z,int iCTR,struct line *l);
void new_p(int no_frame_onB_);
int put_on_p(double x,double y,struct line *l,int on_boundary);
void deleteP_(void);
void remove_holes(double *Z);
void fback(double *Z);
int cmp_p(int *i1,int *i2);
void gSpecPoly(double *Z,int iB,int iN,int no_frame_onB_);
/*------------------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------------------
dContours: draw contourlines

HISTORY:
  v3.0.0 operate in projection plane (Paper Coordinates)			031115kr
  distinguish grid types							001217kr
  minimal pixel size								940526kr
  remove sorting of ctr (required to be given sorted)				920821kr
  v2.0										920524kr
  replaced fwhite by fback							920203kr
--------------------------------------------------------------------------------------*/
int gCrossPoints(double *Z,double ctr);
void get_p(struct line *begin,double *Z,int iCTR,int no_frame_onB_);
struct line *follow_bdr(double *Z,int iCTR,struct line *l);
void new_p(int no_frame_onB_);
int put_on_p(double x,double y,struct line *l,int on_boundary);
void deleteP_(void);
void remove_holes(double *Z);
void fback(double *Z);
int cmp_p(int *i1,int *i2);
void gSpecPoly(double *Z,int iB,int iN,int no_frame_onB_);
/*------------------------------------------------------------------------------------*/
void dContours(double *Z,int filled,int framed)
{
  int ic,iL,ip,bsplit,esplit,*idx;
  double oldPthick=Pthick;
  struct color oldfill;

  oldfill=gFill();
  
  if (L_==NULL) {				/* needed structures available? */
    PSwarning("dContours","no grid defined; operator ignored");
    return;}
  if (B_==NULL) {
    PSwarning("dContours","no boundary for grid defined; operator ignored");
    return;}
  if (N_Ctr<0) {
    PSwarning("dContours","no contourlevels defined; operator ignored");
    return;}
    
  fprintf(PSout,"%%-----dContours\n");		/* begin dContours */
  
  NP_=0;					/* alloc 1. chunk of memory for poly */
  P_=(struct polygon *) malloc(CHUNK*sizeof(struct polygon));
  if (P_==NULL) PSerror("dContours","memory overflow (0)");
  						/* initialize state of boundary */
  for (iL=0;iL<NB_;iL++) {B_[iL].state[0]=0; B_[iL].state[1]=0;}
    
  for (ic=0;ic<N_Ctr;ic++) {			/* get contour polygons for ctr[ic] */
    iL=0;
    if (gCrossPoints(Z,C_ctr[ic])>0)		/* get crossing points */
      while (iL<NL_) {				/* get next polygon */
        while (iL<NL_ && L_[iL++].state<1) ;	/* find next starting point */
	if (iL<NL_)				/* get polygon starting on line iL */
	  get_p(&L_[--iL],Z,ic,framed==-1);
      }
  }
	  
  if (filled) {					/* draw filled (& framed?) polygons */
    if (framed<1) Pthick=0;
    remove_holes(Z);
    iL=-1;					/* get special polygons */
    while (++iL<NB_) {
      if (!B_[iL].state[0] && Z[B_[iL].l->n[0]]>C_ctr[0]) gSpecPoly(Z,iL,0,framed==-1);
      if (!B_[iL].state[1] && Z[B_[iL].l->n[1]]>C_ctr[0]) gSpecPoly(Z,iL,1,framed==-1);
    }
    idx=(int*) malloc(NP_*sizeof(int));		/* allocate & initialize index array */
    if (idx==NULL) PSerror("dContours","memory overflow (1)");
    for (ip=0;ip<NP_;ip++) idx[ip]=ip;
    qsort((void*) idx,NP_,sizeof(int),(int (*)(const void*,const void*))cmp_p);
    fback(Z);					/* fill background */
    for (ip=0;ip<NP_;ip++) {
      sFill(C_col[P_[idx[ip]].color]);
      dP2P(P_[idx[ip]].x,P_[idx[ip]].y,P_[idx[ip]].N);
    }
    free((void*) idx);
  }
  
  Pthick=oldPthick;				/* restore pen thickness */

  if (!filled && framed==1) {			/* normal,unfilled frames */
    F_c0=F_c0=NONE;
    for (ip=0;ip<NP_;ip++) dP2P(P_[ip].x,P_[ip].y,P_[ip].N);
  }

  if (framed==-1) {				/* not framed along grid boundary */
    F_c0=F_c0=NONE;
    for (ip=0;ip<NP_;ip++) {
      iL=0;
      while (iL<P_[ip].N) {
	while (iL<P_[ip].N && P_[ip].boundary[iL]) iL++;	/* find begin split */
	if (iL<P_[ip].N) {
	  bsplit=iL;
	  while (iL<P_[ip].N && !P_[ip].boundary[iL]) iL++;	/* find end split */
	  if ((esplit=iL)<P_[ip].N) {
	    if (esplit-bsplit>1)
	      dP2P(P_[ip].x+bsplit,P_[ip].y+bsplit,esplit-bsplit);
	  }
	  else {dP2P(P_[ip].x+bsplit,P_[ip].y+bsplit,P_[ip].N-bsplit);}
	}
      }
    }
  }
    
  deleteP_();
  sFill(oldfill);
  
  if (below_Pix>0) {
    fprintf(stderr,
            "\n? PSgraf warning\n\tin dContours: %i polygons < minimal pixel size\n\n",
            below_Pix);
    below_Pix=0;
  }

  return;
}
/*--------------------------------------------------------------------------------------
gCrossPoints: calculate crossing points between contour line z=CTR and gridlines;
              set L_[].state and return number of crossings
--------------------------------------------------------------------------------------*/
int gCrossPoints(double Z[],double CTR)
{
  int iCP=0,i;
  double z0,z1,dz;
  
  for (i=0;i<NL_;i++) {
    z0=Z[L_[i].n[0]]; z1=Z[L_[i].n[1]];
    dz=CTR-z0;
    if (dz*(CTR-z1)<0) {			/* "normal" CP on line */
      L_[i].xCP=L_[i].bx+(L_[i].ex-L_[i].bx)*dz/(z1-z0);
      L_[i].yCP=L_[i].by+(L_[i].ey-L_[i].by)*dz/(z1-z0);
      L_[i].state=1;
      iCP++;}
    else
      if (z0==CTR && z1>CTR) {
	L_[i].xCP=L_[i].bx+DBL_EPSILON*(L_[i].ex-L_[i].bx);
	L_[i].yCP=L_[i].by+DBL_EPSILON*(L_[i].ey-L_[i].by);
	L_[i].state=1; iCP++;}
      else if (z1==CTR && z0>CTR) {
	L_[i].xCP=L_[i].ex-DBL_EPSILON*(L_[i].ex-L_[i].bx);
	L_[i].yCP=L_[i].ey-DBL_EPSILON*(L_[i].ey-L_[i].by);
	L_[i].state=1; iCP++;}
      else					/* no CP on line */
	L_[i].state=0;
  }
  
  return iCP;
}
/*--------------------------------------------------------------------------------------
get_p: get polygon starting at line *begin and belonging to contour line #iCTR;
--------------------------------------------------------------------------------------*/
void get_p(struct line *begin,double Z[],int iCTR,int no_frame_onB_)
{
  int nu,nl,n0,off,leaving=0,i;
  double CTR=C_ctr[iCTR],x,y,z0,bxp,byp;
  struct line *l,*cp[2];
  
  first_on_p=l=begin;
  new_p(no_frame_onB_);				/* new,empty polygon */
  P_[NP_].ictr=P_[NP_].color=iCTR;
  
  if (l->boundary && !l->state) { 		/* special polygon: first follow bdr */
    if ((l=follow_bdr(Z,iCTR,l))==NULL) return;
    leaving=1;
  }

  x=l->xCP; y=l->yCP; l->state=-1;
  
  while (put_on_p(x,y,l,0)!=0) {		/* follow CPs until polygon is closed */
    if (l->boundary && !leaving) {
      if ((l=follow_bdr(Z,iCTR,l))==NULL)
        return;
      else {
        l->state=-1;
        if (put_on_p(l->xCP,y=l->yCP,l,0)==0) return;
      }
    }
    leaving=0;

    switch (GT_) {
    case 0:				/*----- topologically regular quadrilateral */
      nu=0; nl=0;
      for (i=0;i<3;i++) {		       	/* count unused CPs */
	if (l->l[i]!=NULL && l->l[i]->state>0) nu++;
	if (l->l[i+3]!=NULL && l->l[i+3]->state>0) nl++;}
    
      if ((nu+1)%2==0) off=0; else {off=3; nu=nl;}
      
      switch (nu) {
      case 1:
        for (i=off;i<3+off;i++)
	  if (l->l[i]!=NULL && l->l[i]->state>0) {
	    l=l->l[i]; x=l->xCP; y=l->yCP; l->state=-1;
	    break;}
        break;
      case 3:
	n0=1; z0=Z[l->n[0]];
	for (i=1+off;i<3+off;i++) 
	  if (l->l[i]!=NULL){
	    n0++;
	    z0+=Z[l->l[i]->n[0]];}
	z0/=n0;
	if ((Z[l->n[0]]-CTR)*(z0-CTR)>0) l=l->l[off]; else l=l->l[2+off];
	x=l->xCP; y=l->yCP; l->state=-1;
        break;
      default:				       	/* polygon needs to be closed */
        x=P_[NP_].x[0]; y=P_[NP_].y[0]; l=first_on_p;
	break;
      }
      break;


    case 1:				/*----- arbitrary triangular */
      nu=0;
      for (i=0;i<4;i++)     		       	/* find & count unused CPs */
	if (l->l[i]!=NULL && l->l[i]->state>0) {
	  cp[nu]=l->l[i]; nu++;
	  if (nu>2) {
	    PSwarning("dContours",
		      "triangular grid with more than three CPs; polygon closed anyway");
	    nu=0;
	    break;
	  }
	}
      if (nu==0) {			       	/* close polygon */
	x=P_[NP_].x[0]; y=P_[NP_].y[0]; l=first_on_p;}
      else {		       			/* follow polygon */
	l=cp[0]; x=l->xCP; y=l->yCP; l->state=-1;}
      break;


    default:
      PSerror("dContours","undefined grid type");
      break;
    }
  }
						/* poly>Min_Pix? (NP_ already updated) */
  bxp=P_[NP_-1].bxur-P_[NP_-1].bxll; byp=P_[NP_-1].byur-P_[NP_-1].byll;
  if (fabs(bxp)<Min_Pix || fabs(byp)<Min_Pix) {
    NP_--;
    below_Pix++;
    free((void*)P_[NP_].x); free((void*)P_[NP_].y);
    if (P_[NP_].boundary!=NULL) free((void*)P_[NP_].boundary);
  }
  return;
}
/*--------------------------------------------------------------------------------------
follow_bdr: follow boundary to next crossing point through region where Z>C_ctr[iCTR];
            entering: - CP on *l (where path touches boundary) is already on path
	    leaving:  - CP on *l (where path leaves boundary) is not yet on path
		      - return *l or NULL if polygon is closed
--------------------------------------------------------------------------------------*/
struct line *follow_bdr(double Z[],int iCTR,struct line *l)
{
  int dir,ib=0;					/* path to follow boundary */
  double CTR=C_ctr[iCTR],x,y;
  struct bline *bl;
  
  P_[NP_].color=iCTR; P_[NP_].testL=NULL;	/* fixed gray level, not hole */
  bl=B_;					/* beginning of boundary */
  while (bl->l!=l && bl-B_<NB_) bl++;		/* find iL on boundary */
  if (bl-B_==NB_) 
    PSerror("dContours\n\t\tin follow_bdr","could not find point on boundary");
    
  if (Z[l->n[0]]>CTR) {				/* next point to move to */
    x=l->bx; y=l->by;
    bl->state[0]=1;
  }
  else {
    x=l->ex; y=l->ey;
    bl->state[1]=1;
  }
  put_on_p(x,y,l,1);				/* put point on polygon path */
    
  bl=bl->path[0];				/* determine dir. to follow boundary */
  if ((bl->l->bx==x && bl->l->by==y)  ||
      (bl->l->ex==x && bl->l->ey==y)) {		/* forward */
    dir=0;}
  else {
    dir=1;					/* reverse */
    bl=bl->path[1]->path[1];}
    
  while (bl->l->state<1) {			/* put boundary points on path */
    bl->state[0]=1; bl->state[1]=1;
    if (bl->l->bx==x && bl->l->by==y) {
      x=bl->l->ex; y=bl->l->ey;
    } 
    else {
      x=bl->l->bx; y=bl->l->by;
    }
    if (put_on_p(x,y,l,1)==0) return NULL;	/* boundary closed */
    if (ib++>NB_) PSerror("dContours\n\t\tin follow_bdr","trapped on boundary");
    bl=bl->path[dir];
  }

  return bl->l;
}
/*--------------------------------------------------------------------------------------
new_p: open new polygon _p
--------------------------------------------------------------------------------------*/
void new_p(int no_frame_onB_)
{
  if ((NP_+1)%CHUNK==0)				/* more memory for polygon pointers */
    if ((P_=(struct polygon *) 
      realloc((void*) P_,(NP_/CHUNK+2)*CHUNK*sizeof(struct polygon)))==NULL)
	PSerror("dContours\n\t\tin new_p","memory overflow (0)");
    
  		  				/* initialize new polygon */
  if ((P_[NP_].x=(double*) malloc(CHUNK*sizeof(double)))==NULL)
    PSerror("dContours\n\t\tin new_p","memory overflow (1)");
  if ((P_[NP_].y=(double*) malloc(CHUNK*sizeof(double)))==NULL)
    PSerror("dContours\n\t\tin new_p","memory overflow (2)");
  if (no_frame_onB_) {
    if ((P_[NP_].boundary=(int*) malloc(CHUNK*sizeof(int)))==NULL)
      PSerror("dContours\n\t\tin new_p","memory overflow (3)");
  }
  else
    P_[NP_].boundary=NULL;
    
  P_[NP_].N=0;
  P_[NP_].bxll=10000.; P_[NP_].bxur=-10000.;
  P_[NP_].byll=10000.; P_[NP_].byur=-10000.;
  P_[NP_].testL=L_-1;				/* set testL to some value!=NULL */
  
  return;
}
/*--------------------------------------------------------------------------------------
put_on_p: put point (x,y) on current polygon; 
	  return the number of points in the path or 0 if the polygon has been closed,
	  i.e. if (x,y) is identical with the first point on the path;
--------------------------------------------------------------------------------------*/
int put_on_p(double x,double y,struct line *l,int on_boundary)
{
  int N=P_[NP_].N;

  P_[NP_].x[N]=x; P_[NP_].y[N]=y;		/* put point on path */
  if (P_[NP_].boundary!=NULL) P_[NP_].boundary[N]=on_boundary;
  N=++P_[NP_].N;				/* update point counter */
  
  						/* close polygon? */
  if (N>1 && P_[NP_].x[0]==x && P_[NP_].y[0]==y && l==first_on_p) {
    P_[NP_].x=(double*) realloc((void*) P_[NP_].x,N*sizeof(double));
    P_[NP_].y=(double*) realloc((void*) P_[NP_].y,N*sizeof(double));
    if (P_[NP_].boundary!=NULL)
      P_[NP_].boundary=(int*) realloc((void*) P_[NP_].boundary,N*sizeof(int));
    NP_++;					/* update polygon counter */
    return 0;} 
  else {					/* update bounding box... */
    if (x<P_[NP_].bxll) P_[NP_].bxll=x;
    if (y<P_[NP_].byll) P_[NP_].byll=y;
    if (x>P_[NP_].bxur) P_[NP_].bxur=x;
    if (y>P_[NP_].byur) {
      P_[NP_].byur=y;
      if (l->boundary) P_[NP_].testL=NULL;	/* ...and testline */
      else if (P_[NP_].testL!=NULL) P_[NP_].testL=l;
    }

    if (N%CHUNK==0) {				/* alloc more memory for polys */
      if ((P_[NP_].x=(double*) 
	realloc((void*) P_[NP_].x,(N/CHUNK+1)*CHUNK*sizeof(double)))==NULL)
	  PSerror("dContours\n\t\tput_on_p","memory overflow (0)");
      if ((P_[NP_].y=(double*) 
	realloc((void*) P_[NP_].y,(N/CHUNK+1)*CHUNK*sizeof(double)))==NULL)
	  PSerror("dContours\n\t\tput_on_p","memory overflow (1)");
      if (P_[NP_].boundary!=NULL) {
	if ((P_[NP_].boundary=(int*) 
	  realloc((void*) P_[NP_].boundary,(N/CHUNK+1)*CHUNK*sizeof(int)))==NULL)
	    PSerror("dContours\n\t\tput_on_p","memory overflow (2)");
      }
    }
    return N;
  }
}
/*--------------------------------------------------------------------------------------
deleteP_: delete set of polygons
--------------------------------------------------------------------------------------*/
void deleteP_(void)
{
  int ip;
  
  for (ip=0;ip<NP_;ip++) {
    free((void*) P_[ip].x);
    free((void*) P_[ip].y);
    if (P_[ip].boundary!=NULL) free((void*) P_[ip].boundary);
  }
  free((void*) P_);
  
  return;
}
/*--------------------------------------------------------------------------------------
remove_holes: find holes in set of all polygons and assign proper pattern to them
--------------------------------------------------------------------------------------*/
void remove_holes(double Z[])
{
  int ip;
  struct line *l;
  
  for (ip=0;ip<NP_;ip++) {
    l=P_[ip].testL;
    if (l!=NULL) {				/* kind of polygon not determined yet */
      if (Z[l->n[0]]==C_ctr[P_[ip].ictr]) {
        if (l->ey>l->by)
	  P_[ip].color=P_[ip].ictr-1;
      }
      else {
        if ((l->by>l->ey && Z[l->n[0]]>C_ctr[P_[ip].ictr]) ||
	    (l->by<l->ey && Z[l->n[0]]<C_ctr[P_[ip].ictr]))
	  P_[ip].color=P_[ip].ictr-1;
      }
    }
  }

  return;
}
/*--------------------------------------------------------------------------------------
fback: fill entire domain with gray/color corresponding to lowest value
--------------------------------------------------------------------------------------*/
void fback(double *Z)
{
  int i;
  double *x,*y,min;
  struct line *l;
  struct bline *b;

  if ((x=(double*) malloc(NB_*sizeof(double)))==NULL)
    PSerror("dContours\n\t\tin fback","memory overflow (0)");
  if ((y=(double*) malloc(NB_*sizeof(double)))==NULL)
    PSerror("dContours\n\t\tin fback","memory overflow (1)");

  b=B_; l=b->l;					/* get bounding polygon */
  x[0]=l->ex; y[0]=l->ey;
  for (i=1;i<NB_;i++) {
    b=b->path[0]; l=b->l;
    if (x[i-1]==l->ex && y[i-1]==l->ey) {x[i]=l->bx; y[i]=l->by;}
    else {x[i]=l->ex; y[i]=l->ey;}
  }
  
  min=Z[0];					/* get gray/color.. */
  for (i=1;i<N_gn;i++) if (min>Z[i]) min=Z[i];
  for (i=0;i<N_Ctr;i++) if (min<C_ctr[i]) break;
  sFill(C_col[i-1]);				/* ..and fill background */
  dP2P(x,y,NB_);

  free((void*) x); free((void*) y);		/* release memory */
  
  return;
}
/*--------------------------------------------------------------------------------------
cmp_p: compare the two polygons P_[i1] and P_[i2];
       a polygon is "bigger" if its size is smaller,or,if the sizes are equal,if 
       its C_ctr-value is larger.
--------------------------------------------------------------------------------------*/
int cmp_p(int *i1,int *i2)
{
  double s1,s2;
  
  s1=(P_[*i1].bxur-P_[*i1].bxll)*(P_[*i1].byur-P_[*i1].byll);
  s2=(P_[*i2].bxur-P_[*i2].bxll)*(P_[*i2].byur-P_[*i2].byll);
  if (s1==s2) {
    return (C_ctr[P_[*i1].ictr]>C_ctr[P_[*i2].ictr])?1:-1;
  }
  else
    return (s1>s2)?-1:1;
}
/*--------------------------------------------------------------------------------------
gSpecPoly: get special polygon starting on boundary line B_[iB].l;
	   Z[B_[iB].l->n[iN]]>C_ctr[0]
--------------------------------------------------------------------------------------*/
void gSpecPoly(double Z[],int iB,int iN,int no_frame_onB_)
{
  int ictr,i;
  double x,y,z0;
  struct line *l;
  struct bline *b;
  
  z0=Z[B_[iB].l->n[iN]];			/* find corresponding C_ctr level */
  if (z0>C_ctr[N_Ctr-1]) 
    ictr=N_Ctr-1;
  else {
    for (i=0;i<N_Ctr;i++) if (C_ctr[i] >= z0) break;
    ictr=--i;
  }

  if (gCrossPoints(Z,C_ctr[ictr])>0) {		/* get crossing points */
    get_p(B_[iB].l,Z,ictr,no_frame_onB_);}
  else {					/* no CP's=>poly follows entire bdr */
    new_p(no_frame_onB_);
    P_[NP_].ictr=ictr; P_[NP_].color=ictr;
    P_[NP_].testL=NULL;
    b=B_; l=b->l;
    put_on_p(l->bx,l->by,l,1);
    x=l->ex; y=l->ey;
    put_on_p(x,y,l,1);
    for (i=1;i<NB_;i++) {
      B_[i].state[0]=1; B_[i].state[1]=1;
      b=b->path[0]; l=b->l;
      if (x==l->ex && y==l->ey) {
        x=l->bx; y=l->by;}
      else {
        x=l->ex; y=l->ey;}
      put_on_p(x,y,l,1);
    }
  }
  
  return;
}

