#undef COM_SYM_DEF
#include "PSgraf_p.h"
/*--------------------------------------------------------------------------------------
dLegend: draw legend;
	 The lower left corner of the legend bar is at (x0,y0) and its extension in x-
	 and y-direction is dx and dy, respectively. (All measures are in Paper 
	 coordinates; dx and dy need not be positive.) If (horiz), the bar is drawn
	 horizontally, else vertically. Every |nth| contour level is numbered using the
	 current format for numbers. The numbers are written in the negative dx- and
	 dy-direction, respectively.
	 For nth<=0 the legend is drawn as a bitmap and corresponds to a density plot.

HISTORY:
  v3.0.0									031115kr
  expanded for use with density plots						971129kr
  v2.0										920524kr
  remove text adjustment if TextRotation!=0					940828kr
  if (C_gry[i]==NONE || i%nth==0) --> if (i%nth==0) at line 65			970828kr
--------------------------------------------------------------------------------------*/
void dLegend(double x0,double y0,double dx,double dy,int horiz,int nth)
{
  char oldhadj=hadj,oldvadj=vadj;
  int i,j,n,nx,ny,anth=(nth>0)?nth:-nth;
  double rdx,rdy,sdx,sdy,tdx,tdy,x[5],y[5],
  	*bm0,*bm1,*bm2,*bm3,
	oldPthick=Pthick;
  struct color oldfill,oldstroke;

  oldfill=gFill(); oldstroke=gStroke();
  
  if (nth>0) {					/*---------- legend for contour plot */
    fprintf(PSout,"%%--dLegend (for contour plot)\n");
    if (horiz) {
      rdx=dx/(N_Ctr+1); rdy=dy;
      sdx=rdx; sdy=0; tdx=0; hadj='C';
      if (dy>0) {tdy=-0.2*dy; vadj='T';} else {tdy=1.2*dy; vadj='B';}
    } else {
      rdx=dx; rdy=dy/(N_Ctr+1);
      sdx=0; sdy=rdy; tdy=0; vadj='C';
      if (dx>0) {tdx=-1.5*X_stretch*curFSize; hadj='R';}
      else {tdx=0.5*X_stretch*curFSize; hadj='L';}
    }
    
    if (TextRotation!=0) {hadj=oldhadj; vadj=oldvadj;}
  
    x[0]=x[3]=x[4]=x0; x[1]=x[2]=x0+dx;		/* fill legend bar */
    y[0]=y[1]=y[4]=y0; y[2]=y[3]=y0+dy;
    sFill(C_col[-1]);
    dPolygon('P',x,y,5);
  
    x0+=sdx; y0+=sdy;				/* fill bar elements */
    Pthick=0;
    for (i=0;i<N_Ctr;i++) {
      x[0]=x[3]=x[4]=x0+i*sdx; x[1]=x[2]=x0+i*sdx+rdx;
      y[0]=y[1]=y[4]=y0+i*sdy; y[2]=y[3]=y0+i*sdy+rdy;
      sFill(C_col[i]);
      dPolygon('P',x,y,5);
    }
    Pthick=oldPthick; sStroke(oldstroke);
  
    for (i=0;i<N_Ctr;i+=anth) {			/* labelling */
      movea('P',x0+i*sdx+tdx,y0+i*sdy+tdy); dNumber(C_ctr[i]);
    }
    for (i=0;i<N_Ctr;i++)
      if (i%anth==0)
	dLine('P',x0+i*sdx,y0+i*sdy,x0+i*sdx+(horiz?0:rdx),y0+i*sdy+(horiz?rdy:0));
    
    if (Pthick<=0) Pthick=0.1;			/* frame legend bar */
    x0-=sdx; y0-=sdy;
    x[0]=x[3]=x[4]=x0; x[1]=x[2]=x0+dx;
    y[0]=y[1]=y[4]=y0; y[2]=y[3]=y0+dy;
    sFill(); dPolygon('P',x,y,5);
  }
  
  
  else {					/*---------- legend for density plot */
    fprintf(PSout,"%%--dLegend (for density plot)\n");
    if (horiz) {
      rdx=dx/(N_Ctr-1); rdy=dy;
      sdx=rdx; sdy=0; tdx=0; hadj='C';
      if (dy>0) {tdy=-0.2*dy; vadj='T';} else {tdy=1.2*dy; vadj='B';}
      n=(int)(dx/(PS_PIX_SIZE*(N_Ctr-1))); nx=(N_Ctr-1)*n; ny=1;
    } else {
      rdx=dx; rdy=dy/(N_Ctr-1);
      sdx=0; sdy=rdy; tdy=0; vadj='C';
      if (dx>0) {tdx=-1.5*X_stretch*curFSize; hadj='R';}
      else {tdx=0.5*X_stretch*curFSize; hadj='L';}
      n=(int)(dy/(PS_PIX_SIZE*(N_Ctr-1))); nx=1; ny=(N_Ctr-1)*n;
    }
    
    if (TextRotation!=0) {hadj=oldhadj; vadj=oldvadj;}

    switch (ColorSpace) {
      case 0:
	if ((bm0=(double*)malloc(N_Ctr*n*sizeof(double)))==NULL)
	  PSerror("dLegend","memory overflow");
	for (i=1;i<N_Ctr;i++)
	  for (j=0;j<n;j++)
	    bm0[j+(i-1)*n]=C_col[i-1].c0+(C_col[i].c0-C_col[i-1].c0)*j/n;
        dBitMap(x0,y0,dx,dy,nx,ny,bm0);
	free((void*)bm0);
	break;
      case 2: case 4:
	if ((bm0=(double*)malloc(N_Ctr*n*sizeof(double)))==NULL)
	  PSerror("dLegend","memory overflow");
	if ((bm1=(double*)malloc(N_Ctr*n*sizeof(double)))==NULL)
	  PSerror("dLegend","memory overflow");
	if ((bm2=(double*)malloc(N_Ctr*n*sizeof(double)))==NULL)
	  PSerror("dLegend","memory overflow");
	for (i=1;i<N_Ctr;i++)
	  for (j=0;j<n;j++) {
	    bm0[j+(i-1)*n]=C_col[i-1].c0+(C_col[i].c0-C_col[i-1].c0)*j/n;
	    bm1[j+(i-1)*n]=C_col[i-1].c1+(C_col[i].c1-C_col[i-1].c1)*j/n;
	    bm2[j+(i-1)*n]=C_col[i-1].c2+(C_col[i].c2-C_col[i-1].c2)*j/n;
	  }
        dBitMap(x0,y0,dx,dy,nx,ny,bm0,bm1,bm2);
	free((void*)bm0); free((void*)bm1); free((void*)bm2);
	break;
      case 8:
	if ((bm0=(double*)malloc(N_Ctr*n*sizeof(double)))==NULL)
	  PSerror("dLegend","memory overflow");
	if ((bm1=(double*)malloc(N_Ctr*n*sizeof(double)))==NULL)
	  PSerror("dLegend","memory overflow");
	if ((bm2=(double*)malloc(N_Ctr*n*sizeof(double)))==NULL)
	  PSerror("dLegend","memory overflow");
	if ((bm3=(double*)malloc(N_Ctr*n*sizeof(double)))==NULL)
	  PSerror("dLegend","memory overflow");
	for (i=1;i<N_Ctr;i++)
	  for (j=0;j<n;j++) {
	    bm0[j+(i-1)*n]=C_col[i-1].c0+(C_col[i].c0-C_col[i-1].c0)*j/n;
	    bm1[j+(i-1)*n]=C_col[i-1].c1+(C_col[i].c1-C_col[i-1].c1)*j/n;
	    bm2[j+(i-1)*n]=C_col[i-1].c2+(C_col[i].c2-C_col[i-1].c2)*j/n;
	    bm3[j+(i-1)*n]=C_col[i-1].c3+(C_col[i].c3-C_col[i-1].c3)*j/n;
	  }
        dBitMap(x0,y0,dx,dy,nx,ny,bm0,bm1,bm2,bm3);
	free((void*)bm0); free((void*)bm1); free((void*)bm2); free((void*)bm3);
	break;
      default:
	PSwarning("dLegend","ColorSpace undefined: operator ignored");
	break;
    }

    for (i=0;i<N_Ctr;i+=anth) {			/* labelling */
      movea('P',x0+i*sdx+tdx,y0+i*sdy+tdy); dNumber(C_ctr[i]);
    }
    for (i=0;i<N_Ctr;i++)
      if (i%anth==0)
	dLine('P',x0+i*sdx,y0+i*sdy,x0+i*sdx+(horiz?0:rdx),y0+i*sdy+(horiz?rdy:0));
    
    if (Pthick<=0) Pthick=0.1;			/* frame legend bar */
    y0-=sdy;
    x[0]=x[3]=x[4]=x0; x[1]=x[2]=x0+dx;
    y[0]=y[1]=y[4]=y0; y[2]=y[3]=y0+dy;
    sFill(); dPolygon('P',x,y,5);
  }

  hadj=oldhadj; vadj=oldvadj;
  sFill(oldfill);

  return;
}
