#undef COM_SYM_DEF
#include "PSgraf_p.h"
/*--------------------------------------------------------------------------------------
dPolygon: draw polygon {xi| i=1,..N} in coordinate system CS [P,W]

HISTORY:
  v3.0.0									031115kr
--------------------------------------------------------------------------------------*/
void dPolygon(char CS,struct dvec *X,int N)
{
  int i;
  double *x,*y,*z;
  
  if (N<2) {
    PSwarning("dPolygon","less than 2 points; operator ignored");
    return;
  }
  if (Pthick<=0 && F_c0==NONE) {
    PSwarning("dPolygon",
	      "filling pattern=NONE and pen thickness=0; operator ignored");
    return;
  }
  CS=toupper(CS);
  if (CS!='W' && CS!='P') {
    PSwarning("dPolygon","coordinate system not implemented; operator ignored");
    return;
  }
  for (i=1;i<N;i++)
    if (X[i-1].dim!=X[i].dim) {
      PSwarning("dPolygon","different dimensions of individual points; operator ignored");
      return;
    }

  x=(double*)malloc(N*sizeof(double)); if (x==NULL) PSerror("dPolygon","memory overflow");
  y=(double*)malloc(N*sizeof(double)); if (y==NULL) PSerror("dPolygon","memory overflow");

  fprintf(PSout,"%%--dPolygon (%c%i)\n",CS,X[0].dim);

  if (X[0].dim==2) {			// 2 d
    for (i=0;i<N;i++) {x[i]=X[i].x; y[i]=X[i].y;}
    if (CS=='P') dP2P(x,y,N); else dP2W(x,y,N);
  } else {				// 3 d
    z=(double*)malloc(N*sizeof(double));
    if (z==NULL) PSerror("dPolygon","memory overflow");
    for (i=0;i<N;i++) {x[i]=X[i].x; y[i]=X[i].y; z[i]=X[i].z;}
    if (CS=='P') dP3P(x,y,z,N); else dP3W(x,y,z,N);
    free((void*)z);
  }

  free((void*)x); free((void*)y);
  
  return;
}
/*--------------------------------------------------------------------------------------
dPolygon: draw polygon {xi,yi| i=1,..N} in 2d coordinate system CS [P,W]

HISTORY:
  v3.0.0, renamed from dPolygon							031115kr
  remove redundant points							971116kr
  v2.0										920524kr
  pointer formulation								920203kr
--------------------------------------------------------------------------------------*/
void dPolygon(char CS,double *x,double *y,int N)
{
  if (N<2) {
    PSwarning("dPolygon","less than 2 points; operator ignored");
    return;
  }
  if (Pthick<=0 && F_c0==NONE) {
    PSwarning("dPolygon",
	      "filling pattern=NONE and pen thickness=0; operator ignored");
    return;
  }

  fprintf(PSout,"%%--dPolygon (%c)\n",CS);

  CS=toupper(CS);

  if (CS=='P') {dP2P(x,y,N); return;}

  if (CS=='W') {dP2W(x,y,N); return;}

  PSwarning("dPolygon","coordinate system not implemented; operator ignored");

  return;
}
/*--------------------------------------------------------------------------------------
dPolygon: draw polygon {xi,yi,zi| i=1,..N} in coordinate system CS [P,W]

HISTORY:
  v3.0.0, renamed from dPolygon							031115kr
--------------------------------------------------------------------------------------*/
void dPolygon(char CS,double *x,double *y,double *z,int N)
{
  if (N<2) {
    PSwarning("dPolygon","less than 2 points; operator ignored");
    return;
  }
  if (Pthick<=0 && F_c0==NONE) {
    PSwarning("dPolygon",
	      "filling pattern=NONE and pen thickness=0; operator ignored");
    return;
  }
  CS=toupper(CS);

  fprintf(PSout,"%%--dPolygon (%c)\n",CS);

  if (CS=='P') {dP3P(x,y,z,N); return;}
  if (CS=='w') {dP3W(x,y,z,N); return;}
  
  PSwarning("dPolygon","coordinate system not implemented; operator ignored");

  return;
}
/*--------------------------------------------------------------------------------------
dP2P: draw polygon {xi,yi| i=1,..N} in 2d P without checks

HISTORY:
  v3.0.0									031220kr
--------------------------------------------------------------------------------------*/
void dP2P(double *x,double *y,int N)
{
  int i;
  double dy0,dy1;

  if (lay_text>0 && F_c0!=NONE) lay_fill=lay_text+1;

  fprintf(PSout,"gsave\n");
  fprintf(PSout,"newpath\n");
  m2Pa(x[0],y[0]);
  for (i=1;i<N-1;i++) {
    dy0=y[i]-y[i-1]; dy1=y[i+1]-y[i-1];
    if (dy0*dy1==0) {
      if (dy0!=dy1) l2Pa(x[i],y[i]);
    } else {
      if (fabs((x[i]-x[i-1])/dy0-(x[i+1]-x[i-1])/dy1)>FLT_EPSILON) l2Pa(x[i],y[i]);
    }
  }
  l2Pa(x[N-1],y[N-1]);
  m2Pa(x[0],y[0]);
  fprintf(PSout,"closepath\n");
  if (F_c0!=NONE) {
    if (Pthick>0) fprintf(PSout,"gsave ");
    if (ColorSpace==0)
      fprintf(PSout,"%f setgray fill ",1-F_c0);
    else if (ColorSpace==2)
      fprintf(PSout,"%f %f %f setrgbcolor fill ",F_c0,F_c1,F_c2);
    else if (ColorSpace==4)
      fprintf(PSout,"%f %f %f sethsbcolor fill ",F_c0,F_c1,F_c2);
    else if (ColorSpace==8)
      fprintf(PSout,"%f %f %f %f setcmykcolor fill ",F_c0,F_c1,F_c2,F_c3);
    if (Pthick>0) fprintf(PSout,"grestore ");
  }
  if (Pthick>0) fprintf(PSout,"stroke\n"); else fprintf(PSout,"\n");
  fprintf(PSout,"grestore\n");
  
  return;
}
/*--------------------------------------------------------------------------------------
dP2W: draw polygon {xi,yi| i=1,..N} in 2d W without checks

HISTORY:
  v3.0.0									031220kr
--------------------------------------------------------------------------------------*/
void dP2W(double *x,double *y,int N)
{
  int i;
  double *xp,*yp;
  
  xp=(double*)malloc(N*sizeof(double));
  if (xp==NULL) PSerror("dP2W","memory overflow");
  yp=(double*)malloc(N*sizeof(double));
  if (yp==NULL) PSerror("dP2W","memory overflow");

  for (i=0;i<N;i++) pWP(x[i],y[i],xp+i,yp+i);
  dP2P(xp,yp,N);

  free((void*)xp); free((void*)yp);

  return;
}
/*--------------------------------------------------------------------------------------
dP3P: draw polygon {xi,yi,zi| i=1,..N} in 3d P without checks

HISTORY:
  v3.0.0									031220kr
--------------------------------------------------------------------------------------*/
void dP3P(double *x,double *y,double *z,int N)
{
  int i;
  double *xp,*yp;
  
  xp=(double*)malloc(N*sizeof(double));
  if (xp==NULL) PSerror("dP2W","memory overflow");
  yp=(double*)malloc(N*sizeof(double));
  if (yp==NULL) PSerror("dP2W","memory overflow");
  
  for (i=0;i<N;i++) pPI(x[i],y[i],z[i],xp+i,yp+i);
  dP2P(xp,yp,N);

  free((void*)xp); free((void*)yp);

  return;
}
/*--------------------------------------------------------------------------------------
dP3W: draw polygon {xi,yi,zi| i=1,..N} in 3d W without checks

HISTORY:
  v3.0.0									031220kr
--------------------------------------------------------------------------------------*/
void dP3W(double *x,double *y,double *z,int N)
{
  int i;
  double *xp,*yp;
  
  xp=(double*)malloc(N*sizeof(double));
  if (xp==NULL) PSerror("dP2W","memory overflow");
  yp=(double*)malloc(N*sizeof(double));
  if (yp==NULL) PSerror("dP2W","memory overflow");
  
  for (i=0;i<N;i++) pWI(x[i],y[i],z[i],xp+i,yp+i);
  dP2P(xp,yp,N);

  free((void*)xp); free((void*)yp);

  return;
}
