#undef COM_SYM_DEF
#include "PSgraf_p.h"
/*--------------------------------------------------------------------------------------
dSymbol: draw symbol SY at position x in coordinate system CS; 
         the entire symbol is contained in a circle of radius R in paper coordinates;
	   "movea" and "linea" are directly encoded for efficiency;
  SY 	0: circle (default)	10: star
	1: square		11: times
	2: diamond		12: cross
	3: btrinagle		13: skel_btrinagle
	4: ttrinagle		14: skel_ttrinagle

HISTORY:
  v3.0.1 debug: filled CMYK							040623kr
  v3.0.0									031115kr
  v2.0										920524kr
--------------------------------------------------------------------------------------*/
void dSymbol(char CS,struct dvec X,double R,int SY)
{
  double b0,b1;

  if (Pthick<=0 && F_c0==NONE) {
    PSwarning("dSymbol",
	      "filling pattern=NONE and pen thickness=0; operator ignored");
    return;
  }
    
  CS=toupper(CS);
  if (CS!='W' && CS!='P')
    PSerror("dSymbol","requested coordinate system not implemented");
  if (X.dim!=2 && X.dim!=3)
    PSerror("dSymbol","dimension different from 2 or 3");

  if (CS=='W')  pWP(X,&X);
  if (X.dim==3) pPI(X,&X);

  cpx=X.x; cpy=X.y;					/* update current point */
  updateBB(X.x+R, X.y); updateBB(X.x-R, X.y);      	/* update BB */
  updateBB(X.x, X.y+R); updateBB(X.x, X.y-R);
  
  fprintf(PSout, "%%--dSymbol (%i)\n", SY);		/* draw symbol */
  fprintf(PSout, "gsave\n  newpath\n");
  switch (SY) {
    default:
	PSerror("dSymbol", "requested symbol not defined: circle used as substitute");
    case 0:						/* circle */
	fprintf(PSout, "    %f %f %f 0 360 arc\n",  X.x, X.y, R);
	fprintf(PSout, "    %f %f  moveto\n", X.x, X.y);
      break;
    case 1:						/* square */
      b0=0.707107*R;
	fprintf(PSout, "    %f %f  moveto\n", X.x+b0, X.y+b0);
	fprintf(PSout, "    %f %f  lineto\n", X.x-b0, X.y+b0);
	fprintf(PSout, "    %f %f  lineto\n", X.x-b0, X.y-b0);
	fprintf(PSout, "    %f %f  lineto\n", X.x+b0, X.y-b0);
      break;
    case 2:						/* diamond */
	fprintf(PSout, "    %f %f  moveto\n", X.x,    X.y+R);
	fprintf(PSout, "    %f %f  lineto\n", X.x-R,  X.y);
	fprintf(PSout, "    %f %f  lineto\n", X.x,    X.y-R);
	fprintf(PSout, "    %f %f  lineto\n", X.x+R,  X.y);
      break;
    case 3:						/* btriangle */
	b0=0.5*R; b1=0.86603*R;
	fprintf(PSout, "    %f %f  moveto\n", X.x,    X.y+R);
	fprintf(PSout, "    %f %f  lineto\n", X.x-b1, X.y-b0);
	fprintf(PSout, "    %f %f  lineto\n", X.x+b1, X.y-b0);
      break;
    case 4:						/* ttriangle */
	b0=0.5*R; b1=0.86603*R;
	fprintf(PSout, "    %f %f  moveto\n", X.x,    X.y-R);
	fprintf(PSout, "    %f %f  lineto\n", X.x+b1, X.y+b0);
	fprintf(PSout, "    %f %f  lineto\n", X.x-b1, X.y+b0);
      break;
    case 10:						/* star */
	b0=0.707107*R;
	fprintf(PSout, "    %f %f  moveto\n", X.x,    X.y+R);
	fprintf(PSout, "    %f %f  lineto\n", X.x,    X.y-R);
	fprintf(PSout, "    %f %f  moveto\n", X.x+R,  X.y);
	fprintf(PSout, "    %f %f  lineto\n", X.x-R,  X.y);
	fprintf(PSout, "    %f %f  moveto\n", X.x+b0, X.y+b0);
	fprintf(PSout, "    %f %f  lineto\n", X.x-b0, X.y-b0);
	fprintf(PSout, "    %f %f  moveto\n", X.x-b0, X.y+b0);
	fprintf(PSout, "    %f %f  lineto\n", X.x+b0, X.y-b0);
	fprintf(PSout, "    %f %f  moveto\n", X.x+b0, X.y+b0);
      break;
    case 11:						/* times */
	b0=0.707107*R;
	fprintf(PSout, "    %f %f  moveto\n", X.x+b0, X.y+b0);
	fprintf(PSout, "    %f %f  lineto\n", X.x-b0, X.y-b0);
	fprintf(PSout, "    %f %f  moveto\n", X.x-b0, X.y+b0);
	fprintf(PSout, "    %f %f  lineto\n", X.x+b0, X.y-b0);
	fprintf(PSout, "    %f %f  moveto\n", X.x+b0, X.y+b0);
      break;
    case 12:						/* cross */
	fprintf(PSout, "    %f %f  moveto\n", X.x,    X.y+R);
	fprintf(PSout, "    %f %f  lineto\n", X.x,    X.y-R);
	fprintf(PSout, "    %f %f  moveto\n", X.x+R,  X.y);
	fprintf(PSout, "    %f %f  lineto\n", X.x-R,  X.y);
	fprintf(PSout, "    %f %f  moveto\n", X.x,    X.y+R);
      break;
    case 13:						/* skel_btrinagle */
	b0=0.5*R; b1=0.86603*R;
	fprintf(PSout, "    %f %f  moveto\n", X.x,     X.y+R);
	fprintf(PSout, "    %f %f  lineto\n", X.x,     X.y);
	fprintf(PSout, "    %f %f  lineto\n", X.x-b1,  X.y-b0);
	fprintf(PSout, "    %f %f  moveto\n", X.x+b1,  X.y-b0);
	fprintf(PSout, "    %f %f  lineto\n", X.x,     X.y);
	fprintf(PSout, "    %f %f  moveto\n", X.x,     X.y+R);
      break;
    case 14:						/* skel_ttrinagle */
	b0=0.5*R; b1=0.86603*R;
	fprintf(PSout, "    %f %f  moveto\n", X.x,     X.y-R);
	fprintf(PSout, "    %f %f  lineto\n", X.x,     X.y);
	fprintf(PSout, "    %f %f  lineto\n", X.x+b1,  X.y+b0);
	fprintf(PSout, "    %f %f  moveto\n", X.x-b1,  X.y+b0);
	fprintf(PSout, "    %f %f  lineto\n", X.x,     X.y);
	fprintf(PSout, "    %f %f  moveto\n", X.x,     X.y-R);
      break;
  }
  fprintf(PSout, "  closepath\n");
  if (F_c0!=NONE) {
    if (ColorSpace==0)
      fprintf(PSout, "gsave %f setgray fill grestore", 1-F_c0);
    else if (ColorSpace==2)
      fprintf(PSout, "gsave %f %f %f setrgbcolor fill grestore",  F_c0,F_c1,F_c2);
    else if (ColorSpace==4)
      fprintf(PSout, "gsave %f %f %f sethsbcolor fill grestore",  F_c0,F_c1,F_c2);
    else if (ColorSpace==8)
      fprintf(PSout, "gsave %f %f %f %f setcmykcolor fill grestore", F_c0,F_c1,F_c2,F_c3);
  }
  if (Pthick>0) fprintf(PSout, " stroke\n"); else fprintf(PSout, "\n");

  fprintf(PSout, "grestore\n");

  return;
}
/*--------------------------------------------------------------------------------------
dSymbol: draw symbol SY at position x in coordinate system CS; 
         the entire symbol is contained in a circle of radius R in paper coordinates;

HISTORY:
  v3.0.0									031115kr
  v2.0										920524kr
--------------------------------------------------------------------------------------*/
void dSymbol(char CS,double x,double y,double R,int SY)
{
  struct dvec X;

  X.dim=2; X.x=x; X.y=y;
  dSymbol(CS,X,R,SY);

  return;
}
/*--------------------------------------------------------------------------------------
dSymbol: draw symbol SY at position x in coordinate system CS; 
         the entire symbol is contained in a circle of radius R in paper coordinates;

HISTORY:
  v3.0.0									031115kr
--------------------------------------------------------------------------------------*/
void dSymbol(char CS,double x,double y,double z,double R,int SY)
{
  struct dvec X;

  X.dim=3; X.x=x; X.y=y; X.z=z;
  dSymbol(CS,X,R,SY);

  return;
}
