#undef COM_SYM_DEF
#include "PSgraf_p.h"
/*--------------------------------------------------------------------------------------
dTIFF:  Draw TIFF image fn in PaperCoordinates with lower left corner (x0,y0) and upper
	right corner (x0+dpx,y0+dpy). The original proportions of the image are
	maintained if dpx=0 or dpy=0. For dpx=0 image height is dpy, for dpy=0 image
	width is dpx and for dpx=dpy=0 natural dimensions are chosen with pixel size
	1/72.27 in. The actual values (dpx,dpy) used for the drawing are returned.
	In the current implementation, the resulting picture will be 24 bit RBG,
	independent of choice of ColorSpace and TIFF-format.

HISTORY:
  v3.0.0									031222kr
  created				       					971206kr
--------------------------------------------------------------------------------------*/
void dTIFF(double x0,double y0,double *dpx,double *dpy,const char *fn)
{
  int ic=0;
  size_t i,npixels;
  uint32 w,h,*raster;
  TIFF *tif;

  if((tif=TIFFOpen(fn,"r"))==0) printf("\ncould not open %s\n",fn);
  TIFFGetField(tif,TIFFTAG_IMAGELENGTH,&h);
  TIFFGetField(tif,TIFFTAG_IMAGEWIDTH,&w);
  npixels=w*h;
  raster=(uint32*) _TIFFmalloc(npixels*sizeof(uint32));
  if (raster==NULL) PSerror("dTIFF","memory overflow");
  TIFFReadRGBAImage(tif,w,h,raster,0);
  TIFFClose(tif);
  
  if (*dpx==0) {
    if (*dpy==0) {*dpx=w*25.4/72.27; *dpy=h*25.4/72.27;}
    else {*dpx=*dpy*w/h;}
  } else {
    if (*dpy==0) {*dpy=*dpx*h/w;}
  }

  fprintf(PSout, "%%-- dTIFF \n");		/* draw RGB bitmap from TIFF file */
  fprintf(PSout,"gsave\n");
  fprintf(PSout,"/DeviceRGB setcolorspace\n");
  fprintf(PSout,"%f %f translate\n%f %f scale\n",x0,y0,*dpx,*dpy);
  fprintf(PSout,"<<\n");
  fprintf(PSout,"  /ImageType 1\n");
  fprintf(PSout,"  /Width %i /Height %i\n",w,h);
  fprintf(PSout,"  /BitsPerComponent 8\n");
  fprintf(PSout,"  /Decode [0 1 0 1 0 1]\n");
  fprintf(PSout,"  /ImageMatrix [%i 0 0 %i 0 0]\n",w,h);
  fprintf(PSout,"  /DataSource currentfile /ASCIIHexDecode filter\n");
  fprintf(PSout,"  /Interpolate true\n");
  fprintf(PSout,">>\n");
  fprintf(PSout,"image\n");
  for (i=0;i<npixels;i++) {
    ic+=6; if (ic>80) {fprintf(PSout,"\n"); ic=6;}
    fprintf(PSout,"%02x%02x%02x",
	    TIFFGetR(raster[i]),TIFFGetG(raster[i]),TIFFGetB(raster[i]));
   }
  fprintf(PSout,">");

  fprintf(PSout,"\ngrestore\n");
  updateBB(x0,y0);
  updateBB(x0+*dpx,y0+*dpy);
  
  _TIFFfree(raster);

return;
}
