#undef COM_SYM_DEF
#include "PSgraf_p.h"
/*--------------------------------------------------------------------------------------
gDRGrid: generate Deformed Regular Grid;
	 The deformed grid is topologically identically to a regular grid with Nx
	 nodes in the x-direction and Ny nodes in the y-direction. The coordinates
	 of the nodes are given by {(x[i],y[i],[z[i]]) | i=0,..,Nx*Ny-1}, where z[i] is
	 optional for the 3d case.

HISTORY:
  closed memory leak								040206kr
  v3.0.0, optional 3d input, generate grid on projection plane			031115kr
  introduced grid type (adaptation to arbitrary triangular grids)      		001217kr
  v2.0										920524kr
--------------------------------------------------------------------------------------*/
void do_DR_Grid(double*X,double*Y,int Nx,int Ny);
void gDRGrid(double*x,double*y,int Nx,int Ny)
{
  int i;
  double *X,*Y;
  struct dvec vw,vp;
  
  X=(double*) malloc(Nx*Ny*sizeof(double));
  if (X==NULL) PSerror("gDRGrid","memory overflow");
  Y=(double*) malloc(Nx*Ny*sizeof(double));
  if (X==NULL) PSerror("gDRGrid","memory overflow");

  vw.dim=2;
  for (i=0;i<Nx*Ny;i++) {
    vw.x=x[i]; vw.y=y[i];
    pWI(vw,&vp);
    X[i]=vp.x; Y[i]=vp.y;
  }

  do_DR_Grid(X,Y,Nx,Ny);

  free((void*) X); free((void*) Y);

  return;
}
void gDRGrid(double*x,double*y,double*z,int Nx,int Ny)
{
  int i;
  double *X,*Y;
  struct dvec vw,vp;
  
  X=(double*) malloc(Nx*Ny*sizeof(double));
  if (X==NULL) PSerror("gDRGrid","memory overflow");
  Y=(double*) malloc(Nx*Ny*sizeof(double));
  if (X==NULL) PSerror("gDRGrid","memory overflow");

  vw.dim=3;
  for (i=0;i<Nx*Ny;i++) {
    vw.x=x[i]; vw.y=y[i]; vw.z=z[i];
    pWI(vw,&vp);
    X[i]=vp.x; Y[i]=vp.y;
  }

  do_DR_Grid(X,Y,Nx,Ny);

  free((void*) X); free((void*) Y);

  return;
}


void do_DR_Grid(double*X,double*Y,int Nx,int Ny)
{
  int iL=0,buf,iy,_Nx,_Ny,i,j;
  struct bline *b;

  _Nx=Nx-1; _Ny=Ny-1;
  if (L_!=NULL) {
    PSwarning("do_DR_Grid in gDRGrid","previously existing grid definition deleted");
    free((void*) L_);}
  if (B_!=NULL) {
    PSwarning("do_DR_Grid in gDRGrid","previously existing boundary definition deleted");
    free((void*) B_);}
  
  NL_=_Nx*Ny+Nx*_Ny;					/* alloc. mem. for grid lines */
  L_=(struct line*) malloc(NL_*sizeof(struct line));
  if (L_==NULL) PSerror("do_DR_Grid in gDRGrid","memory overflow");

  for (j=0;j<Ny;j++) {					/* horizontal lines */
    for (i=0;i<_Nx;i++) {
      L_[iL].id=iL; L_[iL].n[0]=iL+iL/_Nx; L_[iL].n[1]=L_[iL].n[0]+1;
      buf=_Nx*Ny+iL/_Nx+(iL%_Nx)*_Ny;			/* L_[iL].l[2] */
      if (iL<_Nx) {
	L_[iL].boundary=1;
	L_[iL].l[5]=L_[iL].l[4]=L_[iL].l[3]=NULL;
	L_[iL].l[0]=L_+buf+_Ny; L_[iL].l[1]=L_+iL+_Nx; L_[iL].l[2]=L_+buf;
      }
      else {
	L_[iL].l[5]=L_+buf-1; L_[iL].l[4]=L_+iL-_Nx; L_[iL].l[3]=L_+buf+_Ny-1;
	if (iL >=_Nx*_Ny) {
	  L_[iL].boundary=1; L_[iL].l[0]=L_[iL].l[1]=L_[iL].l[2]=NULL;
	}
	else {
	  L_[iL].boundary=0;
	  L_[iL].l[0]=L_+buf+_Ny; L_[iL].l[1]=L_+iL+_Nx; L_[iL].l[2]=L_+buf;
	}
      }
      iL++;
    }
  }

  for (i=0;i<Nx;i++) {					/* vertical lines */
    for (j=0;j<_Ny;j++) {
      iy=iL-_Nx*Ny;
      L_[iL].n[0]=(iy%_Ny)*Nx+iy/_Ny; L_[iL].n[1]=L_[iL].n[0]+Nx;
      buf=iy/_Ny+(iy%_Ny)*_Nx;				/* L_[iL].l[5] */
      if (iy<_Ny) {
	L_[iL].boundary=1;
	L_[iL].l[0]=L_[iL].l[1]=L_[iL].l[2]=NULL;
	L_[iL].l[5]=L_+buf; L_[iL].l[4]=L_+iL+_Ny; L_[iL].l[3]=L_+buf+_Nx;
      }
      else {
	L_[iL].l[0]=L_+buf+_Nx-1; L_[iL].l[1]=L_+iL-_Ny; L_[iL].l[2]=L_+buf-1;
	if (iy >=_Nx*_Ny) {
	  L_[iL].boundary=1; L_[iL].l[5]=L_[iL].l[4]=L_[iL].l[3]=NULL;
	}
	else {
	  L_[iL].boundary=0;
	  L_[iL].l[5]=L_+buf; L_[iL].l[4]=L_+iL+_Ny; L_[iL].l[3]=L_+buf+_Nx;
	}
      }
      iL++;
    }
  }
  
  for (i=0;i<iL;i++) {					/* coordinates of grid nodes */
    L_[i].bx=X[L_[i].n[0]]; L_[i].by=Y[L_[i].n[0]];
    L_[i].ex=X[L_[i].n[1]]; L_[i].ey=Y[L_[i].n[1]];}
  
  NB_=2*(_Nx+_Ny);					/* alloc. mem. for boundary */
  B_=(struct bline*) malloc(NB_*sizeof(struct bline));
  if ((b=B_)==NULL) PSerror("do_DR_Grid in gDRGrid","memory overflow");
  for (i=0;i<_Nx;(b++)->l=L_+i++) ;			/* set lines */
  buf=_Nx*Ny+_Nx*_Ny; for (i=0;i<_Ny;i++) (b++)->l=L_+(buf+i);
  buf=_Nx*Ny-1;       for (i=0;i<_Nx;i++) (b++)->l=L_+(buf-i);
  buf= Nx*Ny-2;       for (i=0;i<_Ny;i++) (b++)->l=L_+(buf-i);
  b=B_;							/* set connections (somewhat  */
  B_->path[1]=B_+(NB_-1); B_->path[0]=B_+1;		/* clumsy to work on RS/6000) */
  for (i=1;i<NB_-1;i++) {(B_+i)->path[1]=B_+(i-1); (B_+i)->path[0]=B_+(i+1);}
  (B_+(NB_-1))->path[1]=B_+(NB_-2); (B_+(NB_-1))->path[0]=B_;
  N_gn=Nx*Ny;						/* # grid nodes */
  
  GT_=0;

  return;
}
