#define COM_SYM_DEF 1 /* all private variable are defined here */
#include "PSgraf_p.h"
#undef COM_SYM_DEF
/*--------------------------------------------------------------------------------------
gPaper: initialize Paper, set default values

HISTORY:
  v3.0.3 debug: save name after closing old file; add version string		041223kr
  v3.0.1 increased version number in output					031115kr
  v3.0.0									031115kr
  give bounding box as (rounded) integers; some previewers cannot handle reals	020519kr
  add bounding box of PostScript file to TeX file      				971221kr
  all coordinates positive							950330kr
  v2.0										920524kr
--------------------------------------------------------------------------------------*/
void gPaper(char *name)
{
  int portrait,i,j,k;
  char fn[255],cbuf[255],c;
  double tx,ty,sx,sy,dx,dy,sxx,syy;
  struct dvec view,right;
  FILE *inttex;

  if (PSgraflibVersion!=PSgrafVersion) {
    fprintf(stderr,
	    "\n\n---------------------------------------------------------------\n");
    fprintf(stderr," different version numbers of PSgraf library and PSgraf3.h \n");
    fprintf(stderr," beware of incompatibilities                               \n");
    k=PSgraflibVersion; i=(int)(k/100); j=(int)((k-i*100)/10); k-=i*100+j*10;
    fprintf(stderr,"      PSgraf library: V%i.%i.%i                               \n",
	    i,j,k);
    k=PSgrafVersion;    i=(int)(k/100); j=(int)((k-i*100)/10); k-=i*100+j*10;
    fprintf(stderr,"      PSgraf.h:       V%i.%i.%i                               \n",
	    i,j,k);
    fprintf(stderr,
	    "---------------------------------------------------------------\n\n");
  }
  
  if (PSdrwN>0) {				/* close old PSout if it exists */
    fprintf(PSout,"grestore\n");
    if (clip_on) sClipping();
    fprintf(PSout,"showpage\n");
    fprintf(PSout,"%%%%Trailer\n");
    fprintf(PSout,"%%%%EOF\n");
    dx=Bxur-Bxll; dy=Byur-Byll; portrait=(dy/dx>1.4142);
    tx=Px/PT_to_MM; ty=Py/PT_to_MM;
    if ((portrait && dy<296*PT_to_MM) || (!portrait && dx<296*PT_to_MM)) {
      dx=105*PT_to_MM-Bxll-dx/2; dy=148*PT_to_MM-Byll-dy/2;
    } else {
      dx=10*PT_to_MM-Bxll; dy=10*PT_to_MM-Byll;
    }
    Bxll+=dx; Bxur+=dx; Byll+=dy; Byur+=dy;
    fseek(PSout,BoundingBox,SEEK_SET);
    Bxll=floor(Bxll); Bxur=ceil(Bxur); Byll=floor(Byll); Byur=ceil(Byur);
    fprintf(PSout,"%%%%BoundingBox: %12.0f %12.0f %12.0f %12.0f\n",Bxll,Byll,Bxur,Byur);
  //  fprintf(PSout,"%%%%BoundingBox: %12.2f %12.2f %12.2f %12.2f\n",Bxll,Byll,Bxur,Byur);
    fseek(PSout,PaperShift,SEEK_SET);
    fprintf(PSout,"%12.5f %12.5f  translate\n",tx+dx,ty+dy);
    fclose(PSout);
    if (EPSencaps) gPreview();
    if (TeXfonts) {				// close intermediary TeXout if it exists
      fclose(TeXout);
                                                // process intermediary TeX output
      sprintf(fn,"%s.tex", paper_name);   TeXout=fopen(fn,"w");
      if (TeXout==NULL) PSerror("gPaper","cannot open TeX file for writing");
      sprintf(fn,"%s.inttex",paper_name); inttex=fopen(fn,"r");
        fprintf(TeXout, "%%Document: %s\n",paper_name);
	fprintf(TeXout,"%%%%Creator: PSgraf %s, copyright Kurt Roth 1992-2004\n",
		PSgraflibVersionString);
        dx=tx+dx-Bxll; dy=ty+dy-Byll;
	fprintf(TeXout,"\\xoff=%12.5fbp\\yoff=%12.5fbp\n",dx,dy);

	while (fscanf(inttex,"%i",&i)!=EOF) {  	// copy text
	  fprintf(TeXout,"\\dlTeXText");
	  while ((c=fgetc(inttex))!='\n') fprintf(TeXout,"%c",c);
	  if (i<0 || i>lay_fill)
	    fprintf(TeXout,"{2}%%\n");
	  else
	    fprintf(TeXout,"{1}%%\n");
	}

        fprintf(TeXout,"\\dlTeXText{%f}{%f}{{}}{0}{0.5}{0.5}{1}{0}{1}%%\n",
		       -dx,-dy);
	fprintf(TeXout,"\\dlTeXText{%f}{%f}{{}}{0}{0.5}{0.5}{1}{0}{1}%%\n",
		       Bxur-Bxll-dx,Byur-Byll-dy);
	fprintf(TeXout,"\\xoff=%12.5fbp\\yoff=%12.5fbp\n",-dx,-dy);
      fclose(inttex); remove(fn); fclose(TeXout);
    }
  }
  PSdrwN++; TeXfonts=0;


                                        //------- start new drawing paper and set defaults
  if (PSinitialized!=1) {
    printf("\n\n---------- entering PSgraf %s, copyright Kurt Roth 1992-2004\n",
	   PSgraflibVersionString);
    PSinitialized=1;
    PSdrwN=1;					/* drawings counter */
    N_Ctr=-1;					/* no ctr */
    L_=NULL;					/* no grid... */
    B_=NULL;					/* ...and thus no boundary */
    EPSencaps=0;				/* no preview added to .eps file */  
  }

  paper_name=name;

  sprintf(fn,"%s.eps",paper_name);		/* name of new PostScript file */
  if ((PSout=fopen(fn,"w"))==NULL) {
    sprintf(cbuf,"cannot write file %s",fn); PSerror("gPaper",cbuf);
  }
  fprintf(PSout,"%%!PS-Adobe-3.0 EPSF-3.0\n");	/* write header */
  fprintf(PSout,"%%%%Title: %s\n",paper_name);
  fprintf(PSout,"%%%%Creator: PSgraf %s, copyright Kurt Roth 1992-2004\n",
	  PSgraflibVersionString);
  fprintf(PSout,"%%%%Pages: 0 1\n");
  BoundingBox=ftell(PSout);			/* save position of bb comment */
  Bxll=10000; Byll=10000; Bxur=-10000; Byur=-10000;/* reset bounding box */  
  fprintf(PSout,"%%%%BoundingBox: %12.0f %12.0f %12.0f %12.0f\n",Bxll,Byll,Bxur,Byur);
  fprintf(PSout,"%%%%EndComments\n");
  fprintf(PSout,"%%%%EndProlog\n");



  fprintf(PSout,"%%-------getPaper\n");
  fprintf(PSout,"gsave\n");

  Px=0; Py=0; Pa=1; Pb=0; Pc=0; Pd=1;      		// Paper Coordinates
  Plx=Ply=Plz=0;
  tx=0; ty=0;
  sx=1; sy=1;
  sxx=sx*PT_to_MM; syy=sy*PT_to_MM;
  fprintf(PSout,"%%--sPCoordinates\n");
  PaperShift=ftell(PSout);				// save position of translate
  fprintf(PSout,"%12.5f %12.5f  translate\n",tx,ty);
  fprintf(PSout,"%12.5f %12.5f  scale\n",sxx,syy);
  Px=tx*PT_to_MM; Py=ty*PT_to_MM; Pa=sxx; Pb=0; Pc=0; Pd=syy;

  sXWorldCoord(0,1,0,1);       				// WorldCoordinates
  sYWorldCoord(0,1,0,1);
  sZWorldCoord(0,1,0,1);

  z_set=0;						// dimensions (turn off 3d)

  clip_on=0;						// clipping

  Pthick=0.1;						// pen
  fprintf(PSout,"%f  setlinewidth\n",Pthick);
  fprintf(PSout,"2  setlinecap\n");

  ColorSpace=0;						// color
  F_c0=NONE; F_c1=NONE; F_c2=NONE; F_c3=NONE;
  S_c0=1;    S_c1=0;    S_c2=0;    S_c3=0; 

  strcpy(curFont,"Helvetica");				// text
  curFSize=10.; hadj='L'; vadj='B';
  TeXfonts=0;
  TextRotation=0;		
  sText(curFont,curFSize,hadj,vadj);
  sprintf(TeXstyle,"{}");
  lay_text=-1; lay_fill=0;

  sNumber('g',4);					// numbers

  xTiMaLe[0]=3.0; yTiMaLe[0]=3.0; zTiMaLe[0]=3.0;	// axis
  xTiMaLe[1]=1.5; yTiMaLe[1]=1.5; zTiMaLe[1]=1.5;
  xTiMaLe[2]=1.0; yTiMaLe[2]=1.0; zTiMaLe[2]=1.0;
  xauto=yauto=zauto=1;

  dp_sym=0;						// data points
  dp_rsym=0.5;
  dp_bar=1;

  arr_lhead=3; arr_wtail=0.4; arr_whead=1.5; arr_tilt=0;// arrows

  below_Pix=0;						// contouring
  Min_Pix=0;

  view.dim=right.dim=3;					// view
  view.x=0; view.y=0; view.z=-1; right.x=1; right.y=0; right.z=0;
  sView(view,right);

  return;
}
