#undef COM_SYM_DEF
#include "PSgraf_p.h"
/*--------------------------------------------------------------------------------------
gRGrid: generate Regular Grid with corner at x0, grid constants x1 and x2, and Ni grid 
        points in i-direction; xi are 2- or 3-dimensional vectors

	grid point ij is located at x_0+i x_1+j x_2

HISTORY:
  v3.0.0, optional 3d input, generate grid on projection plane			031115kr
  v2.0										920524kr
  generate grid vectors and call gDRGrid					920127kr
  created									911001kr
--------------------------------------------------------------------------------------*/
void gRGrid(struct dvec x0,struct dvec x1,struct dvec x2,int N1,int N2)
{
  int ix,iy,i;
  double *x,*y,*z;		/* x-, y- and [z] -coordinates of grid vector */

  if (x0.dim!=2 && x0.dim!=3)
    PSerror("gRGrid","dimension different from 2 or 3");
  if (x0.dim!=x1.dim || x2.dim!=x1.dim) {
    PSerror("gRGrid","dimensions of vectors different");
  }

  if (x0.dim==2) {
    if ((x=(double*) malloc(N1*N2*sizeof(double)))==NULL)
      PSerror("gRGrid","memory overflow");
    if ((y=(double*) malloc(N1*N2*sizeof(double)))==NULL)
      PSerror("gRGrid","memory overflow");
  
    for (iy=0;iy<N2;iy++)
      for (ix=0;ix<N1;ix++) {
	i=ix+iy*N1;
	x[i]=x0.x+ix*x1.x+iy*x2.x; y[i]=x0.y+ix*x1.y+iy*x2.y;
      }
    gDRGrid(x,y,N1,N2);
    free((void*)x); free((void*)y);
  } else {
    if ((x=(double*) malloc(N1*N2*sizeof(double)))==NULL)
      PSerror("gRGrid","memory overflow");
    if ((y=(double*) malloc(N1*N2*sizeof(double)))==NULL)
      PSerror("gRGrid","memory overflow");
    if ((z=(double*) malloc(N1*N2*sizeof(double)))==NULL)
      PSerror("gRGrid","memory overflow");
  
    for (iy=0;iy<N2;iy++)
      for (ix=0;ix<N1;ix++) {
	i=ix+iy*N1;
	x[i]=x0.x+ix*x1.x+iy*x2.x; y[i]=x0.y+ix*x1.y+iy*x2.y; z[i]=x0.z+ix*x1.z+iy*x2.z;
      }
    gDRGrid(x,y,z,N1,N2);
    free((void*)x); free((void*)y); free((void*)z);
  }
  
  return;
}
/*--------------------------------------------------------------------------------------
gRGrid: generate Regular Grid with the lower right corner at (x0,y0), grid constants dx
      and dy,and Nx nodes in the x-direction and Ny in the y-direction
	
History:
  v3.0.0, grid created in Paper Coordinates					031115kr
  v2.0										920524kr
  generate grid vectors and call gDRGrid					920127kr
  created									911001kr
--------------------------------------------------------------------------------------*/
void gRGrid(double x0,double dx,int Nx,double y0,double dy,int Ny)
{
  int ix,iy,i;
  double *x,*y;		/* x- and y-coordinates of grid vector */

  if ((x=(double*) malloc(Nx*Ny*sizeof(double)))==NULL)
    PSerror("gRGrid","memory overflow");
  if ((y=(double*) malloc(Nx*Ny*sizeof(double)))==NULL)
    PSerror("gRGrid","memory overflow");
  
  for (iy=0;iy<Ny;iy++)
    for (ix=0;ix<Nx;ix++) {i=ix+iy*Nx; x[i]=x0+ix*dx; y[i]=y0+iy*dy;}
  gDRGrid(x,y,Nx,Ny);
  free((void*)x); free((void*)y);
  
  return;
}
