#undef COM_SYM_DEF
#include "PSgraf_p.h"
/*--------------------------------------------------------------------------------------
gSRGrid: generate orthogonal Semi-Regular Grid with corner at x0, directions x1 and x2
         and values {x} and {y} in x1- and x2-direction, respectively; notice that {x}
	 and {y} must start with 0 for the first point to coincide with x0
	
History:
  v3.0.0, optional 3d input, generate grid on projection plane			031115kr
--------------------------------------------------------------------------------------*/
void gSRGrid(struct dvec X0,struct dvec X1,double*x1,int N1,
	                    struct dvec X2,double*x2,int N2)
{
  int ix,iy,dim,i;
  double *x,*y,*z;		/* x-, y- and [z] -coordinates of grid vector */

  if (X0.dim!=2 && X0.dim!=3)
    PSerror("gSRGrid","dimension different from 2 or 3");
  if (X0.dim!=X1.dim || X2.dim!=X1.dim) {
    PSerror("gSRGrid","dimensions of vectors different");
  }

  dim=X0.dim;

  if ((x=(double*) malloc(N1*N2*sizeof(double)))==NULL)
    PSerror("gSRGrid", "memory overflow");
  if ((y=(double*) malloc(N1*N2*sizeof(double)))==NULL)
    PSerror("gSRGrid", "memory overflow");

  normalize(&X1); normalize(&X2);

  if (dim==2) {
    for (iy=0; iy<N2; iy++)
      for (ix=0; ix<N1; ix++) {
	i=ix+iy*N1;
	x[i]=X0.x+x1[ix]*X1.x+x2[ix]*X2.x;
	y[i]=X0.y+x1[ix]*X1.y+x2[ix]*X2.y;
      }
    gDRGrid(x,y,N1,N2);
  } else {
    if ((z=(double*) malloc(N1*N2*sizeof(double)))==NULL)
      PSerror("gSRGrid", "memory overflow");
    for (iy=0; iy<N2; iy++)
      for (ix=0; ix<N1; ix++) {
	i=ix+iy*N1;
	x[i]=X0.x+x1[ix]*X1.x+x2[ix]*X2.x;
	y[i]=X0.y+x1[ix]*X1.y+x2[ix]*X2.y;
	z[i]=X0.z+x1[ix]*X1.z+x2[ix]*X2.z;
    }
    gDRGrid(x,y,z,N1,N2);
    free((void*)z);
  }
  free((void*)x); free((void*)y);
  
  return;
}
/*--------------------------------------------------------------------------------------
gSRGrid: generate orthogonal Semi-Regular Grid {x} x {y}, where {x} is the grid vector
	pointing from the origin in x-direction and {y} pointing from the origin in
	y-direction.
	
History:
  v2.0										920524kr
  created									920127kr
--------------------------------------------------------------------------------------*/
void gSRGrid(double *x,int Nx,double *y,int Ny)
{
  int ix, iy, i;
  double *gx, *gy;		/* x- and y-coordinates of grid vector */

  if ((gx=(double *) malloc(Nx*Ny*sizeof(double)))==NULL)
    PSerror("gRGrid", "memory overflow");
  if ((gy=(double *) malloc(Nx*Ny*sizeof(double)))==NULL)
    PSerror("gRGrid", "memory overflow");
  
  for (iy=0; iy<Ny; iy++) for (ix=0; ix<Nx; ix++) {i=ix+iy*Nx; gx[i]=x[ix]; gy[i]=y[iy];}
  gDRGrid(gx, gy, Nx, Ny);
  free(gx); free(gy);
  
  return;
}
