#undef COM_SYM_DEF
#include "PSgraf_p.h"
/*--------------------------------------------------------------------------------------
gTGrid: generate triangular Grid with arbitrary topology;
        NN : # nodes
	x,y: coordinates of nodes
	NL : # lines
	b,e: beginning and end nodes of lines

        Notice: this implementation is not very efficient and only useful for rather
	        small grids with # nodes not exceeding a few hundreds

created										001216kr
--------------------------------------------------------------------------------------*/
void gTGrid(double*x,double*y,int NN,int*b,int*e,int NL)
{
  int *i0,*i1,*ii0,*ii1,n0,n1,np,lmin=0,found,failed=0,i,iL,j,k;
  double minlen,buf,angle0,angle1;
  char cbuf[127];
  struct dvec xw,xp;

  if (L_!=NULL) {
    PSwarning("gTGrid","previously existing grid definition deleted");
    free((void*) L_);}
  if (B_!=NULL) {
    PSwarning("gTGrid","previously existing boundary definition deleted");
    free((void*) B_);}
  
  N_gn=NN; NL_=NL;
  i0  =(int*) malloc(NL_*sizeof(int));			/* alloc. memory */
  if (i0==NULL) PSerror("gTGrid","memory overflow");
  i1  =(int*) malloc(NL_*sizeof(int));
  if (i1==NULL) PSerror("gTGrid","memory overflow");
  ii0 =(int*) malloc(NL_*sizeof(int));
  if (ii0==NULL) PSerror("gTGrid","memory overflow");
  ii1 =(int*) malloc(NL_*sizeof(int));
  if (ii1==NULL) PSerror("gTGrid","memory overflow");
  L_  =(struct line*) malloc(NL_*sizeof(struct line));
  if (L_==NULL) PSerror("gTGrid","memory overflow");

  for (iL=0;iL<NL_;iL++) {L_[iL].id=iL; L_[iL].n[0]=b[iL]; L_[iL].n[1]=e[iL];}

  for (iL=0;iL<NL_;iL++) {				/* topological def of lines */
    n0=0;				/* find all lines connected to L_[iL].n[0] */
    for (i=0;i<NL_;i++)
      if (i!=iL)
	if (L_[i].n[0]==L_[iL].n[0] || L_[i].n[1]==L_[iL].n[0]) {i0[n0]=i; n0++;}
    n1=0;				/* find all lines connected to L_[iL].n[1] */
    for (i=0;i<NL_;i++)
      if (i!=iL)
	if (L_[i].n[0]==L_[iL].n[1] || L_[i].n[1]==L_[iL].n[1]) {i1[n1]=i; n1++;}

    np=0; minlen=DBL_MAX;	       	/* find two-line paths L_[iL].n[0]-->L_[iL].n[1]*/
    for (i=0;i<n0;i++)
      for (j=0;j<n1;j++) {
	if (L_[i0[i]].n[0]==L_[i1[j]].n[0] || L_[i0[i]].n[1]==L_[i1[j]].n[0] ||
	    L_[i0[i]].n[0]==L_[i1[j]].n[1] || L_[i0[i]].n[1]==L_[i1[j]].n[1]) {
	  ii0[np]=i0[i]; ii1[np]=i1[j];
	  buf=sqrt(pow(x[L_[ii0[np]].n[1]]-x[L_[ii0[np]].n[0]],2)+
		   pow(y[L_[ii0[np]].n[1]]-y[L_[ii0[np]].n[0]],2))+
	    sqrt(pow(x[L_[ii1[np]].n[1]]-x[L_[ii1[np]].n[0]],2)+
		 pow(y[L_[ii1[np]].n[1]]-y[L_[ii1[np]].n[0]],2));
	  if (buf<minlen) {minlen=buf; lmin=np;}
	  np++;
	}
      }
    if (np==0) {
      sprintf(cbuf,"lone line %i detected",iL);
      PSwarning("gTGrid",cbuf);
      failed=1;
    }
    if (np==1) {			/* boundary line */
      L_[iL].l[0]=L_+ii0[0]; L_[iL].l[1]=L_+ii1[0];
      L_[iL].l[2]=NULL;      L_[iL].l[3]=NULL;
      L_[iL].boundary=-1;
    }
    if (np==2) {			/* simple regular line */
      L_[iL].l[0]=L_+ii0[0]; L_[iL].l[1]=L_+ii1[0];
      L_[iL].l[2]=L_+ii0[1]; L_[iL].l[3]=L_+ii1[1];
      L_[iL].boundary=0;
    }
    if (np>2) {				/* multiply connected regular line */
      if (lmin!=0) {
	ii0[NL_-1]=ii0[lmin]; ii0[lmin]=ii0[0]; ii0[0]=ii0[NL_-1];
	ii1[NL_-1]=ii1[lmin]; ii1[lmin]=ii1[0]; ii1[0]=ii1[NL_-1];
      }
      L_[iL].l[0]=L_+ii0[0]; L_[iL].l[1]=L_+ii1[0];
      angle0=
	(x[L_[iL].n[1]]-x[L_[iL].n[0]])*(x[L_[ii0[0]].n[1]]-x[L_[ii0[0]].n[0]])+
	(y[L_[iL].n[1]]-y[L_[iL].n[0]])*(y[L_[ii0[0]].n[1]]-y[L_[ii0[0]].n[0]]);
      if (L_[iL].n[0]==L_[ii0[0]].n[0]) angle0=-angle0;
      minlen=DBL_MAX;
      for (i=1;i<np;i++) {
	angle1=
	  (x[L_[iL].n[1]]-x[L_[iL].n[0]])*(x[L_[ii0[1]].n[1]]-x[L_[ii0[1]].n[0]])+
	  (y[L_[iL].n[1]]-y[L_[iL].n[0]])*(y[L_[ii0[1]].n[1]]-y[L_[ii0[1]].n[0]]);
	if (L_[iL].n[0]==L_[ii0[1]].n[0]) angle1=-angle1;
	if (angle0*angle1<0) {
	  buf=sqrt(pow(x[L_[ii0[i]].n[1]]-x[L_[ii0[i]].n[0]],2)+
		   pow(y[L_[ii0[i]].n[1]]-y[L_[ii0[i]].n[0]],2))+
	    sqrt(pow(x[L_[ii1[i]].n[1]]-x[L_[ii1[i]].n[0]],2)+
		 pow(y[L_[ii1[i]].n[1]]-y[L_[ii1[i]].n[0]],2));
	  if (buf<minlen) {minlen=buf; lmin=i;}
	}
      }
      L_[iL].l[2]=L_+ii0[lmin]; L_[iL].l[3]=L_+ii1[lmin];
      L_[iL].boundary=0;
    }
  }
  xw.dim=2;		       			       	/* metric: coord of grid nodes */
  for (iL=0;iL<NL_;iL++) {
    xw.x=x[L_[iL].n[0]]; xw.y=y[L_[iL].n[0]];
    pWP(xw,&xp); L_[iL].bx=xp.x; L_[iL].by=xp.y;
    xw.x=x[L_[iL].n[1]]; xw.y=y[L_[iL].n[1]];
    pWP(xw,&xp); L_[iL].ex=xp.x; L_[iL].ey=xp.y;
  }



                                                        /* topology of boundary */
  for (n0=0;n0<NL_;n0++) if (L_[n0].boundary<0) break;
  i0[0]=n0; L_[n0].boundary=1;	/* n0: first boundary element */
  k=1;				/* counter on boundary */
  for (;;) {
    found=0;
    for (iL=n0+1;iL<NL_;iL++) {	/* get next boundary element (no lines below n0) */
      if (L_[iL].boundary<0 &&	/* continue to follow boundary */
	  (L_[iL].n[0]==L_[i0[k-1]].n[0] || L_[iL].n[1]==L_[i0[k-1]].n[0] ||
	   L_[iL].n[0]==L_[i0[k-1]].n[1] || L_[iL].n[1]==L_[i0[k-1]].n[1])) {
	i0[k]=iL; L_[iL].boundary=1; k++; iL=n0+1; found=1;
      }
    }
    if (!found) break;
  }
  NB_=k;
  B_  =(struct bline*) malloc(NB_*sizeof(struct bline));
  if (B_==NULL) PSerror("gTGrid","memory overflow");
  for (k=0;k<NB_;k++) B_[k].l=&L_[i0[k]];
  B_->path[0]=B_+1; B_->path[1]=B_+(NB_-1);
  for (j=1;j<NB_-1;j++) {(B_+j)->path[0]=B_+(j+1); (B_+j)->path[1]=B_+(j-1);}
  (B_+(NB_-1))->path[0]=B_; (B_+(NB_-1))->path[1]=B_+(NB_-2);

  free((void*) i0);  free((void*) i1);			/* tidy up */
  free((void*) ii0); free((void*) ii1);

  GT_=1;

  if (failed) PSerror("gTGrid","input file inconsistent");

  return;
}
#undef COM_SYM_DEF
