#undef COM_SYM_DEF
#include "PSgraf_p.h"
/*--------------------------------------------------------------------------------------
one-way projectors are implemented for
    W -> P: pWP
    W -> I: pWI
    P -> I: pPI
where W are World Coordinates, P Paper Coordinates, and I Image Plane.
Notice that pWP preserves the dimension. pWI and pPI always result in a 2d point.
--------------------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------------------
pWP: project world point Xw to paper point Xp

v3.0.0										031115kr
--------------------------------------------------------------------------------------*/
void pWP(struct dvec Xw,struct dvec *Xp)
{
  if (Xw.dim==2) {					       	// W2 -> P2
    (*Xp).dim=2;
    pWP(Xw.x,Xw.y,&((*Xp).x),&((*Xp).y));
    return;
  }

  if (Xw.dim==3) {					       	// W3 -> P3
    (*Xp).dim=3;
    pWP(Xw.x,Xw.y,Xw.z,&((*Xp).x),&((*Xp).y),&((*Xp).z));
    return;
  }

  PSerror("pWP","dimension of vector must be 2 or 3");
}
/*------------------------------------------------------------------------------------*/
struct dvec pWP(struct dvec Xw)
{
  struct dvec Xp;

  Xp.dim=Xw.dim;
  pWP(Xw,&Xp);
  return Xp;
}
/*------------------------------------------------------------------------------------*/
void pWP(double xw,double yw,double*xp,double*yp)
{
  *xp=Plx+(xw-Wlx)*PWx;
  *yp=Ply+(yw-Wly)*PWy;

  return;
}
/*------------------------------------------------------------------------------------*/
void pWP(double xw,double yw,double zw,double*xp,double*yp,double*zp)
{
  *xp=Plx+(xw-Wlx)*PWx;
  *yp=Ply+(yw-Wly)*PWy;
  *zp=Plz+(zw-Wlz)*PWz;

  return;
}


/*--------------------------------------------------------------------------------------
pPW: project paper point Xp to world point Xw

v3.0.1										040623kr
--------------------------------------------------------------------------------------*/
void pPW(struct dvec Xp,struct dvec *Xw)
{
  if (Xp.dim==2) {					       	// P2 -> W2
    (*Xw).dim=2;
    pPW(Xp.x,Xp.y,&((*Xw).x),&((*Xw).y));
    return;
  }

  if (Xp.dim==3) {					       	// P3 -> W3
    (*Xw).dim=3;
    pPW(Xp.x,Xp.y,Xp.z,&((*Xw).x),&((*Xw).y),&((*Xw).z));
    return;
  }

  PSerror("pPW","dimension of vector must be 2 or 3");
}
/*------------------------------------------------------------------------------------*/
struct dvec pPW(struct dvec Xp)
{
  struct dvec Xw;

  Xw.dim=Xp.dim;
  pPW(Xp,&Xw);
  return Xw;
}
/*------------------------------------------------------------------------------------*/
void pPW(double xp,double yp,double*xw,double*yw)
{
  *xw=(-Plx+PWx*Wlx+xp)/PWx;
  *yw=(-Ply+PWy*Wly+yp)/PWy;

  return;
}
/*------------------------------------------------------------------------------------*/
void pPW(double xp,double yp,double zp,double*xw,double*yw,double*zw)
{
  *xw=(-Plx+PWx*Wlx+xp)/PWx;
  *yw=(-Ply+PWy*Wly+yp)/PWy;
  *zw=(-Plz+PWz*Wlz+zp)/PWz;

  return;
}


/*--------------------------------------------------------------------------------------
pWI: project world point Xw to image point Xi

v3.0.0										031115kr
--------------------------------------------------------------------------------------*/
void pWI(struct dvec Xw,struct dvec *Xi)
{
  struct dvec Xp;

  pWP(Xw,&Xp);
  pPI(Xp,Xi);
  return;
}
/*------------------------------------------------------------------------------------*/
struct dvec pWI(struct dvec Xw)
{
  struct dvec Xi;

  pWP(Xw,&Xi);
  pPI(Xi,&Xi);
  return Xi;
}
/*------------------------------------------------------------------------------------*/
void pWI(double xw,double yw,double zw,double*xi,double*yi)
{
  pWP(xw,yw,zw,&xw,&yw,&zw);
  pPI(xw,yw,zw,xi,yi);
  return;
}


/*--------------------------------------------------------------------------------------
pPI: project paper point Xp to image point Xi
          Xi.x=[\bX_w-\bX_0]\cdot\bX_1
	  Xi.y=[\bX_w-\bX_0]\cdot\bX_2

v3.0.0										031115kr
--------------------------------------------------------------------------------------*/
void pPI(struct dvec Xp,struct dvec *Xi)
{
  (*Xi).dim=2;

  if (Xp.dim==2) {					       	// P2 -> P3
    Xp.dim=3; Xp.z=0;
  }

  if (Xp.dim==3) {					       	// P3 -> I
    pPI(Xp.x,Xp.y,Xp.z,&((*Xi).x),&((*Xi).y));
    return;
  }

  PSerror("pPI","dimension of vector must be 2 or 3");
}
/*--------------------------------------------------------------------------------------
pPI: Xp -> Xi
--------------------------------------------------------------------------------------*/
struct dvec pPI(struct dvec Xp)
{
  struct dvec Xi;

  pPI(Xp,&Xi);
  return Xi;
}
/*--------------------------------------------------------------------------------------
pPI: (xp,yp,zp) -> (xi,yi)
--------------------------------------------------------------------------------------*/
void pPI(double xp,double yp,double zp,double*xi,double*yi)
{
  *xi=(xp-R32x0)*R32x1+(yp-R32y0)*R32y1+(zp-R32z0)*R32z1;
  *yi=(xp-R32x0)*R32x2+(yp-R32y0)*R32y2+(zp-R32z0)*R32z2;

  return;
}
