#undef COM_SYM_DEF
#include "PSgraf_p.h"
/*--------------------------------------------------------------------------------------
sContours: set contour values and gray/color levels

HISTORY:
  v3.0.0       									031122kr
  check ordering of ctr								950618kr
  variable argument list       							920819kr
  v2.0										920524kr
--------------------------------------------------------------------------------------*/
void sContours(double*ctr,int N,struct color*c)
{
  int i,j;

  if (ColorSpace!=(c[0]).CS) {
    PSerror("sContours(double*,int,struct color*)","wrong color space");
  }
  
  for (i=0;i<N;i++)
    if ((c[i]).CS!=(c[N]).CS)
      PSerror("sContours",
	      "colors given in different space");

  C_ctr=ctr; N_Ctr=N;
  i=0;							/* check if ctr is ordered */
  for (j=1;j<N;j++) if (ctr[i]<ctr[j]) {i++; ctr[i]=ctr[j];}
  if (i+1<j) {
    PSwarning("sContours",
              "Contour values must be strictly ordered:");
    N_Ctr=i+1;
    fprintf(stderr,"        only the following values have been used:\n");
    for (i=0;i<N_Ctr;i++) fprintf(stderr,"%10i %g\n",i,ctr[i]);
  }

  C_col=c; C_col++;

  return;
}
/*--------------------------------------------------------------------------------------
sContours: set contour values and gray/color levels

HISTORY:
  v3.0.0       									031122kr
  check ordering of ctr								950618kr
  variable argument list       							920819kr
  v2.0										920524kr
--------------------------------------------------------------------------------------*/
void sContours(double *ctr,int N,double*c0)
{
  int i;
  struct color *col;
  
  if (ColorSpace!=0) {
    PSerror("sContours(double*,int,double*)","only useable for G color space");
    return;
  }

  col=(struct color*)malloc((N+1)*sizeof(struct color));
  if (col==NULL) PSerror("sContours","memory overflow");

  for (i=0;i<=N;i++) {
    col[i].CS=0;
    col[i].c0=c0[i]; col[i].c1=col[i].c2=col[i].c3=0;
  }

  sContours(ctr,N,col);

  return;
}
/*--------------------------------------------------------------------------------------
sContours: set contour values and gray/color levels
--------------------------------------------------------------------------------------*/
void sContours(double *ctr,int N,double*c0,double*c1,double*c2)
{
  int i;
  struct color *col;
  
  if (ColorSpace!=2 && ColorSpace!=4) {
    PSerror("sContours(double*,int,double*,double*,double*)",
	    "only useable for RGB or HSB color space");
    return;
  }

  col=(struct color*)malloc((N+1)*sizeof(struct color));
  if (col==NULL) PSerror("sContours","memory overflow");

  for (i=0;i<=N;i++) {
    col[i].CS=ColorSpace;
    col[i].c0=c0[i]; col[i].c1=c1[i]; col[i].c2=c2[i]; col[i].c3=0;
  }

  sContours(ctr,N,col);

  return;
}
/*--------------------------------------------------------------------------------------
sContours: set contour values and gray/color levels
--------------------------------------------------------------------------------------*/
void sContours(double *ctr,int N,double*c0,double*c1,double*c2,double*c3)
{
  int i;
  struct color *col;
  
  if (ColorSpace!=8) {
    PSerror("sContours(double*,int,double*,double*,double*,double*)",
	    "only useable for CMYK color space");
    return;
  }

  col=(struct color*)malloc((N+1)*sizeof(struct color));
  if (col==NULL) PSerror("sContours","memory overflow");

  for (i=0;i<=N;i++) {
    col[i].CS=8;
    col[i].c0=c0[i]; col[i].c1=c1[i]; col[i].c2=c2[i]; col[i].c3=c3[i];
  }

  sContours(ctr,N,col);

  return;
}
