#undef COM_SYM_DEF
#include "PSgraf_p.h"
/*--------------------------------------------------------------------------------------
sDensity: set color transfer for density plot;
	notice that C_...[N] indicates color of missing values

HISTORY:
  v3.0.0									031222kr
  created      									971128kr
--------------------------------------------------------------------------------------*/
void sDensity(double *val,int N,struct color*c)
{
  int i,j;

  if (ColorSpace!=c[0].CS) {
    PSerror("sDensity(double*,int,struct color*)","wrong color space");
  }
  
  for (i=0;i<N;i++)
    if (c[i].CS!=c[N].CS)
      PSerror("sDensity(double*,int,struct color*)",
		"colors given in different space");

  C_ctr=val; N_Ctr=N;
  i=0;							/* check if val is ordered */
  for (j=1;j<N;j++) if (val[i]<val[j]) {i++; val[i]=val[j];}
  if (i+1<j) {
    PSwarning("sDensity",
              "color transfer values must be strictly ordered:");
    N_Ctr=i+1;
    fprintf(stderr,"        only the following values have been used:\n");
    for (i=0;i<N_Ctr;i++) fprintf(stderr,"%10i %g\n",i,val[i]);
  }

  C_col=c;

  return;
}
/*------------------------------------------------------------------------------------*/
void sDensity(double *val,int N,double*c0)
{
  int i;
  struct color *col;
  
  if (ColorSpace!=0) {
    PSerror("sDensity(double*,int,double*)","only useable for G color space");
    return;
  }

  col=(struct color*)malloc((N+1)*sizeof(struct color));
  if (col==NULL) PSerror("sDensity","memory overflow");

  for (i=0;i<=N;i++) {
    col[i].CS=0;
    col[i].c0=c0[i]; col[i].c1=col[i].c2=col[i].c3=0;
  }

  sDensity(val,N,col);

  return;
}
/*------------------------------------------------------------------------------------*/
void sDensity(double *val,int N,double*c0,double*c1,double*c2)
{
  int i;
  struct color *col;
  
  if (ColorSpace!=2 && ColorSpace!=4) {
    PSerror("sDensity(double*,int,double*,double*,double*)",
	    "only useable for RGB or HSB color space");
    return;
  }

  col=(struct color*)malloc((N+1)*sizeof(struct color));
  if (col==NULL) PSerror("sDensity","memory overflow");

  for (i=0;i<=N;i++) {
    col[i].CS=ColorSpace;
    col[i].c0=c0[i]; col[i].c1=c1[i]; col[i].c2=c2[i]; col[i].c3=0;
  }

  sDensity(val,N,col);

  return;
}
/*------------------------------------------------------------------------------------*/
void sDensity(double *val,int N,double*c0,double*c1,double*c2,double*c3)
{
  int i;
  struct color *col;
  
  if (ColorSpace!=8) {
    PSerror("sDensity(double*,int,double*,double*,double*,double*)",
	    "only useable for CMYK color space");
    return;
  }

  col=(struct color*)malloc((N+1)*sizeof(struct color));
  if (col==NULL) PSerror("sDensity","memory overflow");

  for (i=0;i<=N;i++) {
    col[i].CS=8;
    col[i].c0=c0[i]; col[i].c1=c1[i]; col[i].c2=c2[i]; col[i].c3=c3[i];
  }

  sDensity(val,N,col);

  return;
}
