#undef COM_SYM_DEF
#include "PSgraf_p.h"
/*--------------------------------------------------------------------------------------
sIPlane: set image plane by defining
       1. image plane by vectors x'=(x1,y1,z1) and y'=(x2,y2,z2) in normal space
       2. origin (x0,y0,z0) of plane

v3.0.0										031115kr
--------------------------------------------------------------------------------------*/
void sIPlane(struct dvec x0,struct dvec x1,struct dvec x2)
{
  double buf;

  if (norm(x1)<1e-6)
    PSerror("sIPlane","vector x1 too short for defining image plane");
  else normalize(&x1);

  if (norm(x2)<1e-6)
    PSerror("sIPlane","vector x2 too short for defining image plane");
  else normalize(&x2);

  buf=dotprod(x1,x2);
  if (buf>0.99999)
    PSerror("sIPlane","angle between x1 and x2 too small for defining image plane");

  if (buf>1e-6)
    PSwarning("sIPlane","x1 and x2 not orthogonal; image will be distorted");

  R32x0=x0.x; R32y0=x0.y; R32z0=x0.z;
  R32x1=x1.x; R32y1=x1.y; R32z1=x1.z;
  R32x2=x2.x; R32y2=x2.y; R32z2=x2.z;

  R32_view=normalize(vecprod(x2,x1));

  return;
}
/*--------------------------------------------------------------------------------------
sIPlane: set image plane by defining
       1. image plane by vectors x'=(x1,y1,z1) and y'=(x2,y2,z2) in normal space
       2. origin (0,0,0) of plane

v3.0.0										031115kr
--------------------------------------------------------------------------------------*/
void sIPlane(struct dvec x1,struct dvec x2)
{
  struct dvec x0;

  x0.dim=3; x0.x=x0.y=x0.z=0;
  sIPlane(x0,x1,x2);

  return;
}
