#undef COM_SYM_DEF
#include "PSgraf_p.h"
/*--------------------------------------------------------------------------------------
sStroke: set stroke color

HISTORY:
  v3.0.0       									031129kr
---------------------------------------------------------------------------------------*/
void sStroke(struct color c)
{
  if (ColorSpace!=c.CS)			// set color space implicitly
    if (sColorSpace(c.CS)) {
      PSwarning("sStroke","color space not implemented; operator ignored");
      return;
    } else
      PSwarning("sStroke","color space changed implicitly");

  switch (c.CS) {
    case 0:
      S_c0=c.c0; S_c1=S_c2=S_c3=0;
      fprintf(PSout, "%%-- sStroke (G)\n");
      fprintf(PSout, "%f  setgray\n", 1-c.c0);
      return;
    case 2:
      S_c0=c.c0; S_c1=c.c1; S_c2=c.c2; S_c3=0;
      fprintf(PSout, "%%-- sStroke (RGB)\n");
      fprintf(PSout, "%f %f %f  setrgbcolor\n",c.c0,c.c1,c.c2);
      return;
    case 4:
      S_c0=c.c0; S_c1=c.c1; S_c2=c.c2; S_c3=0;
      fprintf(PSout, "%%-- sStroke (HSB)\n");
      fprintf(PSout, "%f %f %f  sethsbcolor\n",c.c0,c.c1,c.c2);
      return;
    case 8:
      S_c0=c.c0; S_c1=c.c1; S_c2=c.c2; S_c3=c.c3;
      fprintf(PSout, "%%-- sStroke (CMYK)\n");
      fprintf(PSout, "%f %f %f %f  setcmykcolor\n",c.c0,c.c1,c.c2,c.c3);
      return;
  }

  return;
}
/*--------------------------------------------------------------------------------------
gStroke: get actual stroke color

HISTORY:
  v3.0.0       									031129kr
---------------------------------------------------------------------------------------*/
struct color gStroke()
{
  struct color c;

  c.CS=ColorSpace;
  c.c0=S_c0; c.c1=S_c1; c.c2=S_c2; c.c3=S_c3;
  
  return c;
}
/*--------------------------------------------------------------------------------------
sStroke: set gray level for stroke

HISTORY:
  v3.0.0       									031122kr
  bugfix       									021128kr
  variable argument lists      							920819kr
  v2.0										920524kr
---------------------------------------------------------------------------------------*/
void sStroke(double c0)
{
  struct color c;

  if (ColorSpace!=0) {
    PSwarning("sStroke(double)","only useable for G color space; operator ignored");
    return;
  }

  c.CS=0; c.c0=c0;
  sStroke(c);

  return;
}
/*--------------------------------------------------------------------------------------
sStroke: set color components in RGB or HSB ColorSpace for stroke
---------------------------------------------------------------------------------------*/
void sStroke(double c0,double c1,double c2)
{
  struct color c;

  if (ColorSpace==2) {
    c.CS=2; c.c0=c0; c.c1=c1; c.c2=c2;
    sStroke(c);
    return;
  }

  if (ColorSpace==4) {
    c.CS=4; c.c0=c0; c.c1=c1; c.c2=c2;
    sStroke(c);
    return;
  }

  PSwarning("sStroke(double,double,double)",
	    "only useable for RGB or HSB color space; operator ignored");
  return;
}
/*--------------------------------------------------------------------------------------
sStroke: set color components in CMYK ColorSpace for stroke
---------------------------------------------------------------------------------------*/
void sStroke(double c0,double c1,double c2,double c3)
{
  struct color c;

  if (ColorSpace!=8) {
    PSwarning("sStroke(double,double,double,double)",
	      "only useable for CMYK color space; operator ignored");
    return;
  }
  
  c.CS=8; c.c0=c0; c.c1=c1; c.c2=c2; c.c3=c3;
  sStroke(c);

  return;
}
