#undef COM_SYM_DEF
#include "PSgraf_p.h"
/*--------------------------------------------------------------------------------------
identity: return x
	
History:
  v3.0.0									031122kr
--------------------------------------------------------------------------------------*/
struct dvec identity(struct dvec x)
{

  if (x.dim==2) {
    x.x=x.x; x.y=x.y;
    return x;
  }
  if (x.dim==3) {
    x.x=x.x; x.y=x.y; x.z=x.z;
    return x;
  }

  PSerror("identity","dimension different from 2 or 3");
  exit(0);
}
/*--------------------------------------------------------------------------------------
negvec: return -x
	
History:
  v3.0.0									031122kr
--------------------------------------------------------------------------------------*/
struct dvec negvec(struct dvec x)
{

  if (x.dim==2) {
    x.x=-x.x; x.y=-x.y;
    return x;
  }
  if (x.dim==3) {
    x.x=-x.x; x.y=-x.y; x.z=-x.z;
    return x;
  }

  PSerror("negvec","dimension different from 2 or 3");
  exit(0);
}
/*--------------------------------------------------------------------------------------
norm: return norm of vector x
	
History:
  v3.0.0									031122kr
--------------------------------------------------------------------------------------*/
double norm(struct dvec x)
{
  if (x.dim==2) return sqrt(pow(x.x,2)+pow(x.y,2));
  if (x.dim==3) return sqrt(pow(x.x,2)+pow(x.y,2)+pow(x.z,2));

  PSerror("normalize","dimension different from 2 or 3");
  exit(0);
}
/*--------------------------------------------------------------------------------------
normalize: normalize vector x
	
History:
  v3.0.0									031115kr
--------------------------------------------------------------------------------------*/
void normalize(struct dvec *x)
{
  double n;
  
  n=norm(*x);

  if (n==0) PSerror("normalize","cannot normalize zero-vector");

  if ((*x).dim==2) {
    (*x).x/=n; (*x).y/=n;
  } else {
    (*x).x/=n; (*x).y/=n; (*x).z/=n;
  }

  return;
}
/*--------------------------------------------------------------------------------------
normalize: normalize vector x
--------------------------------------------------------------------------------------*/
struct dvec normalize(struct dvec x)
{
  double n;
  struct dvec y;  
  
  if ((n=norm(x))==0) PSerror("normalize","cannot normalize zero-vector");

  if (x.dim==2) {			// dim==2 || dim==3 already checked by norm
    y.dim=2;
    y.x=x.x/n; y.y=x.y/n;
  } else {
    y.dim=3;
    y.x=x.x/n; y.y=x.y/n; y.z=x.z/n;
  }

  return y;
}
/*--------------------------------------------------------------------------------------
addvec: c = a + b
	
History:
  v3.0.0									031122kr
--------------------------------------------------------------------------------------*/
struct dvec addvec(struct dvec a,struct dvec b)
{
  struct dvec c;

  if (a.dim!=b.dim) {
    PSerror("addvec","dimensions of vectors a and b different");
  }

  if (a.dim==2) {
    c.dim=2; c.x=a.x+b.x; c.y=a.y+b.y;
    return c;
  }
  if (a.dim==3) {
    c.dim=3; c.x=a.x+b.x; c.y=a.y+b.y; c.z=a.z+b.z;
    return c;
  }

  PSerror("addvec","dimension different from 2 or 3");
  exit(0);
}
/*--------------------------------------------------------------------------------------
subvec: c = a - b
	
History:
  v3.0.0									031122kr
--------------------------------------------------------------------------------------*/
struct dvec subvec(struct dvec a,struct dvec b)
{
  struct dvec c;

  if (a.dim!=b.dim) {
    PSerror("subvec","dimensions of vectors a and b different");
  }

  if (a.dim==2) {
    c.dim=2; c.x=a.x-b.x; c.y=a.y-b.y;
    return c;
  }
  if (a.dim==3) {
    c.dim=3; c.x=a.x-b.x; c.y=a.y-b.y; c.z=a.z-b.z;
    return c;
  }

  PSerror("subvec","dimension different from 2 or 3");
  exit(0);
}
/*--------------------------------------------------------------------------------------
scalmult: scalar multiplication c = a b, where a is a scalar and b a vector
--------------------------------------------------------------------------------------*/
struct dvec scalmult(double a,struct dvec b)
{
  struct dvec c;

  if (b.dim==2) {
    c.dim=2;
    c.x=a*b.x; c.y=a*b.y;
  } else {
    c.dim=3;
    c.x=a*b.x; c.y=a*b.y; c.z=a*b.z;
  }

  return c;
}
/*--------------------------------------------------------------------------------------
dotprod: dot product c = a . b
	
History:
  v3.0.0									031122kr
--------------------------------------------------------------------------------------*/
double dotprod(struct dvec a,struct dvec b)
{
  if (a.dim!=b.dim) {
    printf("%i %f %f\t %i %f %f\n",a.dim,a.x,a.y,b.dim,b.x,b.y);
    PSerror("dotprod","dimensions of vectors a and b different");
  }

  if (a.dim==2) return a.x*b.x+a.y*b.y;
  if (a.dim==3) return a.x*b.x+a.y*b.y+a.z*b.z;

  PSerror("dotprod","dimension different from 2 or 3");
  exit(0);
}
/*--------------------------------------------------------------------------------------
vecprod: vector product c = a x b
	
History:
  v3.0.0									031122kr
--------------------------------------------------------------------------------------*/
void vecprod(struct dvec a,struct dvec b,struct dvec *c)
{
  if (a.dim!=b.dim) {
    PSerror("vecprod","dimensions of vectors a and b different");
  }
  if (a.dim!=2 && a.dim!=3)
    PSerror("vecprod","dimension different from 2 or 3");

  (*c).dim=3;
  if (a.dim==2) {
    (*c).x=0;		    (*c).y=0;		    (*c).z=a.x*b.y-a.y*b.x;
  } else {
    (*c).x=a.y*b.z-a.z*b.y; (*c).y=a.z*b.x-a.x*b.z; (*c).z=a.x*b.y-a.y*b.x;
  }

  return;
}
/*--------------------------------------------------------------------------------------
vecprod: vector product c = a x b
--------------------------------------------------------------------------------------*/
struct dvec vecprod(struct dvec a,struct dvec b)
{
  struct dvec c;

  if (a.dim!=b.dim) {
    PSerror("vecprod","dimensions of vectors a and b different");
  }
  if (a.dim!=2 && a.dim!=3)
    PSerror("vecprod","dimension different from 2 or 3");

  c.dim=3;
  if (a.dim==2) {
    c.x=0;		 c.y=0;		      c.z=a.x*b.y-a.y*b.x;
  } else {
    c.x=a.y*b.z-a.z*b.y; c.y=a.z*b.x-a.x*b.z; c.z=a.x*b.y-a.y*b.x;
  }

  return c;
}
