/********************************************************
  g++ -g -o _demo demo.c  -lm -ltiff -lQuantim4 -lPSgraf3
********************************************************/


#include <quantim4.h>
#include "string.h"


main(int arg_c,char *arg_v[])
{
  char buffer[127],outfile[127];
  image_cc  *image, *copy, *grey, *seg, *plane, *bin, *erod, *opened, *distance, *diff;
  double vv, sv, epc1, epc2, *cdf, **acov, corl;
  long *h;
  int i, n;

for(n=0;n<5;n++){
//for(n=4;n<5;n++){

	switch(n){
	/**** load image *********/
        case 0: sprintf(buffer,"./images/soil"); //2D grey
		image = LoadImage(buffer);
		break;
        case 1: sprintf(buffer,"./images/hele-rgb"); //rgb with alpha channel
		image = LoadImage(buffer);
		break;
        case 2: sprintf(buffer,"./images/CTmed");
		image = LoadImage(buffer);
		break; //ddd image
        
	case 3: image=GetCorImage(cdf,128,128,1,1,10,20,3,acov); //2d grey random fields **	 
		sprintf(buffer,"./images/");
		sprintf(outfile,"%srandfield",buffer);
		StoreImage(image,outfile);
		break;
	case 4: sprintf(buffer,"./images/grey16bit");
		image = LoadImage(buffer);
		break;
	default: exit(0);
	}

	for(i=0;i<image->ndim;i++)
		image->res[i]=1.0/image->dim[0];

	copy=CopyImage(image);
	sprintf(outfile,"%s_copy",buffer);
	StoreImage(copy,outfile);


//exit(0);


	if(strcmp(image->itype,"ddd")==0){

		plane = GetPlane(image, 20,0);
		sprintf(outfile,"%s_plane",buffer);
		StoreImage(plane,outfile);
		DeleteImage(plane);

		cdf=GetDddAcov(image, &corl, 20, 1);
		printf("\n correlation length is: %lf\n",corl);

		//Mean(image, 2);
		bin=DddBin(image,120);
		plane = GetPlane(bin, 20,0);
		sprintf(outfile,"%s_plane_bin",buffer);
		StoreImage(plane,outfile);
		DeleteImage(plane);

		copy=CopyImage(bin);
		seg=ImageSegment(copy, 20, 20,0, 100, 100,30);

		erod=BtdErodeMir(seg,1,0);
		plane = GetPlane(erod, 20,0);
		sprintf(outfile,"%s_plane_eroded",buffer);
		StoreImage(plane,outfile);
		DeleteImage(plane);

		opened=BtdErodeMir(erod,1,1);
		plane = GetPlane(opened, 20,0);
		sprintf(outfile,"%s_plane_opened",buffer);
		StoreImage(plane,outfile);
		DeleteImage(plane);

		diff=Diff(erod,opened);
		plane = GetPlane(diff, 20,0);
		sprintf(outfile,"%s_plane_diff",buffer);
		StoreImage(plane,outfile);
		DeleteImage(plane);

	}



	if(strcmp(image->itype,"rgb")==0){
		grey=RGBtoGray(image);
		sprintf(outfile,"%s_grey",buffer);
		StoreImage(grey,outfile);

		InvertImage(grey);
		sprintf(outfile,"%s_grey_inverted",buffer);
		StoreImage(grey,outfile);
	}

	if(n<2||n==4){
		copy=CopyImage(image);
		sprintf(outfile,"%s_copy",buffer);
		StoreImage(copy,outfile);
		DeleteImage(copy);

		//seg=InitImage(100,100, 15000, image->itype);
		seg=ImageSegment(image, 20, 20, 100, 100);
		sprintf(outfile,"%s_segment",buffer);
		StoreImage(seg,outfile);

		if(strcmp(image->itype,"gry16")==0){
			copy=gry16to8(image);
			sprintf(outfile,"%s_8bit",buffer);
			StoreImage(copy,outfile);

			gBin(30200,0,65000,image);
			sprintf(outfile,"%s_bin",buffer);
			StoreImage(image,outfile);
		}

		FlipImage(seg, 0);
		sprintf(outfile,"%s_fipped_hor",buffer);
		StoreImage(seg,outfile);
		FlipImage(seg, 1);
		sprintf(outfile,"%s_fipped_ver",buffer);
		StoreImage(seg,outfile);

		RotateImage(seg, -1);
		sprintf(outfile,"%s_rotated",buffer);
		StoreImage(seg,outfile);

		InvertImage(seg);
		sprintf(outfile,"%s_inverted",buffer);
		StoreImage(seg,outfile);
		InvertImage(seg);
	
		TurnImage(seg,33);
		sprintf(outfile,"%s_turned",buffer);
		StoreImage(seg,outfile);
	
	
		/**** some filters *********/
		DeleteImage(copy);
		copy=CopyImage(image);


		Mean(image, 1);
		sprintf(outfile,"%s_mean",buffer);
		StoreImage(image,outfile);
	
		diff=Diff(copy,image);
		sprintf(outfile,"%s_diff_mean",buffer);
		StoreImage(diff,outfile);

		Gauss(image, 2, 0.5);
		sprintf(outfile,"%s_gauss",buffer);
		StoreImage(image,outfile);
	
		Median(image,3);
		sprintf(outfile,"%s_median",buffer);
		StoreImage(image,outfile);

		if(strcmp(image->itype,"gry")==0){
			/**** draw histogram ***/
			psHisto(image,"histo",1);
	
			/*** binarization **/
			gBin(120,0,255,image);
			sprintf(outfile,"%s_bin",buffer);
			StoreImage(image,outfile);
	
	
			/***quantification **/
			h=bQuant(image);
			epc1= bEuler8(image,h);	
	
			/*** morphological operations **/
			bErodeMir(image,4,0);
			sprintf(outfile,"%s_eroded",buffer);
			StoreImage(image,outfile);
			bErodeMir(image,4,1);
			sprintf(outfile,"%s_opened",buffer);
			StoreImage(image,outfile);
	
			/***quantification **/
			h=bQuant(image);
			epc2= bEuler8(image,h);
			vv=bVoldens(image,h);
			sv=bLengthdens(image,h);
	
			printf("\nEuler number before: %lf  and after opening: %lf\n",epc1, epc2);
			printf("\n area density: %lf\n",vv);
			printf("\n boundary length density: %lf\n",sv);

			InvertImage(image);
			distance=CopyImage(image);
			opened=CopyImage(image);
			i=bGetDistOpenMap(image,distance,opened,50,4);
			sprintf(outfile,"%s_distmap",buffer);
			StoreImage(distance,outfile);
			sprintf(outfile,"%s_openmap",buffer);
			StoreImage(opened,outfile);
			DeleteImage(opened);
			DeleteImage(distance);
		}
	}
}
DeleteImage(image);

}


