/*---------------------------------------------------------------------
VERSION 1.0,COPYRIGHT 5. 1998 H.J. VOGEL
quantim IS LICENSED FREE OF CHARGE. THEREFORE THIS FILE,AND ALL THE ACCOMPANYING FILES WHICH IN THEIR ENTIRETY CONSTITUTE quantim, ARE PROVIDED "AS IS",WITHOUT WARRANTY OF ANY KIND,WHETHER EXPRESSED OR IMPLIED. YOU ARE RESPONSIBLE FOR
ASCERTAINING THE FITNES


%--------------------------------- hj 26.1.00
\command{       % COMMAND
double BtdDiffusionZ(image\_c *in, FILE *dif, unsigned long max, unsigned long min, int fluxstep, double sens);
}{              % DESCRIPTION
Calculates diffusion through phase [1] in z-direction. The concentration at z=0 is kept fixed while the opposite side is fixed at zero. Diffusive flow across the plane z=zmax is calculated iteratively using explicite finite differences. }

\medskip        %ARGUMENTS AND RETURN VALUE
\argu{*in}{Pointer to the 3D binary image}
\argu{*dif}{Pointer to the file where the diffusive flow is stored in intervals indicated by {\tt fluxstep}}
\argu{max}{Maximum number of iterations (depends on sample size, may be 50.000 to 1.000.000)}
\argu{min}{Minimum number of iterations }
\argu{fluxstep}{Interval of iterations to store the diffusive flux at z=zmax (e.g. 100)}
\argu{sens}{Interupt criteria: stop iterations if  flux[j]-flux[j-1] $<$ flux[j]*sens }
\return{Relative apparent diffusion coefficient $D_s/D_0$ }
}

---------------------------------------------------------------------------*/
#include "quantim4.h"
#include "PSgraf3.h"

#define R 0.16   /*must be < 1/6, accuracy increase with decreasing value (but also calculation time)*/ 
#define CONZ 100

//int N6x[6]={0,1,0,-1,0,0};
//int N6y[6]={0,0,0,0,-1,1};
//int N6z[6]={-1,0,1,0,0,0};

void psPlotTit2(char *fname, int n, double *xdat, double *ydat, int mode, char *xtitle, char *ytitle);

double BtdDiffusionZ(image_cc *in, FILE *dif, unsigned long max, unsigned long min, int fluxstep, double sens)
{
	int i,x,y,z,r,j;
	char buffer[127];
	double *incon, *outcon,val,val2,sum, *temp,*flux,fluxold,*ti;

	incon=(double *)malloc(in->dim[1]*in->dim[0]*in->dim[2]*sizeof(double));
	outcon=(double *)malloc(in->dim[1]*in->dim[0]*in->dim[2]*sizeof(double));
	flux=(double *)malloc((max/fluxstep+1)*sizeof(double));
	ti=(double *)malloc((max/fluxstep+1)*sizeof(double));

	for(i=0;i< max/fluxstep+1; i++) flux[i]=0;

	/*** write boundary conditions ***/
	for(z=0;z<in->dim[2];z++)
	for(y=0;y<in->dim[1];y++)
	  for(x=0;x<in->dim[0];x++){
	    if(!z)
	      *(incon+x+in->dim[0]*(y+z*in->dim[1]))=*(outcon+x+in->dim[0]*(y+z*in->dim[1]))=CONZ;
	    else if(!y || y==in->dim[1]-1 || !x || x==in->dim[0]-1)
	      *(incon+x+in->dim[0]*(y+z*in->dim[1]))=*(outcon+x+in->dim[0]*(y+z*in->dim[1]))=0;
	    else
	      *(incon+x+in->dim[0]*(y+z*in->dim[1]))=*(outcon+x+in->dim[0]*(y+z*in->dim[1]))=(double)rPixel(x,y,z,in);
	  }
	
	/****Iterations ***********/
	j=i=0; 
	while(1){
	  for(z=1;z<in->dim[2]-1;z++)	   
	  for(y=1;y<in->dim[1]-1;y++)	   
	   for(x=1;x<in->dim[0]-1;x++){	      
	       	if(val=*(incon+x+in->dim[0]*(y+z*in->dim[1]))){
	        	sum=0;
			for(r=0;r<6;r++){
				val2=incon[x+N6x[r]+in->dim[0]*((y+N6y[r])+(z+N6z[r])*in->dim[1])];
				if(!val2) sum+=R*val;
				else sum+=R*val2;
			}
			*(outcon+x+in->dim[0]*(y+z*in->dim[1]))=val-6*R*val+sum;
	       }
	    }
	  temp=incon; incon=outcon; outcon=temp; 
	  if(!(i%fluxstep)){
		z=in->dim[2]-2;
	   	for(y=0; y<in->dim[1];y++) 
		  for(x=0;x<in->dim[0];x++)	      
			if(val= *(incon+x+in->dim[0]*(y+z*in->dim[1])))
			  flux[j]+=val-1;                   /** corrsponds to Ds **/
    		flux[j]/=((double)CONZ/in->dim[2])*in->dim[0]*in->dim[1];  /** corrsponds to D0 **/
		ti[j]=i;
 		printf("\nIteration %d (%d X %d):  flux %lf\n ",i,in->dim[0],in->dim[1],flux[j]);
 		fprintf(dif,"%d %lf\n ",i,flux[j]);

		if((i>min && flux[j]-flux[j-1] < flux[j]*sens) || i>max)
		  break;
		j++;
	  }
	  i++;
	}
	sprintf(buffer,"conc%d",in->dim[2]);
	psPlotTit2(buffer,j,ti,flux,1,"iterations", "diffusion"); 

	val=flux[j];
	free(incon); free(outcon); free(flux); free(ti);
	return val;
}

void psPlotTit2(char *fname, int n, double *xdat, double *ydat, int mode, char *xtitle, char *ytitle)
{
	int i;
	double sx, sy, maxx, maxy, minx, miny;
	
	gPaper(fname);

 	maxx=minx=xdat[0];
	maxy=miny=ydat[0];
	for(i=0;i<n;i++){
		if(xdat[i]>maxx) maxx=xdat[i];
		if(xdat[i]<minx) minx=xdat[i];
		if(ydat[i]>maxy) maxy=ydat[i];
		if(ydat[i]<miny) miny=ydat[i];
	}
	maxx+=fabs(maxx*0.05);
	minx-=fabs(minx*0.05);
	maxy+=fabs(maxy*0.05);
	miny-=fabs(miny*0.05);

	//sy = (maxy-miny)/(PSHIGHT+XTIT);
	//sx = (maxx-minx)/(PSWIDTH+YTIT);
	//sWorldCoord(20,20,sx,sy);
	sXWorldCoord(minx,maxx,0.,PSWIDTH+YTIT);
	sYWorldCoord(miny,maxy,0.,PSHIGHT+XTIT);

 	dYAxis(minx, miny, maxy,1);
	dXAxis(miny,minx, maxx,1);
	sText(FONT, 12,'C','T');
	sTextRotation(90);
	movea('W',minx,(maxy+miny)/2);
	mover('P', YTIT,0);
	dText(ytitle);
	sText(FONT, 12,'C','B');
	sTextRotation(0);
	movea('W',(maxx+minx)/2,miny);
	mover('P',0, XTIT);
	dText(xtitle);
		
 	if(mode==0)
 	   for(i=0;i<n;i++)
	   	dSymbol('W',xdat[i],ydat[i],1,0);
	else
		dPolygon('W',xdat,ydat,n);
	endPS();
}
 
