/*---------------------------------------------------------------------
VERSION 1.0,COPYRIGHT 5. 1998 H.J. VOGEL
quantim IS LICENSED FREE OF CHARGE. THEREFORE THIS FILE,AND ALL THE ACCOMPANYING FILES WHICH IN THEIR ENTIRETY CONSTITUTE quantim, ARE PROVIDED "AS IS",WITHOUT WARRANTY OF ANY KIND,WHETHER EXPRESSED OR IMPLIED. YOU ARE RESPONSIBLE FOR
ASCERTAINING THE FITNES


%--------------------------------- hj 04.06.03
\command{       % COMMAND
image\_c *BtdErodeMir(image\_c *im, int step, int mode)
}{              % DESCRIPTION
Same as BtdErode except that the shell is not cut off. To calculate the erosion at the border of the image, it is enlarged by mirroring the structure at the boundaries. 

\medskip        %ARGUMENTS AND RETURN VALUE
\argu{*im}{Pointer to the 3D image}
\argu{step}{Radius of the structuring element[voxels]}
\argu{mode}{Phase to be eroded [0,1]}
\return{Pointer to the image containing the eroded structure}
}

---------------------------------------------------------------------------*/
#include "quantim4.h"

image_cc *BtdErodeMir(image_cc *im, int step, int mode)
{
	int x,y,z,X,Y,Z,XX,YY,ZZ, Xx, Yy, Zz,i;
	unsigned long j, NB, count;
	char *se, *voxbuf;
	image_cc *outim, *mirim, *plane;
	
	NB = im->dim[0] * im->dim[1] * im->numbytes;
	voxbuf=(char *)malloc(NB);
	for(j=0;j<NB;j++) *(voxbuf+j)=*(im->pix+j); /** just a copy **/


	se = GetSphereElement(step,im->res[0],im->res[1],im->res[2]);
	//mirim=InitBtdImage(im->dim[0]+2*se[0],im->dim[1]+2*se[1],im->dim[2]+2*se[2],im->res[0],im->res[1],im->res[2]);	
	mirim=InitImage(im->dim[0]+2*se[0],im->dim[1]+2*se[1],im->dim[2]+2*se[2],0, "btd");
	for(i=0;i<3;i++) mirim->res[i]=im->res[i];	

	for(z=0;z<mirim->dim[2];z++)
	 for(y=0;y<mirim->dim[1];y++)
	  for(x=0;x<mirim->dim[0];x++){
	    XX=abs(x-se[0]); YY=abs(y-se[1]); ZZ=abs(z-se[2]);
	    Xx=XX/im->dim[0]; Yy=YY/im->dim[1]; Zz=ZZ/im->dim[2]; 
	    X=XX-2*(XX-im->dim[0]+1)*Xx; 
	    Y=YY-2*(YY-im->dim[1]+1)*Yy; 
	    Z=ZZ-2*(ZZ-im->dim[2]+1)*Zz;     
      	    wPixel(x,y,z,mirim,rPixel(X,Y,Z,im));
	  }
    if(step==9){
   	 //plane = GetBtdXYPlane(mirim, mirim->dim[2]/2);
 	 plane = GetPlane(mirim, mirim->dim[2]/2,0);
	 StoreImage(plane,"plane-mirim-step9.tiff");
	 DeleteImage(plane);
    }	
	for(z=0;z<im->dim[2];z++)
	  for(y=0;y<im->dim[1];y++)
	    for(x=0;x<im->dim[0];x++)
		    if((rPixel(x,y,z,im)+mode+1)%2)
		    { /**if 1 for mode=1, if 0 for mode=0**/
		      count=0;
		      for(Z=-se[2];Z<=se[2];Z++)
		        for(Y=-se[1];Y<=se[1];Y++)
		          for(X=-se[0];X<=se[0];X++)
							{
			          if((rPixel(x+X+se[0],y+Y+se[1],z+Z+se[2],mirim)+mode)%2 && *(se+3+count/8) & 1<<count%8)
								{
			            if(mode)
				            *(voxbuf+z/8+x*im->numbytes+y*im->dim[0]*im->numbytes) &= ~(1<<z%8);
			            else
				            *(voxbuf+z/8+x*im->numbytes+y*im->dim[0]*im->numbytes) |= 1<<z%8;
			          }
			          count++;
		          }
	      }
	//outim = InitBtdImage(im->dim[0],im->dim[1],im->dim[2],im->res[0],im->res[1],im->res[2]);
	outim = InitImage(im->dim[0],im->dim[1],im->dim[2],0,"btd");
	for(i=0;i<3;i++) outim->res[i]=im->res[i];	

	for(z=0;z<im->dim[2];z++)
	  for(y=0;y<im->dim[1];y++)
	    for(x=0;x<im->dim[0];x++)
	 	    wPixel(x,y,z,outim,(*(voxbuf + z/8 + x*im->numbytes + y*im->dim[0]*im->numbytes) & 1<<z%8)); /** just a copy **/
	free(voxbuf);
	return outim;
}
