/*---------------------------------------------------------------------
VERSION 1.0,COPYRIGHT 5. 1998 H.J. VOGEL
quantim IS LICENSED FREE OF CHARGE. THEREFORE THIS FILE,AND ALL THE ACCOMPANYING FILES WHICH IN THEIR ENTIRETY CONSTITUTE quantim, ARE PROVIDED "AS IS",WITHOUT WARRANTY OF ANY KIND,WHETHER EXPRESSED OR IMPLIED. YOU ARE RESPONSIBLE FOR
ASCERTAINING THE FITNES


%--------------------------------- hj 14.04.04
\command{       % COMMAND
int BtdGetDistOpenMapDouble(image\_c *image, image_cc *distance, image_cc *opened,  int gval, int gstep, double step)
}{              % DESCRIPTION
As BtdGetDistOpenMap(), but the diameter of the spherical structuring element is incremented by steps of 0.5 to get a better resolution of the size distribution. This function converts a binary 3d image (btd-format) to its distance map (ddd-format). Additionally a 'granulometry map' is calculated where  each voxel in phase 1 is written to a grey value which corresponds to the diameter of the maximum ball that can be placed inside phase 1 at that location (corresponding to the 'opening size'). The smallest ball is marked by the grey value {\\tt gval} which increases by steps {\\tt gstep} with the size of the ball.
 

\medskip        %ARGUMENTS AND RETURN VALUE
\argu{*image}{Pointer to the 3D binary image (btd-format)}
\argu{*distance}{Must be a copy of {\\tt image} in ddd-format and contains the resulting distance map after execution}
\argu{*opened}{Must be a copy of {\\tt image} in ddd-format and contains the resulting distance map after execution}
\argu{gval}{Grey level for the smallest size class}
\argu{gstep}{Grey level step between subsequent size classes}
\argu{step}{increment for structuring element (typically 0.5)}
\return{Number of detected size classes}
}

---------------------------------------------------------------------------*/
#include "quantim4.h"

image_cc *BtdErodeMirCondDouble(image_cc *im,image_cc *cond, double step, int mode); 

int BtdGetDistOpenMapDouble(image_cc *image, image_cc *distance, image_cc *opened,  int gval, int gstep, double dstep){

  image_cc *dilat,*op, *plane;
  int x,y,z, count, val, istep;
  double step;
  char buffer[127];

  step=1.;
  istep=1;
  count=1;
  val=0;
  while(count){
    count=0;
    printf("\n erosion step : %lf \n",step);
    dilat=BtdErodeMirCondDouble(image, distance, step, 1);
    for(x=0;x<dilat->dim[0];x++)
     for(y=0;y<dilat->dim[1];y++)
      for(z=0;z<dilat->dim[2];z++)
//for i in *.c; do sed -e 's/RBtdVoxel(\([^,]*\)\([^)]*\)    ->rcol/\1->res\[0\]/g' $i > $i.tmp; mv $i.tmp $i; done
	if(!rPixel(x,y,z,dilat) && rPixel(x,y,z,distance)==255){
	  wPixel(x,y,z,distance,gval+istep*gstep);
	  count++;
	}

    /****************************
    if(step){
  	 plane = GetBtdXYPlane(dilat, dilat->dim[2]/2);
	 sprintf(buffer,"plane-dilat-step-%d.tiff",step);
	 SaveImage(plane,buffer);
	 DeleteImage(plane);
  	 plane = GetDddXYPlane(distance, distance->dim[2]/2);
	 sprintf(buffer,"plane-distance-step-%d.tiff",step);
	 SaveImage(plane,buffer);
	 DeleteImage(plane);
      }
    ********************************/

    DeleteImage(dilat);
    dilat=DddBin(distance,254);
    op=BtdErodeMirCondDouble(dilat,opened, step, 0);
    val=gval+istep*gstep;
    for(x=0;x<op->dim[0];x++)
     for(y=0;y<op->dim[1];y++)
      for(z=0;z<op->dim[2];z++)
	if(!rPixel(x,y,z,op) && rPixel(x,y,z,opened)==255){
	  wPixel(x,y,z,opened,val);
	  count++;
	}

    /****************************
    if(step){
  	 plane = GetBtdXYPlane(dilat, dilat->dim[2]/2);
	 sprintf(buffer,"plane-dilat2-step-%d.tiff",step);
	 SaveImage(plane,buffer);
	 DeleteImage(plane);
  	 plane = GetBtdXYPlane(op, op->dim[2]/2);
	 sprintf(buffer,"plane-op-step-%d.tiff",step);
	 SaveImage(plane,buffer);
	 DeleteImage(plane);
  	 plane = GetDddXYPlane(opened, opened->dim[2]/2);
	 sprintf(buffer,"plane-opened-step-%d.tiff",step);
	 SaveImage(plane,buffer);
	 DeleteImage(plane);
    }
    ********************************/

    DeleteImage(dilat);
    DeleteImage(op);
    step+=dstep;
    istep++;
  }
  
  return istep-1;   
}


image_cc *BtdErodeMirCondDouble(image_cc *im,image_cc *cond, double step, int mode)
{
	int x,y,z,X,Y,Z,XX,YY,ZZ, Xx, Yy, Zz,i, vol;
	unsigned long j, NB, count;
	char *se, *voxbuf;
	image_cc *outim, *mirim, *plane;
	
	NB = im->dim[0] * im->dim[1] * im->numbytes;
	voxbuf=(char *)malloc(NB);
	for(j=0;j<NB;j++) *(voxbuf+j)=*(im->pix+j); /** just a copy **/


	se = GetSphereElementDouble(step,im->res[0],im->res[1],im->res[2], &vol);
	mirim=InitImage(im->dim[0]+2*se[0],im->dim[1]+2*se[1],im->dim[2]+2*se[2],0,"btd");
	for(i=0;i<3;i++) mirim->res[i]=im->res[i];

	for(z=0;z<mirim->dim[2];z++)
	 for(y=0;y<mirim->dim[1];y++)
	  for(x=0;x<mirim->dim[0];x++){
	    XX=abs(x-se[0]); YY=abs(y-se[1]); ZZ=abs(z-se[2]);
	    Xx=XX/im->dim[0]; Yy=YY/im->dim[1]; Zz=ZZ/im->dim[2]; 
	    X=XX-2*(XX-im->dim[0]+1)*Xx; 
	    Y=YY-2*(YY-im->dim[1]+1)*Yy; 
	    Z=ZZ-2*(ZZ-im->dim[2]+1)*Zz;     
      	    wPixel(x,y,z,mirim,rPixel(X,Y,Z,im));
	  }

	  for(z=0;z<im->dim[2];z++)
	  for(y=0;y<im->dim[1];y++)
	    for(x=0;x<im->dim[0];x++)
		    if((rPixel(x,y,z,im)+mode+1)%2 && rPixel(x,y,z,cond)==255)
		    { /**if 1 for mode=1, if 0 for mode=0**/
		      count=0;
		      for(Z=-se[2];Z<=se[2];Z++)
		        for(Y=-se[1];Y<=se[1];Y++)
		          for(X=-se[0];X<=se[0];X++){
			          if((rPixel(x+X+se[0],y+Y+se[1],z+Z+se[2],mirim)+mode)%2 && *(se+3+count/8) & 1<<count%8){
			            if(mode){
				            *(voxbuf+z/8+x*im->numbytes+y*im->dim[0]*im->numbytes) &= ~(1<<z%8);
					    Z=se[2]+1; Y=se[1]+1; X=se[0]+1;
				    }
			            else{
				            *(voxbuf+z/8+x*im->numbytes+y*im->dim[0]*im->numbytes) |= 1<<z%8;
					    Z=se[2]+1; Y=se[1]+1; X=se[0]+1;
				    }
			          }
			          count++;
		          }
	      }
	
	outim = InitImage(im->dim[0],im->dim[1],im->dim[2],0,"btd");
	for(i=0;i<3;i++) outim->res[i]=im->res[i];

	for(z=0;z<im->dim[2];z++)
	  for(y=0;y<im->dim[1];y++)
	    for(x=0;x<im->dim[0];x++)
	 	    wPixel(x,y,z,outim,(*(voxbuf + z/8 + x*im->numbytes + y*im->dim[0]*im->numbytes) & 1<<z%8)); /** just a copy **/
	free(voxbuf);
	DeleteImage(mirim);
	return outim;
}	/**** demo of routines *********/


