/*---------------------------------------------------------------------
VERSION 1.0,COPYRIGHT 5. 1998 H.J. VOGEL
quantim IS LICENSED FREE OF CHARGE. THEREFORE THIS FILE,AND ALL THE ACCOMPANYING FILES WHICH IN THEIR ENTIRETY CONSTITUTE quantim, ARE PROVIDED "AS IS",WITHOUT WARRANTY OF ANY KIND,WHETHER EXPRESSED OR IMPLIED. YOU ARE RESPONSIBLE FOR
ASCERTAINING THE FITNES


%--------------------------------- hj 26.1.00
\command{       % COMMAND
double BtdSurfdens(image\_c *image, long int *h);
}{              % DESCRIPTION
Returnes the surface density of a binary structure}

\medskip        %ARGUMENTS AND RETURN VALUE
\argu{*h}{Pointer to the array of voxel configurations obtained by BtdQuant()}
\return{surface density [L$^2$/L$^3$] in units of image->res[0]}
}

---------------------------------------------------------------------------*/
#include "quantim4.h"

void weights(double *Delta,double *weight);

double BtdSurfdens(image_cc *image, long int *h)
{
	int i,l;
	unsigned long sv,le=0;
	double wi[13], r[13], Sv,Lv, *Delta, *weight;	
	int kl[13][2] = {{1,2},{1,4},{1,16},{1,8},{2,4},{1,32},{2,16},{1,64},{4,16},{1,128},{2,64},{4,32},{8,16}};

	Delta=(double *)malloc(3*sizeof(double)); 
	weight=(double *)malloc(7*sizeof(double)); 
	for(i=0;i<7;i++) weight[i]=0;

	r[0]=Delta[0]=image->res[0]; 
	r[1]=Delta[1]=image->res[1]; 
	r[2]=Delta[2]=image->res[2];
	r[3]=r[4]=sqrt(r[0]*r[0] + r[1]*r[1]);
	r[5]=r[6]=sqrt(r[0]*r[0] + r[2]*r[2]);
	r[7]=r[8]=sqrt(r[1]*r[1] + r[2]*r[2]);
	r[9]=r[10]=r[11]=r[12]=sqrt(r[0]*r[0] + r[1]*r[1] + r[2]*r[2]);

	weights(Delta, weight);
 	wi[0]=weight[0]; wi[1]=weight[1]; wi[2]=weight[2]; wi[3]=weight[3];
	wi[4]=weight[3]; wi[5]=weight[5]; wi[6]=weight[5]; wi[7]=weight[4];
	wi[8]=weight[4]; wi[9]=weight[6]; wi[10]=weight[6]; wi[11]=weight[6];
	wi[12]=weight[6];

	for(l=0;l<256;l++) le+= h[l];
	  Sv=Lv=0;
	  for(i=0;i<13;i++)
	    if(wi[i]){
	     sv=0;
	     for(l=0;l<256;l++){	       
	       sv+= h[l] * (l==(l|kl[i][0])) * (0==(l&kl[i][1]));
	       sv+= h[l] * (l==(l|kl[i][1])) * (0==(l&kl[i][0]));
 	     }
	     Sv+= wi[i]*(double)sv;
	     Lv+= wi[i]*(double)le*r[i];
 	   }
	free(Delta);
	free(weight);
	if(!Lv) return 0;
	else return 2.0 * Sv/Lv;
}


void weights(double *Delta,double *weight)
/* Parameters: the side lengths of the unit cell Delta[0..2],
   Returns: the weights weights[0..6] for the computation of the surface denstiy,
   Initial: N. O. Aragones Salazar , 18 Sep 1999
*/
{  int i,j,k; 
   double delta_xy, delta_yz, delta_zx, delta; 
   double v[8][3][6];   /* vertex of the polyhedron */
   double dir[8][3][6]; /* projection of the vertex over the unit sphere */ 
   double  prod0, prod1, prod2; 
  
   delta_xy=sqrt(Delta[0]*Delta[0]+Delta[1]*Delta[1]); 
   delta_yz=sqrt(Delta[1]*Delta[1]+Delta[2]*Delta[2]); 
   delta_zx=sqrt(Delta[2]*Delta[2]+Delta[0]*Delta[0]); 
   delta=sqrt(Delta[0]*Delta[0]+Delta[1]*Delta[1]+Delta[2]*Delta[2]);

   v[0][0][0]=1;
   v[0][1][0]=(delta_xy-Delta[0])/Delta[1];
   v[0][2][0]=(delta-delta_xy)/Delta[2];
   v[1][0][0]=1;
   v[1][1][0]=(delta-delta_zx)/Delta[1];
   v[1][2][0]=(delta_zx-Delta[0])/Delta[2];
   v[2][0][0]=v[1][0][0]; v[2][1][0]=-v[1][1][0]; v[2][2][0]=v[1][2][0];
   v[3][0][0]=v[0][0][0]; v[3][1][0]=-v[0][1][0]; v[3][2][0]=v[0][2][0];
   v[4][0][0]=v[0][0][0]; v[4][1][0]=-v[0][1][0]; v[4][2][0]=-v[0][2][0];
   v[5][0][0]=v[1][0][0]; v[5][1][0]=-v[1][1][0]; v[5][2][0]=-v[1][2][0];
   v[6][0][0]=v[1][0][0]; v[6][1][0]=v[1][1][0]; v[6][2][0]=-v[1][2][0];
   v[7][0][0]=v[0][0][0]; v[7][1][0]=v[0][1][0]; v[7][2][0]=-v[0][2][0];

   v[0][0][1]=(delta-delta_yz)/Delta[0];
   v[0][1][1]=1;
   v[0][2][1]=(delta_yz-Delta[1])/Delta[2];
   v[1][0][1]=(delta_xy-Delta[1])/Delta[0];
   v[1][1][1]=1;
   v[1][2][1]=(delta-delta_xy)/Delta[2];
   v[2][0][1]=v[1][0][1];
   v[2][1][1]=v[1][1][1]; v[2][2][1]=-v[1][2][1]; v[3][0][1]=v[0][0][1];
   v[3][1][1]=v[0][1][1]; v[3][2][1]=-v[0][2][1]; v[4][0][1]=-v[0][0][1];
   v[4][1][1]=v[0][1][1]; v[4][2][1]=-v[0][2][1]; v[5][0][1]=-v[1][0][1];
   v[5][1][1]=v[1][1][1]; v[5][2][1]=-v[1][2][1]; v[6][0][1]=-v[1][0][1];
   v[6][1][1]=v[1][1][1]; v[6][2][1]=v[1][2][1]; v[7][0][1]=-v[0][0][1];
   v[7][1][1]=v[0][1][1]; v[7][2][1]=v[0][2][1];

   v[0][0][2]=(delta_zx-Delta[2])/Delta[0];
   v[0][1][2]=(delta-delta_zx)/Delta[1];
   v[0][2][2]=1;
   v[1][0][2]=(delta-delta_yz)/Delta[0];
   v[1][1][2]=(delta_yz-Delta[2])/Delta[1];
   v[1][2][2]=1;
   v[2][0][2]=-v[1][0][2]; v[2][1][2]=v[1][1][2]; v[2][2][2]=v[1][2][2];
   v[3][0][2]=-v[0][0][2]; v[3][1][2]=v[0][1][2]; v[3][2][2]=v[0][2][2];
   v[4][0][2]=-v[0][0][2]; v[4][1][2]=-v[0][1][2]; v[4][2][2]=v[0][2][2];
   v[5][0][2]=-v[1][0][2]; v[5][1][2]=-v[1][1][2]; v[5][2][2]=v[1][2][2];
   v[6][0][2]=v[1][0][2]; v[6][1][2]=-v[1][1][2]; v[6][2][2]=v[1][2][2];
   v[7][0][2]=v[0][0][2]; v[7][1][2]=-v[0][1][2]; v[7][2][2]=v[0][2][2];

   for(k=0;k<=2;k++)
      for(i=0;i<=7;i++)
         for(j=0;j<=2;j++)
            dir[i][j][k]=v[i][j][k]/sqrt(pow(v[i][0][k],2)
              +pow(v[i][1][k],2)
              +pow(v[i][2][k],2));
 
   for(k=0;k<=2;k++) 
    {  for(i=0;i<=7;i++) 
        {   prod0=dir[i%8][0][k]*dir[(i+2)%8][0][k] 
               +dir[i%8][1][k]*dir[(i+2)%8][1][k] 
               +dir[i%8][2][k]*dir[(i+2)%8][2][k]; 
             prod1=dir[i%8][0][k]*dir[(i+1)%8][0][k] 
               +dir[i%8][1][k]*dir[(i+1)%8][1][k] 
               +dir[i%8][2][k]*dir[(i+1)%8][2][k]; 
             prod2=dir[(i+1)%8][0][k]*dir[(i+2)%8][0][k] 
              +dir[(i+1)%8][1][k]*dir[(i+2)%8][1][k] 
              +dir[(i+1)%8][2][k]*dir[(i+2)%8][2][k]; 
             weight[k]=weight[k] 
              +acos((prod0-prod1*prod2)/(sqrt(1.-prod1*prod1)*sqrt(1.-prod2*prod2))); 
      } 
        weight[k]=(weight[k]-6.*M_PI)/(4.*M_PI); 
   } 
 
    v[0][0][3]=v[0][0][0]; v[0][1][3]=v[0][1][0]; v[0][2][3]=v[0][2][0]; 
    v[1][0][3]=v[0][0][0]; v[1][1][3]=v[0][1][0]; v[1][2][3]=-v[0][2][0]; 
    v[2][0][3]=v[1][0][1]; v[2][1][3]=v[1][1][1]; v[2][2][3]=-v[1][2][1]; 
    v[3][0][3]=v[1][0][1]; v[3][1][3]=v[1][1][1]; v[3][2][3]=v[1][2][1]; 
    v[0][0][4]=v[0][0][1]; v[0][1][4]=v[0][1][1]; v[0][2][4]=v[0][2][1]; 
    v[1][0][4]=-v[0][0][1]; v[1][1][4]=v[0][1][1]; v[1][2][4]=v[0][2][1]; 
    v[2][0][4]=-v[1][0][2]; v[2][1][4]=v[1][1][2]; v[2][2][4]=v[1][2][2]; 
    v[3][0][4]=v[1][0][2]; v[3][1][4]=v[1][1][2]; v[3][2][4]=v[1][2][2]; 
 
    v[0][0][5]=v[0][0][2]; v[0][1][5]=v[0][1][2]; v[0][2][5]=v[0][2][2]; 
    v[1][0][5]=v[0][0][2]; v[1][1][5]=-v[0][1][2]; v[1][2][5]=v[0][2][2]; 
    v[2][0][5]=v[1][0][0]; v[2][1][5]=-v[1][1][0]; v[2][2][5]=v[1][2][0]; 
    v[3][0][5]=v[1][0][0]; v[3][1][5]=v[1][1][0]; v[3][2][5]=v[1][2][0]; 
 
    for (k=3; k<=5; k++) 
        for (i=0; i<=3; i++) 
            for (j=0; j<=2; j++) 
                dir[i][j][k]=v[i][j][k]/sqrt(pow(v[i][0][k],2) 
                  +pow(v[i][1][k],2) 
                  +pow(v[i][2][k],2)); 
 
    for (k=3; k<=5; k++) 
    {   for (i=0; i<=3; i++) 
        {   prod0=dir[i%4][0][k]*dir[(i+2)%4][0][k] 
                 +dir[i%4][1][k]*dir[(i+2)%4][1][k] 
                 +dir[i%4][2][k]*dir[(i+2)%4][2][k]; 
             prod1=dir[i%4][0][k]*dir[(i+1)%4][0][k] 
                 +dir[i%4][1][k]*dir[(i+1)%4][1][k] 
                 +dir[i%4][2][k]*dir[(i+1)%4][2][k]; 
             prod2=dir[(i+1)%4][0][k]*dir[(i+2)%4][0][k] 
                 +dir[(i+1)%4][1][k]*dir[(i+2)%4][1][k] 
                 +dir[(i+1)%4][2][k]*dir[(i+2)%4][2][k]; 
             weight[k]=weight[k] 
                 +acos((prod0-prod1*prod2)/(sqrt(1-pow(prod1,2))*sqrt(1-pow(prod2,2)))); 
        } 
        weight[k]=(weight[k]-2*M_PI)/(4*M_PI); 
    } 
 
    weight[6]=(1-2*(weight[0]+weight[1]+weight[2])-4*(weight[3]+weight[4]+weight[5]))/8; 
} 

