/*---------------------------------------------------------------------
VERSION 1.0,COPYRIGHT 5. 1998 H.J. VOGEL
quantim IS LICENSED FREE OF CHARGE. THEREFORE THIS FILE,AND ALL THE ACCOMPANYING FILES WHICH IN THEIR ENTIRETY CONSTITUTE quantim, ARE PROVIDED "AS IS",WITHOUT WARRANTY OF ANY KIND,WHETHER EXPRESSED OR IMPLIED. YOU ARE RESPONSIBLE FOR
ASCERTAINING THE FITNES


%--------------------------------- hj 13.08.01
\command{       % COMMAND
image\_c *GetCorDDDImage2(double *cdf, int col, int row, int dep, double rx, double ry, double rz, double cx, double cy, double cz, int mode);
}{              % DESCRIPTION
Returnes a random 3D grey image with predefined grey histogram and/or correlation lengths. The image is periodic only for dimensions = power of 2}

\medskip        %ARGUMENTS AND RETURN VALUE
\argu{*cdf}{Pointer to the 256-element array containing the cumulative density function of grey levels. }
\argu{col, row, dep}{Size of the image (number of voxels in x, y and z). }
\argu{rx, ry, ry}{Size of the voxels in x, y and z. }
\argu{cx, cy, cz}{Correlation lengths (number of voxels in x, y and z). }
\argu{mode}{Correlation model:\\ 
0 = equal grey distribution without any correlation (cx,cy,cz and cdf have no meaning here)\\
1 = equal grey distribution with correlation (correlation lengths of resulting image will be different from cx, cy, cz!)\\ 
2 = predefined greylevel (cdf) distribution and gaussian correlation (correlation lengths of resulting image will be different from cx, cy, cz!)\\ 
3 = Gaussian covariance (cdf have no meaning here and in the following modes)\\
4 = Lorentz covariance \\
5 = Exponential  covariance\\ 
6 = von Karman covariance}
\return{pointer to the generated image.}
}

---------------------------------------------------------------------------*/
#include "quantim4.h"


image_cc *GetCorDDDImage2(double *cdf, int col,int row,int dep,double rx,double ry, double rz,double cx,double cy, double cz, int mode)
{
	double ***sfa, randnum,rval,min,max, *iscdf,sum;
	int x,y,z,val,i, N;
        image_cc *ddd;
	long idum, count=0;
	
	N=col;
	min=max=0;
	idum=-1;
	ddd=InitImage(col,row,dep,255,"ddd");
	ddd->res[0]=rx; ddd->res[1]=ry;ddd->res[2]=rz;
	
	/***** Generation of random field ****/
	iscdf=(double *)malloc(256*sizeof(double));
	for(i=0;i<256;i++)
		iscdf[i]=0;

	sfa=(double ***)malloc(N*sizeof(double **));
	for(x=0;x<N;x++) {
	     sfa[x]=(double **)malloc(N*sizeof(double *));
	     for(y=0;y<N;y++)
	         sfa[x][y]=(double *)malloc(N*sizeof(double));
	}
	if(mode){
	   if(mode<4)	grf3(sfa,rx,ry,rz,cx,cy,cz,1,N,0);
	   else    	grf3(sfa,rx,ry,rz,cx,cy,cz,1,N,mode-3);
	   for(z=0;z<ddd->dim[2];z++)
	   for(y=0;y<ddd->dim[1];y++)
	   for(x=0;x<ddd->dim[0];x++){
		 if(sfa[x][y][z]<min) min=(sfa[x][y][z]);
		 if(sfa[x][y][z]>max) max=(sfa[x][y][z]);
	   }
	   printf("\nmin: %lf    max: %lf \n",min,max);

	   /*** calculate cdf of generated values ****/
	   for(z=0;z<ddd->dim[2];z++)
	   for(y=0;y<ddd->dim[1];y++)
	   for(x=0;x<ddd->dim[0];x++){
	     val =int((sfa[x][y][z]-min)/(max-min) * 255);
	     iscdf[val]++;
	     count++;
	   }
	   for(i=0;i<256;i++)
		iscdf[i]= (double) iscdf[i]/count;

	   sum=iscdf[0];
	   for(i=1;i<256;i++){
		sum+=iscdf[i];
		iscdf[i]+=iscdf[i-1];
	   }
	   for(i=0;i<256;i++)
		iscdf[i]= iscdf[i]/sum;	
	   iscdf[0]=0;
	}

	for(z=0;z<ddd->dim[2];z++)
	for(y=0;y<ddd->dim[1];y++)
	for(x=0;x<ddd->dim[0];x++){
	   if(mode==0)             val=int(ran3(&idum) * 255);

	   if(mode==1){
	     val =int((sfa[x][y][z]-min)/(max-min) * 255);
	     for(i=0;i<256;i++)
		if(iscdf[val]< (double)i/256) 
		  break;
             val = i>255?255:i;  
	   }
	   if(mode==2){                          //predefined cdf 
	     val =int((sfa[x][y][z]-min)/(max-min) * 255);
	     for(i=0;i<256;i++)
		if(iscdf[val]<cdf[i]) 
		  break;
             val = i>255?255:i;  
	   }

	   if(mode>2) 
	     val =int((sfa[x][y][z]-min)/(max-min) * 255);

	   wPixel(x,y,z, ddd,val);
	}
	for(x=0;x<N;x++) for(y=0;y<N;y++) free(sfa[x][y]);
	for(x=0;x<N;x++) free(sfa[x]);
	free(sfa);
return ddd;	
}

