/*---------------------------------------------------------------------
VERSION 1.0,COPYRIGHT 5. 1998 H.J. VOGEL
quantim IS LICENSED FREE OF CHARGE. THEREFORE THIS FILE,AND ALL THE ACCOMPANYING FILES WHICH IN THEIR ENTIRETY CONSTITUTE quantim, ARE PROVIDED "AS IS",WITHOUT WARRANTY OF ANY KIND,WHETHER EXPRESSED OR IMPLIED. YOU ARE RESPONSIBLE FOR
ASCERTAINING THE FITNES


%--------------------------------- hj 25.02.02
\command{       % COMMAND
image\_c *GetPercolClus(int width, int height, double lamx, double lamy, int mode);
}{              % DESCRIPTION
Returnes a percolation cluster based on the excursion set of a random greyscale image  

\medskip        %ARGUMENTS AND RETURN VALUE
\argu{width, height}{size of image}
\argu{lamx, lamy}{Correlation length in x and y}
\argu{mode}{0: random (correlation lengths have no meaning)\\ 
            1: gaussian covariance with correlation lengths }
\return{pointer to the binary image of resulting percolation cluster}
}

---------------------------------------------------------------------------*/
#include "quantim4.h"
#include <time.h>

int bPerClus(image_cc *image);
void bContin(image_cc *image);
image_cc *GetCorImageExt( int col,int row, double cx,double cy, int mode);

image_cc *GetPercolClus(int width, int height, double lamx, double lamy, int mode)
{
  image_cc *percol, *cop;
  int i;

  percol=GetCorImageExt(width, height,lamx,lamy,mode);
  for(i=1;i<255;i++){
    cop = CopyImage(percol);
    gBin(i,0,255,cop);
    if(bPerClus(cop))
       break;
    DeleteImage(cop);
  }

  bContin(cop); //isolate Percolation cluster 
  //  bThinning(cop); //isolate backbone

  InvertImage(cop);
  //SaveImage(cop,"percol.tiff");
  return cop;
}

int bPerClus(image_cc *image)
{
	int x,y,b,i,j,count, sum, val=10;
	image_cc *wnd1;
	int ispercol=0;

	wnd1 = InitImage(image->dim[0],image->dim[1],255);

	for(x=0; x<wnd1->dim[0]; x++)
	  if(!rPixel(x,0,image))
	    wPixel(x,0,wnd1,val);
	
	count=1;
	while(count){
	     count=0;
	     for(y=1;y<image->dim[1];y++)
	      for (x=0;x<image->dim[0];x++){
		     if(!rPixel(x,y,image) && rPixel(x,y,wnd1)!=val){
			for(j=-1;j<=1;j++)
			for(i=-1;i<=1;i++)
			  if(x+i>=0&&x+i<image->dim[0]&&y+j>=0&&y+j<image->dim[1])
			     if(rPixel(x+i,y+j,wnd1)==val){
				wPixel(x,y,wnd1,val);
				if(y==image->dim[1]-1) ispercol=1;
				count=1;
			     }
		     }
		}
	}
	DeleteImage(wnd1);
	return ispercol;
}
 
void bContin(image_cc *image)
{
	int x,y,b,i,j,count, val=0;
	image_cc *wnd1;
	
	wnd1 = InitImage(image->dim[0],image->dim[1],255);

	for(x=0; x<wnd1->dim[0]; x++)
	  if(!rPixel(x,0,image))
	    wPixel(x,0,wnd1,val);
      
	count=1;
	while(count){
	     count=0;
	     for(y=0;y<wnd1->dim[1];y++)
	      for (x=0;x<wnd1->dim[0];x++){
		     if(rPixel(x,y,wnd1)==val){
			for(j=-1;j<=1;j++)
			for(i=-1;i<=1;i++)
			  if(x+i>=0&&x+i<image->dim[0]&&y+j>=0&&y+j<image->dim[1])
			     if(!rPixel(x+i,y+j,image) && (rPixel(x+i,y+j,wnd1)==255)){
				wPixel(x+i,y+j,wnd1,val);
				count=1;
			     }
		     }
		}
	}
	for(y=0;y<wnd1->dim[1];y++)
	   for (x=0;x<wnd1->dim[0];x++)
	     wPixel(x,y,image,rPixel(x,y,wnd1));
	DeleteImage(wnd1);
}
 


image_cc *GetCorImageExt( int col,int row, double cx,double cy, int mode)
/*
Ext: size of image must not be power of 2 
mode:

0:	equal distribution without any correlation (cx,cy,cz and cdf have no meaning here)
1:	equal distribution with  gaussian correlation   
2:      predefined distribution (cdf) distribution and gaussian correlation   
3:	Gaussian correlation (cdf have no meaning here)
4:	Lorentz-Correlation model (cdf have no meaning here)
5:	Exponential-Correlation model (cdf have no meaning here)
6:	von Karman-Correlation model (cdf have no meaning here)
7:      explicite autocovariance (acov[2][lag])
*/
{
	double *sfa, randnum,rval,min,max, *iscdf,sum, **acov;
	int x,y,z,val,i, N,NN, model;
        image_cc *random;
	long idum, count=0;
	
	min=max=0;
	N=col>=row?col:row;
	for(i=1;i<11;i++)
	  if( (NN=int(pow(2,i)))>=N)
	    break;
       printf("\ngenerating percolation cluster, size  %d\n ",NN);
        //idum=-1;
	time(&idum);  
	random=InitImage(NN,NN,255);
	random->res[0]=1; random->res[1]=1;
	
	if(mode){
	  sfa=(double *)malloc(NN*NN*sizeof(double));
	  grf2ext(sfa,NN,NN,1.,1.,cx,cy,1.,NN,mode>2?mode-3:0, acov); /* hj */
	  for(i=0;i<NN*NN;i++){
	    if(sfa[i]<min) min=(sfa[i]);
	    if(sfa[i]>max) max=(sfa[i]);
	  }
	}
	for(y=0;y<random->dim[1];y++)
	for(x=0;x<random->dim[0];x++){
	   if(!mode)         
	         val=int(ran3(&idum) * 255);
	   else 
	         val =int((*(sfa+y*NN+x)-min)/(max-min) * 255);
	   wPixel(x,y, random,val);
	}
	if(mode) free(sfa);
return random;	
}

