/*---------------------------------------------------------------------
VERSION 1.0,COPYRIGHT 5. 1998 H.J. VOGEL
quantim IS LICENSED FREE OF CHARGE. THEREFORE THIS FILE,AND ALL THE ACCOMPANYING FILES WHICH IN THEIR ENTIRETY CONSTITUTE quantim, ARE PROVIDED "AS IS",WITHOUT WARRANTY OF ANY KIND,WHETHER EXPRESSED OR IMPLIED. YOU ARE RESPONSIBLE FOR
ASCERTAINING THE FITNES


%--------------------------------- hj 24.02.02
\command{       % COMMAND
image\_c *GetVoronoiTes(int Xdim, int Ydim , int Nump, int mode);
}{              % DESCRIPTION
 Generates a Voronoi tesselation based on {\tt NumP} random seed point. In the resulting image the Voronoi-cells are marked by different grey levels [1-254], the edges between the cells are white (g=255). The variable {\tt mode} determines the type of random point process. 

mode = 0 -> radom points 
mode = 1 -> points of 'growing size'
mode >1  -> explicite size of points (radius in #pixels)

\medskip        %ARGUMENTS AND RETURN VALUE
\argu{Xdim}{x-size of resulting image }
\argu{Ydim}{y-size of resulting image }
\argu{Nump}{number of seed points }
\argu{mode}{0: random points; 1: points of growing size; $>$1: explicit size of seed points (radius in pixel)}
\return{pointer to the resulting image}
}

---------------------------------------------------------------------------*/
#include "quantim4.h"

void SetPoints(image_cc *image, int NumP, int mode);

image_cc *GetVoronoiTes(int Xdim, int Ydim , int Nump)
{
  	image_cc *image, *copy;
	int i,j,k, x,y, xx,yy, count, change, val, val2, edge, mode=0;
	char buffer[127];

	image=InitImage(Xdim,Ydim,255);

	printf("\ngenerating Voronoi Tesselation using mode %d\n",mode);

	SetPoints(image, Nump, mode);
	//SaveImage(image, "test.tiff");

	copy=CopyImage(image);
	change=1;
	k=0;
	while(change){
	  change=0;
	  for(y=1;y<image->dim[1]-1;y++)
	    for(x=1;x<image->dim[0]-1;x++){
	     if(rPixel(x,y,image)==255){
	       count=edge=0;
	       if(k%2){
		 for(j=0;j<8;j++){
		   if((val=rPixel(x+neigh8x[j],y+neigh8y[j],image))!=255){
		     if(!count){    val2=val;}
		     else{          if(val2!=val)  edge=1;}
		     count++;
		   }
		 }
	       }
	       else{
		 for(j=0;j<4;j++){
		   if((val=rPixel(x+neigh4x[j],y+neigh4y[j],image))!=255){
		     if(!count){    val2=val;}
		     else{          if(val2!=val)  edge=1;}
		     count++;
		   }
		 }
	       }
	       if(!edge && count){
		 wPixel(x,y,copy,val2);
		 change++;
	       }
	     }
	  }
	  for(y=0;y<image->dim[1];y++)
	   for(x=0;x<image->dim[0];x++)
	     wPixel(x,y,image,rPixel(x,y,copy));
	  //printf("\nchanged %d\n",change);
	  k++;
	  //if(!(k%10)){
	  //  sprintf(buffer,"voronoi%d.tiff",k);
	  //  SaveImage(image, buffer);
	  //}
	}
	/** white lines between the cells **/
	for(y=0;y<image->dim[1];y++)
	   for(x=1;x<image->dim[0];x++)
	     if((val=rPixel(x,y,image))!=(val2=rPixel(x-1,y,image)) && val !=255 && val2 !=255)
	       wPixel(x,y,image,255);
	for(x=0;x<image->dim[0];x++)
	 for(y=1;y<image->dim[1];y++)
	     if((val=rPixel(x,y,image))!=(val2=rPixel(x,y-1,image)) && val !=255 && val2 !=255)
	       wPixel(x,y,image,255);	

	return image;
}

image_cc *GetVoronoiTes(int Xdim, int Ydim , int Nump, int mode)
{
  	image_cc *image, *copy;
	int i,j,k, x,y, xx,yy, count, change, val, val2, edge;
	char buffer[127];

	image=InitImage(Xdim,Ydim,255);

	printf("\ngenerating Voronoi Tesselation using mode %d\n",mode);

	SetPoints(image, Nump, mode);
	//SaveImage(image, "test.tiff");

	copy=CopyImage(image);
	change=1;
	k=0;
	while(change){
	  change=0;
	  for(y=1;y<image->dim[1]-1;y++)
	    for(x=1;x<image->dim[0]-1;x++){
	     if(rPixel(x,y,image)==255){
	       count=edge=0;
	       if(k%2){
		 for(j=0;j<8;j++){
		   if((val=rPixel(x+neigh8x[j],y+neigh8y[j],image))!=255){
		     if(!count){    val2=val;}
		     else{          if(val2!=val)  edge=1;}
		     count++;
		   }
		 }
	       }
	       else{
		 for(j=0;j<4;j++){
		   if((val=rPixel(x+neigh4x[j],y+neigh4y[j],image))!=255){
		     if(!count){    val2=val;}
		     else{          if(val2!=val)  edge=1;}
		     count++;
		   }
		 }
	       }
	       if(!edge && count){
		 wPixel(x,y,copy,val2);
		 change++;
	       }
	     }
	  }
	  for(y=0;y<image->dim[1];y++)
	   for(x=0;x<image->dim[0];x++)
	     wPixel(x,y,image,rPixel(x,y,copy));
	  //printf("\nchanged %d\n",change);
	  k++;
	  //if(!(k%10)){
	  //  sprintf(buffer,"voronoi%d.tiff",k);
	  //  SaveImage(image, buffer);
	  //}
	}
	/** white lines between the cells **/
	for(y=0;y<image->dim[1];y++)
	   for(x=1;x<image->dim[0];x++)
	     if((val=rPixel(x,y,image))!=(val2=rPixel(x-1,y,image)) && val !=255 && val2 !=255)
	       wPixel(x,y,image,255);
	for(x=0;x<image->dim[0];x++)
	 for(y=1;y<image->dim[1];y++)
	     if((val=rPixel(x,y,image))!=(val2=rPixel(x,y-1,image)) && val !=255 && val2 !=255)
	       wPixel(x,y,image,255);	

	return image;
}


/* Generates NumP random points in the image (typically blanc white with g=255). The points are written to grey levels in the range [1, 254], so that equal values for neighboring points are avoided. May be used to generate seed poibts for a Voronoi Tesselation. 
mode = 0 -> radom points 
mode = 1 -> points of 'growing size'
mode >1  -> explicite size of points (radius in #pixels)
*/
int DrawCircle(image_cc *im, int xmid, int ymid, int rad, int val);

void  SetPoints(image_cc *image, int NumP, int mode)
{
        int change, count, j,k, x,y, size;
	long idum;

	idum = 1;
	change=0;
	k=0;
	while(change<NumP){
	  k++;
	  if(mode==1) size=(NumP-k)/2;
	  if(mode>1)  size=mode; 
	  x=int(ran3(&idum)*(image->dim[0]-2*mode)+1);
	  y=int(ran3(&idum)*(image->dim[1]-2*mode)+1);
	  if(rPixel(x,y,image)==255){
	    count=0;
	    if(!mode){
	      for(j=0;j<8;j++)
		if(rPixel(x+neigh8x[j],y+neigh8y[j],image)!=255)
		  count++;
	      if(!count){
		wPixel(x,y,image,change%254*254/(NumP%254)+1);
		change++;
	      }
	    }
	    else{
	      count=DrawCircle(image, x, y, size, change%254*254/(NumP%254)+1);
	      if(!count) change++;
	    }
	  }
	}	
	printf("\nseed points generated: %d\n",change);
}

int DrawCircle(image_cc *image, int xmid, int ymid, int rad, int val)
{
  int x,y;

    for(x=xmid-rad;x<xmid+rad;x++)
      for(y=ymid-rad;y<ymid+rad;y++){
	x=(x+image->dim[0])%image->dim[0];
	y=(y+image->dim[0])%image->dim[1];
	if(sqrt((x-xmid)*(x-xmid)+(y-ymid)*(y-ymid))<rad){
	    if(rPixel(x,y,image)!=255)
	      return 1;
	}
      }
    for(x=xmid-rad;x<xmid+rad;x++)
      for(y=ymid-rad;y<ymid+rad;y++){
	x=(x+image->dim[0])%image->dim[0];
	y=(y+image->dim[0])%image->dim[1];
	if(sqrt((x-xmid)*(x-xmid)+(y-ymid)*(y-ymid))<rad)
	  wPixel(x,y,image,val);
      }
    return 0;
}
  
