/*---------------------------------------------------------------------
VERSION 1.0,COPYRIGHT 5. 1998 H.J. VOGEL
quantim IS LICENSED FREE OF CHARGE. THEREFORE THIS FILE,AND ALL THE ACCOMPANYING FILES WHICH IN THEIR ENTIRETY CONSTITUTE quantim, ARE PROVIDED "AS IS",WITHOUT WARRANTY OF ANY KIND,WHETHER EXPRESSED OR IMPLIED. YOU ARE RESPONSIBLE FOR
ASCERTAINING THE FITNES


%--------------------------------- hj 25.02.02
\command{       % COMMAND
image\_c *GetPercolClus(int width, int height, double lamx, double lamy, int mode);
}{              % DESCRIPTION
Returnes a percolation cluster based on the excursion set of a random greyscale image  

\medskip        %ARGUMENTS AND RETURN VALUE
\argu{width, height}{size of image}
\argu{lamx, lamy}{Correlation length in x and y}
\argu{mode}{0: random (correlation lengths have no meaning)\\ 
            1: gaussian covariance with correlation lengths }
\return{pointer to the binary image of resulting percolation cluster}
}

---------------------------------------------------------------------------*/
#include "quantim4.h"
#include <time.h>

int bPerClus(image_cc *image);
void bContin(image_cc *image);
image_cc *GetCorImageExt( int col,int row, double cx,double cy, int mode);

image_cc *GetPercolClus(int width, int height, double lamx, double lamy, int mode)
{
  image_cc *percol, *cop;
  int i;

  percol=GetCorImageExt(width, height,lamx,lamy,mode);
  for(i=1;i<255;i++){
    cop = CopyImage(percol);
    gBin(i,0,255,cop);
    if(bPerClus(cop))
       break;
    DeleteImage(cop);
  }

  bContin(cop); //isolate Percolation cluster 
  //  bThinning(cop); //isolate backbone

  InvertImage(cop);
  //SaveImage(cop,"percol.tiff");
  return cop;
}

int isPercol(image_cc *image,int backbone, int mode)
/*******************
returns 1 if the 0-values of a binary structure percolate in the direction(s) indicated by mode (bit1 x, bit2 y, bit3 z). if Backbone is not 0 the complete backbone is stored as additional image if backbone is 0 the calculation is interupted as soon as percolation was detected (faster) - yet to be tested 
*************/
{
	int x,y,z,b,i,j,k,count, sum, val=100;
	image_cc *wnd1;
	int npercol=0, ispercol=0;



	if(image->ndim==2){	
/******** 2D images *******************/
	
	wnd1 = InitImage(image->dim[0],image->dim[1],255);

    	if(mode & 1){
	 for(x=0; x<wnd1->dim[0]; x++)
	  if(!rPixel(x,0,image))
	    wPixel(x,0,wnd1,val);	
	 count=1;
	 //while(count && !ispercol){
	 while(count && (backbone?count:(!ispercol))){
	     count=0;
	     for(y=1;y<image->dim[1];y++)
	      for (x=0;x<image->dim[0];x++){
		     if(!rPixel(x,y,image) && rPixel(x,y,wnd1)!=val){
			for(j=-1;j<=1;j++)
			for(i=-1;i<=1;i++)
			  if(x+i>=0&&x+i<image->dim[0]&&y+j>=0&&y+j<image->dim[1])
			     if(rPixel(x+i,y+j,wnd1)==val){
				wPixel(x,y,wnd1,val);
				if(y==image->dim[1]-1) ispercol=1;
				count=1;
			     }
		     }
		}
	 }
	 if(backbone)
		StoreImage(wnd1,"backbone");
   	}
    	npercol+=ispercol;
    	ispercol=0;		
    	val=200; 
   	if(mode & 1<<1){
	 for(y=0; y<wnd1->dim[1]; y++)
	  if(!rPixel(0,y,image))
	    wPixel(0,y,wnd1,val);	
	count=1;
	//while(count && !ispercol){
	while(count && (backbone?count:(!ispercol))){
	     count=0;
	     for (x=1;x<image->dim[0];x++)
	       for(y=0;y<image->dim[1];y++){
		     if(!rPixel(x,y,image) && rPixel(x,y,wnd1)!=val){
			for(j=-1;j<=1;j++)
			for(i=-1;i<=1;i++)
			  if(x+i>=0&&x+i<image->dim[0]&&y+j>=0&&y+j<image->dim[1])
			     if(rPixel(x+i,y+j,wnd1)==val){
				wPixel(x,y,wnd1,val);
				if(x==image->dim[0]-1) ispercol=1;
				count=1;
			     }
		     }
		}
	}
	if(backbone)
		StoreImage(wnd1,"backbone");
     	}
    	npercol+=ispercol;
    	ispercol=0;		
	}
/******** 3D images *******************/
	else{
	
	      wnd1 = InitImage(image->dim[0],image->dim[1],image->dim[2],255,"ddd");

    	      if(mode & 1<<2){
	 	for(x=0; x<wnd1->dim[0]; x++)
	 	for(y=0; y<wnd1->dim[1]; y++)
	  		if(!rPixel(x,y,0,image))
	    			wPixel(x,y,0,wnd1,val);	
	 	count=1;
		while(count && (backbone?count:(!ispercol))){
	 	//while(count && !ispercol){
	     	count=0;
	     	for(z=1;z<image->dim[2];z++)
	      	for (y=0;y<image->dim[1];y++)
	      	for (x=0;x<image->dim[0];x++){
		     if(!rPixel(x,y,z,image) && rPixel(x,y,z,wnd1)!=val){
			for(k=-1;k<=1;k++)
			for(j=-1;j<=1;j++)
			for(i=-1;i<=1;i++)
			  if(x+i>=0&&x+i<image->dim[0]&&y+j>=0&&y+j<image->dim[1]&&z+k>=0&&z+k<image->dim[2])
			     if(rPixel(x+i,y+j,z+k,wnd1)==val){
				wPixel(x,y,z,wnd1,val);
				if(z==image->dim[2]-1) ispercol=1;
				count=1;
			     }
		     }
		}
	 }
	 if(backbone)
		StoreImage(wnd1,"backbone");
   	}
    	npercol+=ispercol;
    	ispercol=0;		
    	val=150; 
   	if(mode & 1){
	 	for(z=0; z<wnd1->dim[2]; z++)
	 	for(y=0; y<wnd1->dim[1]; y++)
	  		if(!rPixel(0,y,z,image))
	    			wPixel(0,y,z,wnd1,val);
	 	count=1;
		while(count && (backbone?count:(!ispercol))){
	 	//while(count && !ispercol){
	     	count=0;
	      	for (x=1;x<image->dim[0];x++)
	     	for(z=0;z<image->dim[2];z++)
	      	for (y=0;y<image->dim[1];y++){
		     if(!rPixel(x,y,z,image) && rPixel(x,y,z,wnd1)!=val){
			for(k=-1;k<=1;k++)
			for(j=-1;j<=1;j++)
			for(i=-1;i<=1;i++)
			  if(x+i>=0&&x+i<image->dim[0]&&y+j>=0&&y+j<image->dim[1]&&z+k>=0&&z+k<image->dim[2])
			     if(rPixel(x+i,y+j,z+k,wnd1)==val){
				wPixel(x,y,z,wnd1,val);
				if(x==image->dim[0]-1) ispercol=1;
				count=1;
			     }
		     }
		}
	 }
	 if(backbone)
		StoreImage(wnd1,"backbone");
   	}
    	npercol+=ispercol;
    	ispercol=0;		
    	val=200; 
   	if(mode & 1<<1){
	 	for(z=0; z<wnd1->dim[2]; z++)
	 	for(x=0; x<wnd1->dim[0]; x++)
	  		if(!rPixel(x,0,z,image))
	    			wPixel(x,0,z,wnd1,val);	
	 	count=1;
		while(count && (backbone?count:(!ispercol))){
	 	//while(count && (backbone?(!ispercol):count)){
	     	count=0;
	      	for (y=1;y<image->dim[1];y++)
	      	for (x=0;x<image->dim[0];x++)
	     	for(z=0;z<image->dim[2];z++){
		     if(!rPixel(x,y,z,image) && rPixel(x,y,z,wnd1)!=val){
			for(k=-1;k<=1;k++)
			for(j=-1;j<=1;j++)
			for(i=-1;i<=1;i++)
			  if(x+i>=0&&x+i<image->dim[0]&&y+j>=0&&y+j<image->dim[1]&&z+k>=0&&z+k<image->dim[2])
			     if(rPixel(x+i,y+j,z+k,wnd1)==val){
				wPixel(x,y,z,wnd1,val);
				if(y==image->dim[1]-1) ispercol=1;
				count=1;
			     }
		     }
		}
	 }
	 if(backbone)
		StoreImage(wnd1,"backbone");
   	}
    	npercol+=ispercol;
	}

    	DeleteImage(wnd1);

	j=0;
	for(i=0;i<3; i++) if(mode & 1<<i) j++;
	if(npercol==j) 	return 1;
	else 		return 0;
}



int isPercol_old(image_cc *image, int mode)
/*******************
returns 1 if the 0-values of a binary structure percolates in the direction(s) indicated by mode (bit1 x, bit2 y, bit3 z) - yet to be tested 
*************/

{
	int x,y,z,b,i,j,k,count, sum, val=10;
	image_cc *wnd1;
	int npercol=0, ispercol=0;



	if(image->ndim==2){	
/******** 2D images *******************/
	
	wnd1 = InitImage(image->dim[0],image->dim[1],255);

    	if(mode & 1){
	 for(x=0; x<wnd1->dim[0]; x++)
	  if(!rPixel(x,0,image))
	    wPixel(x,0,wnd1,val);	
	 count=1;
	 while(count && !ispercol){
	     count=0;
	     for(y=1;y<image->dim[1];y++)
	      for (x=0;x<image->dim[0];x++){
		     if(!rPixel(x,y,image) && rPixel(x,y,wnd1)!=val){
			for(j=-1;j<=1;j++)
			for(i=-1;i<=1;i++)
			  if(x+i>=0&&x+i<image->dim[0]&&y+j>=0&&y+j<image->dim[1])
			     if(rPixel(x+i,y+j,wnd1)==val){
				wPixel(x,y,wnd1,val);
				if(y==image->dim[1]-1) ispercol=1;
				count=1;
			     }
		     }
		}
	 }
   	}
    	npercol+=ispercol;
    	ispercol=0;		
    	val=20; 
   	if(mode & 1<<1){
	 for(y=0; y<wnd1->dim[1]; y++)
	  if(!rPixel(0,y,image))
	    wPixel(0,y,wnd1,val);	
	count=1;
	while(count && !ispercol){
	     count=0;
	     for (x=1;x<image->dim[0];x++)
	       for(y=0;y<image->dim[1];y++){
		     if(!rPixel(x,y,image) && rPixel(x,y,wnd1)!=val){
			for(j=-1;j<=1;j++)
			for(i=-1;i<=1;i++)
			  if(x+i>=0&&x+i<image->dim[0]&&y+j>=0&&y+j<image->dim[1])
			     if(rPixel(x+i,y+j,wnd1)==val){
				wPixel(x,y,wnd1,val);
				if(x==image->dim[0]-1) ispercol=1;
				count=1;
			     }
		     }
		}
	}
     	}
    	npercol+=ispercol;
    	ispercol=0;		
	}
/******** 3D images *******************/
	else{
	
	      wnd1 = InitImage(image->dim[0],image->dim[1],image->dim[2],255);

    	      if(mode & 1){
	 	for(x=0; x<wnd1->dim[0]; x++)
	 	for(y=0; y<wnd1->dim[1]; y++)
	  		if(!rPixel(x,y,0,image))
	    			wPixel(x,y,0,wnd1,val);	
	 	count=1;
	 	while(count && !ispercol){
	     	count=0;
	     	for(z=1;z<image->dim[2];z++)
	      	for (y=0;y<image->dim[1];y++)
	      	for (x=0;x<image->dim[0];x++){
		     if(!rPixel(x,y,z,image) && rPixel(x,y,z,wnd1)!=val){
			for(k=-1;k<=1;k++)
			for(j=-1;j<=1;j++)
			for(i=-1;i<=1;i++)
			  if(x+i>=0&&x+i<image->dim[0]&&y+j>=0&&y+j<image->dim[1]&&y+k>=0&&y+k<image->dim[2])
			     if(rPixel(x+i,y+j,z+k,wnd1)==val){
				wPixel(x,y,z,wnd1,val);
				if(z==image->dim[2]-1) ispercol=1;
				count=1;
			     }
		     }
		}
	 }
   	}
    	npercol+=ispercol;
    	ispercol=0;		
    	val=20; 
   	if(mode & 1<<1){
	 	for(z=0; z<wnd1->dim[2]; z++)
	 	for(y=0; y<wnd1->dim[1]; y++)
	  		if(!rPixel(0,y,z,image))
	    			wPixel(0,y,z,wnd1,val);	
	 	count=1;
	 	while(count && !ispercol){
	     	count=0;
	      	for (x=1;x<image->dim[0];x++)
	     	for(z=0;z<image->dim[2];z++)
	      	for (y=0;y<image->dim[1];y++){
		     if(!rPixel(x,y,z,image) && rPixel(x,y,z,wnd1)!=val){
			for(k=-1;k<=1;k++)
			for(j=-1;j<=1;j++)
			for(i=-1;i<=1;i++)
			  if(x+i>=0&&x+i<image->dim[0]&&y+j>=0&&y+j<image->dim[1]&&y+k>=0&&y+k<image->dim[2])
			     if(rPixel(x+i,y+j,z+k,wnd1)==val){
				wPixel(x,y,z,wnd1,val);
				if(x==image->dim[0]-1) ispercol=1;
				count=1;
			     }
		     }
		}
	 }
   	}
    	npercol+=ispercol;
    	npercol+=ispercol;
    	ispercol=0;		
    	val=30; 
   	if(mode & 1<<2){
	 	for(z=0; z<wnd1->dim[2]; z++)
	 	for(x=0; x<wnd1->dim[0]; x++)
	  		if(!rPixel(x,0,z,image))
	    			wPixel(x,0,z,wnd1,val);	
	 	count=1;
	 	while(count && !ispercol){
	     	count=0;
	      	for (y=1;y<image->dim[1];y++)
	      	for (x=0;x<image->dim[0];x++)
	     	for(z=0;z<image->dim[2];z++){
		     if(!rPixel(x,y,z,image) && rPixel(x,y,z,wnd1)!=val){
			for(k=-1;k<=1;k++)
			for(j=-1;j<=1;j++)
			for(i=-1;i<=1;i++)
			  if(x+i>=0&&x+i<image->dim[0]&&y+j>=0&&y+j<image->dim[1]&&y+k>=0&&y+k<image->dim[2])
			     if(rPixel(x+i,y+j,z+k,wnd1)==val){
				wPixel(x,y,z,wnd1,val);
				if(x==image->dim[0]-1) ispercol=1;
				count=1;
			     }
		     }
		}
	 }
   	}
    	npercol+=ispercol;
	}

    	DeleteImage(wnd1);

	j=0;
	for(i=0;i<3; i++) if(mode & 1<<i) j++;
	if(npercol==j) 	return 1;
	else 		return 0;
}

double *PercolProb(image_cc *im, int *maxwin; int mode)
{
	
	int i, x,y,z,xx,yy,zz, cw, cwmax;
	double *percp;
	long int *h, count, *absperc;
	image_cc *sim;
	
	cwmax= im->dim[0]>im->dim[1]? im->dim[1]:im->dim[0];
	cwmax= cwmax>im->dim[2]?im->dim[2]:cwmax;
	percp=(double *)malloc(cwmax*sizeof(double));
	abspercp=(long int *)malloc(cwmax*sizeof(long int));
	
	h=m3Quant(im);
	percp[0]= m3voldens(im,h);

    	for(cw=2;cw<cwmax;cw++){
		absperc[cw-1]=0;
		count=0;
		for(z=0;z<im->dim[2]-cw;z++)
		for(y=0;y<im->dim[1]-cw;y++)
		for(x=0;x<im->dim[0]-cw;x++){
			sim=ImageSegment(im,x,y,z,cw,cw,cw);
			count++;
			absperc[cw-1]+=isPercol(sim,0,mode);
		}
		percp[cw-1]=(double)absperc[cw-1]/(double)count;
	}
	*maxwin=cwmax-1;
	return percp;
}
 
void bContin(image_cc *image)
{
	int x,y,b,i,j,count, val=0;
	image_cc *wnd1;
	
	wnd1 = InitImage(image->dim[0],image->dim[1],255);

	for(x=0; x<wnd1->dim[0]; x++)
	  if(!rPixel(x,0,image))
	    wPixel(x,0,wnd1,val);
      
	count=1;
	while(count){
	     count=0;
	     for(y=0;y<wnd1->dim[1];y++)
	      for (x=0;x<wnd1->dim[0];x++){
		     if(rPixel(x,y,wnd1)==val){
			for(j=-1;j<=1;j++)
			for(i=-1;i<=1;i++)
			  if(x+i>=0&&x+i<image->dim[0]&&y+j>=0&&y+j<image->dim[1])
			     if(!rPixel(x+i,y+j,image) && (rPixel(x+i,y+j,wnd1)==255)){
				wPixel(x+i,y+j,wnd1,val);
				count=1;
			     }
		     }
		}
	}
	for(y=0;y<wnd1->dim[1];y++)
	   for (x=0;x<wnd1->dim[0];x++)
	     wPixel(x,y,image,rPixel(x,y,wnd1));
	DeleteImage(wnd1);
}
 


image_cc *GetCorImageExt( int col,int row, double cx,double cy, int mode)
/*
Ext: size of image must not be power of 2 
mode:

0:	equal distribution without any correlation (cx,cy,cz and cdf have no meaning here)
1:	equal distribution with  gaussian correlation   
2:      predefined distribution (cdf) distribution and gaussian correlation   
3:	Gaussian correlation (cdf have no meaning here)
4:	Lorentz-Correlation model (cdf have no meaning here)
5:	Exponential-Correlation model (cdf have no meaning here)
6:	von Karman-Correlation model (cdf have no meaning here)
7:      explicite autocovariance (acov[2][lag])
*/
{
	double *sfa, randnum,rval,min,max, *iscdf,sum, **acov;
	int x,y,z,val,i, N,NN, model;
        image_cc *random;
	long idum, count=0;
	
	min=max=0;
	N=col>=row?col:row;
	for(i=1;i<11;i++)
	  if( (NN=int(pow(2,i)))>=N)
	    break;
       printf("\ngenerating percolation cluster, size  %d\n ",NN);
        //idum=-1;
	time(&idum);  
	random=InitImage(NN,NN,255);
	random->res[0]=1; random->res[1]=1;
	
	if(mode){
	  sfa=(double *)malloc(NN*NN*sizeof(double));
	  grf2ext(sfa,NN,NN,1.,1.,cx,cy,1.,NN,mode>2?mode-3:0, acov); /* hj */
	  for(i=0;i<NN*NN;i++){
	    if(sfa[i]<min) min=(sfa[i]);
	    if(sfa[i]>max) max=(sfa[i]);
	  }
	}
	for(y=0;y<random->dim[1];y++)
	for(x=0;x<random->dim[0];x++){
	   if(!mode)         
	         val=int(ran3(&idum) * 255);
	   else 
	         val =int((*(sfa+y*NN+x)-min)/(max-min) * 255);
	   wPixel(x,y, random,val);
	}
	if(mode) free(sfa);
return random;	
}

