
/*---------------------------------------------------------------------
VERSION 1.0,COPYRIGHT 5. 1998 H.J. VOGEL
quantim IS LICENSED FREE OF CHARGE. THEREFORE THIS FILE,AND ALL THE ACCOMPANYING FILES WHICH IN THEIR ENTIRETY CONSTITUTE quantim, ARE PROVIDED "AS IS",WITHOUT WARRANTY OF ANY KIND,WHETHER EXPRESSED OR IMPLIED. YOU ARE RESPONSIBLE FOR
ASCERTAINING THE FITNES

---------------------
Schlueter - June 2010 
---------------------
FUNCTION:
ConLength() calculates the mean connectivity length according to Western et al.2001:WRR 37(1),83-97,2000WR900241.  and its electronic supplement from ftp://kosmos.agu.org

ARGUMENTS: 
- image: the binary image to be evaluated
- phase: 0-black, 1-white
- outfile: stores data for pair connectivity as a function of distance

RETURN VALUE: mean connectivity length (integral scale of pair connectivity)

changes:
SS 18.11.2013: replaced ConCom4() with Cluster()
SS 18.11.2013: added 3D
SS 18.11.2013: incremenation by every to increase speed

---------------------------------------------------------------------------*/

#include "quantim4.h"

double ConLength(image_cc *image, int phase, int every, char *outfile)
{
	image_cc *regions;
	int x,y,z,xx,yy,zz,cluster,i, offset=0, val;
	FILE *fp;
	float intscale=0, separation, *sum_separation;
	long total_bins, *number_pairs,*number_connected, ldum=-1;

	regions=InitImage(image->dim[0],image->dim[1],0,"gry16");
	if(image->ndim==2)
	    total_bins=long(pow(image->dim[0]*image->dim[0]+image->dim[1]*image->dim[1],0.5));
	else
	    total_bins=long(pow(image->dim[0]*image->dim[0]+image->dim[1]*image->dim[1]+image->dim[2]*image->dim[2],0.5));
	
	number_pairs=(long*)calloc(total_bins,sizeof(long));
	number_connected=(long*)calloc(total_bins,sizeof(long));
	sum_separation=(float*)calloc(total_bins,sizeof(float));
	
	
	if ( (fp=fopen(outfile, "w")) == NULL)
	{
	    fprintf(stderr, "Cannot open %s in ConLength().\n",outfile);
	    exit(1);
	} 
	
	//Labeling with Hoshen & Kopelman (1976)
	printf("Labeling all objects in the image\n");
	regions=Cluster(image,&cluster);

				
	//Calculate connectivity function
	printf("Calculate pair connectivity (costly!)\n");
	
	if(image->ndim==2)
	{
   
	    for(i=0;i<total_bins;i++) number_pairs[i]=number_connected[i]=sum_separation[i]=0;
	    int **dist;
	    dist=(int**)calloc(image->dim[0],sizeof(int*));
	    for(x=0;x<image->dim[0];x++)
		dist[x]=(int*)calloc(image->dim[1],sizeof(int));
    
	    for(y=0;y<image->dim[1];y++)
		for(x=0;x<image->dim[0];x++)
		{
		    val=x*x+y*y;
		    if(val)
			dist[x][y]=int(round(pow(val,0.5)));
		    else
			dist[x][y]=0;
		}
	    
	    for(y=0;y<image->dim[1];y+=every)
	    {
		offset=int(ran3(&ldum)*every);
		for(x=offset;x<image->dim[0];x+=every)
		{
		    if(rPixel(x,y,image))
		    {
			for(yy=offset;yy<image->dim[1];yy+=every)
			{
			    for(xx=offset;xx<image->dim[0];xx+=every)
			    {
				    //separation=pow((xx-x)*(xx-x)+(yy-y)*(yy-y),0.5);
				    //i=int(round(separation));
				    //sum_separation[i]+=separation;
				    i=dist[abs(xx-x)][abs(yy-y)];
				    sum_separation[i]+=i;
				    number_pairs[i]++;
				    if(rPixel(x,y,regions)==rPixel(xx,yy,regions))
					number_connected[i]++;
			    }
			}
		    }
		}
	    }
	    for(x=0;x<image->dim[0];x++)
	    	free(dist[x]);
	    free(dist);
	    
		    
	}
	else
	{
    
	    for(i=0;i<total_bins;i++) number_pairs[i]=number_connected[i]=sum_separation[i]=0;
	    int ***dist;
	    dist=(int***)calloc(image->dim[0],sizeof(int**));
	    for(x=0;x<image->dim[0];x++)
	    {
		dist[x]=(int**)calloc(image->dim[1],sizeof(int*));
		for(y=0;y<image->dim[1];y++)
		    dist[x][y]=(int*)calloc(image->dim[2],sizeof(int));
	    }
	    for(z=0;z<image->dim[2];z++)
		for(y=0;y<image->dim[1];y++)
		    for(x=0;x<image->dim[0];x++)
		    {
			val=x*x+y*y+z*z;
			if(val)
			    dist[x][y][z]=int(round(pow(val,0.5)));
			else
			    dist[x][y][z]=0;
		    }

	    for(z=0;z<image->dim[2];z+=every)
	    {
		offset=int(ran3(&ldum)*every);
		for(y=offset;y<image->dim[1];y+=every)
		{
		    for(x=offset;x<image->dim[0];x+=every)
		    {
			if(rPixel(x,y,image))
			{
			    for(zz=offset;zz<image->dim[2];zz+=every) //start at 1 to always have some direct neighbor comparison in spite of every>1
			    {
				for(yy=offset;yy<image->dim[1];yy+=every)
				{
				    for(xx=offset;xx<image->dim[0];xx+=every)
				    {
					//separation=pow((xx-x)*(xx-x)+(yy-y)*(yy-y)+(zz-z)*(zz-z),0.5);
					//i=int(round(separation));
					//sum_separation[i]+=separation;
					i=dist[abs(xx-x)][abs(yy-y)][abs(zz-z)];
					sum_separation[i]+=i;
					number_pairs[i]++;
					if(rPixel(x,y,regions)==rPixel(xx,yy,zz,regions))
					    number_connected[i]++;
				    }
				}
			    }
			}
		    }
		}
	    }
	    for(x=0;x<image->dim[0];x++)
	    {
		for(y=0;y<image->dim[1];y++)
		    free(dist[x][y]);
		free(dist[x]);
	    }
	    free(dist);
		    	    
	}
	fprintf(fp,"lag\tmean_sep\t2p_cluster\tnr_connected\tnr_pairs\n");
	for(i=0;i<image->dim[0];i++)
		if(number_pairs[i]>0){
			fprintf(fp,"%d\t%lf\t%lf\t%ld\t%ld\n",i,sum_separation[i]/double(number_pairs[i]),double(number_connected[i])/double(number_pairs[i]),number_connected[i],number_pairs[i]);
			intscale+=double(number_connected[i])/double(number_pairs[i]);
		}
	fclose(fp);
	DeleteImage(regions);
	free(number_connected);
	free(number_pairs);
	free(sum_separation);

		
	return(intscale);

}