/*---------------------------------------------------------------------
VERSION 1.0,COPYRIGHT 5. 1998 H.J. VOGEL
quantim IS LICENSED FREE OF CHARGE. THEREFORE THIS FILE,AND ALL THE ACCOMPANYING FILES WHICH IN THEIR ENTIRETY CONSTITUTE quantim, ARE PROVIDED "AS IS",WITHOUT WARRANTY OF ANY KIND,WHETHER EXPRESSED OR IMPLIED. YOU ARE RESPONSIBLE FOR
ASCERTAINING THE FITNES


%--------------------------------- hj 12.09.03
\command{       % COMMAND
image\_c *bGetDistMap(image\_c *image, int *n, int gval, int gstep);
}{              % DESCRIPTION
Converts a binary image to its distance map: Each pixel in the white phase (255) is written to a grey value which corresponds to the distance of that pixel to phase 0. The closest distance gets grey value {\\tt gval} which increases ba steps {\\tt gstep} with distance. The phase 0 is not changed. The total number of distance classes is written to {\\tt n}. Note that it is a good idea to chose the parameters such that $ n \\cdot gstep > 255 - gval$. The resulting distance map can be used as input to a watershed segmentation e.g. to separate sintered grains or to calculate Minkowski functions. 
  

\medskip        %ARGUMENTS AND RETURN VALUE
\argu{*image}{Pointer to the binary image}
\argu{*n}{Number of detected distance classes}
\argu{gval}{Grey level for the first distance class}
\argu{gstaep}{Grey level step between adjacent distance classes}
\return{Pointer to the image containing the distance map}
}

---------------------------------------------------------------------------*/
#include "quantim4.h"

image_cc *bErodeMirCond(image_cc *im, image_cc *cond, int step, int mode,int fast);


image_cc *bGetDistMap(image_cc *image, int *n, int gval, int gstep)
{

  image_cc *out, *dilat;
  int x,y,z, count, oldcount=0, step;
  char buffer[127];

  out=CopyImage(image);
  
  step=1;
  count=1;
  while(count){
    count=0;
    printf("\n erosion step : %d \n",step);
    dilat=bErodeMirCond(image, out, step, 1,1);
    for(x=0;x<dilat->dim[0];x++)
     for(y=0;y<dilat->dim[1];y++)
	if(!rPixel(x,y,dilat) && rPixel(x,y,out)==255){
	  wPixel(x,y,out,gval+step*gstep);
	  count++;
	}
    DeleteImage(dilat);
    step++;
    //if(MAXDIL && step>MAXDIL) count=0;
  }
  *n = step-1;
  return out;   
}

int IsNeigh(int x, int y,image_cc *im);
unsigned char *GetCircElement_ext(int rad, double rx, double ry);

image_cc *bErodeMirCond(image_cc *im, image_cc *cond, int step, int mode,int fast)
{
	int x,y,X,Y,XX,YY, Xx, Yy,i;
	unsigned long j, count;
	char buffer[127];
	unsigned char *se;

	image_cc *outim, *mirim;
	

	//outim = InitImage(im->dim[0],im->dim[1],mode*255);
	outim = CopyImage(im);
	mirim=InitImage(im->dim[0]+2*step,im->dim[1]+2*step,0);
	mirim->res[0]=im->res[0];mirim->res[1]=im->res[1];	

	se = GetCircElement_ext(step,im->res[0], im->res[1]);

	for(y=0;y<mirim->dim[1];y++)
	  for(x=0;x<mirim->dim[0];x++){
	    XX=abs(x-step); YY=abs(y-step);
	    Xx=XX/im->dim[0]; Yy=YY/im->dim[1]; 
	    X=XX-2*(XX-im->dim[0]+1)*Xx; 
	    Y=YY-2*(YY-im->dim[1]+1)*Yy; 
      	    wPixel(x,y,mirim,rPixel(X,Y,im));
	  }

	for(y=0;y<im->dim[1];y++)
	  for(x=0;x<im->dim[0];x++){	
	    //if(rPixel(x,y,cond)==255 && IsNeigh(x, y, cond)){
	    if(rPixel(x,y,cond)==255)
	      if(!fast || IsNeigh(x, y, cond)){
	        count=0;
		for(Y=-se[1];Y<=se[1];Y++)
		 for(X=-se[0];X<=se[0];X++){
      		      if(rPixel(x+X+se[0],y+Y+se[1],mirim)!=mode*255 && *(se+2+count/8) & 1<<count%8){
			if(mode){
			  wPixel(x,y,outim,0);
			  X=Y=im->dim[0];
			}
			else{
			  wPixel(x,y,outim,255);
			  X=Y=im->dim[0];
			}
		      }
		      count++;
		 }
	    }
	  }
	return outim;
}

int IsNeigh(int x, int y,image_cc *im)
{
  int i;

  for(i=0;i<8;i++)
	if(!isborder(x+neigh8x[i],y+neigh8y[i],im))
    		if(rPixel(x+neigh8x[i],y+neigh8y[i],im)!=255)
      			return 1;    
  return 0;
}

unsigned char *GetCircElement_ext(int rad, double rx, double ry)
{
        int x,y,  i,N;
	long count=0;
	double Min,r,rr;
	unsigned char *s;
	
	Min = rx<=ry?rx:ry;

	/*** scale resolution towards 1 **/
	rx/=Min; ry/=Min;

	N=2+(rad*2+1)*(rad*2+1)/8+1;
	s = (unsigned char *)malloc(N);
	for(i=0;i<N;i++) s[i]=0;
	s[0]=(unsigned char)(rad/rx); s[1]=(unsigned char)(rad/ry);
	printf("s[0]=%d\n",s[0]);
	
	for(y=-s[1];y<=s[1];y++)
	for(x=-s[0];x<=s[0];x++){
	  if(sqrt((double)y*ry*y*ry+x*rx*x*rx)<=rad*1.01)
			*(s+2+count/8) |= 1<<count%8;
	 	count++;
	}
	return s;
}
 
