image_cc *CopyRGBImage(image_cc *image)
{
	unsigned long j;
	image_cc *copy;
	
	copy = (image_cc *)malloc(sizeof(image_cc));
	
	copy->numbits= image->numbits;
	copy->dim[2]= image->dim[2];
	copy->dim[0] = image->dim[0];
	copy->dim[1] = image->dim[1];
	copy->numbytes=image->numbytes; 
	copy->res[0] = image->res[0];
	copy->res[1] = image->res[1];
	copy->res[2] =image->res[2]; 
	copy->pix=(unsigned char *)malloc(image->dim[1]*image->dim[0]*image->numbytes);
	for(j=0;j<image->dim[1]*image->dim[0]*image->numbytes;j++) 
		*(copy->pix+j) = *(image->pix+j);
	return copy;
}

image_cc *InitRGBImage(int x, int y)
{
	unsigned long j;
	image_cc *image;
	
	image = (image_cc *)malloc(sizeof(image_cc));

	image->ndim= 2;	
	image->numbits= 8;
	image->dim[0] = x;
	image->dim[1] = y;
	image->numbytes = 3;
	image->numchannel = 3;
	image->res[0] = 0;
	image->res[1] = 0;
	image->res[2] = 0; 
	image->pix=(unsigned char *)malloc(image->numbytes*x*y);
	for(j=0;j<image->numbytes*x*y;j++) 
		*(image->pix+j) = 0;
	return image;
}

image_cc *InitDDDImage(int x, int y, int z, unsigned char val)
{
	unsigned long j;
	image_cc *image;
	
	image = (image_cc *)malloc(sizeof(image_cc));
	
	sprintf(image->itype,"ddd");
	image->dim[2]= z;
	image->dim[0] = x;
	image->dim[1] = y;
	image->numbytes = z;
	image->res[0] = 0;
	image->res[1] = 0;
	image->res[2] = 0; 
	image->pix=(unsigned char *)malloc(x*y*z);
	for(j=0;j<x*y*z;j++) 
		*(image->pix+j) = val;
	return image;

}

image_cc *InitBtdImage(int x, int y, int z, double rx,double ry,double rz)
{
	unsigned long j;
	image_cc *image;
	
	image = (image_cc *)malloc(sizeof(image_cc));
	
	sprintf(image->itype,"btd");
	image->dim[2]= z;
	image->dim[0] = x;
	image->dim[1] = y;
	image->numbytes = image->dim[2]%8? image->dim[2]/8+1 : image->dim[2]/8;
	image->res[0] = rx;
	image->res[1] = ry;
	image->res[2] = rz; 
	image->pix=(unsigned char *)malloc(image->numbytes*x*y);
	for(j=0;j<image->numbytes*x*y;j++) 
		*(image->pix+j) = 0;
	return image;
}

image_cc *CopyBtdImage(image_cc *image)
{
	unsigned long j;
	image_cc *copy;
	
	copy = (image_cc *)malloc(sizeof(image_cc));
	
	copy->numbits= image->numbits;
	copy->dim[2]= image->dim[2];
	copy->dim[0] = image->dim[0];
	copy->dim[1] = image->dim[1];
	copy->numbytes=image->numbytes; 
	copy->res[0] = image->res[0];
	copy->res[1] = image->res[1];
	copy->res[2] =image->res[2]; 
	copy->pix=(unsigned char *)malloc(image->numbytes*image->dim[1]*image->dim[0]);
	for(j=0;j<image->numbytes*image->dim[1]*image->dim[0];j++) 
		*(copy->pix+j) = *(image->pix+j);
	return copy;
}

void SaveRGBImage(image_cc *image, char *buf)
{
	long row;
	TIFF *tif;

	tif = TIFFOpen(buf, "w");
	TIFFSetField(tif, TIFFTAG_IMAGELENGTH, image->dim[1]);
	TIFFSetField(tif, TIFFTAG_PHOTOMETRIC, 2);
	TIFFSetField(tif, TIFFTAG_IMAGEWIDTH, image->dim[0]);
	TIFFSetField(tif, TIFFTAG_COMPRESSION, 1);
	TIFFSetField(tif, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
	TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE, 8,8,8);
	TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, image->numbytes);

	TIFFSetField(tif, TIFFTAG_ROWSPERSTRIP, 1);

	for (row = 0; row < image->dim[1]; row++)
TIFFWriteScanline(tif, image->pix+(row*image->dim[0]*image->numbytes), row,2);

TIFFClose(tif); 
}

void SaveImage(image_cc *image, char *buf)
{
	long row;
	TIFF *tif;

	tif = TIFFOpen(buf, "w");
	TIFFSetField(tif, TIFFTAG_IMAGELENGTH, image->dim[1]);
	TIFFSetField(tif, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_MINISBLACK);
	TIFFSetField(tif, TIFFTAG_IMAGEWIDTH, image->dim[0]);
	TIFFSetField(tif, TIFFTAG_COMPRESSION, COMPRESSION_NONE);
	TIFFSetField(tif, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
	TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE, 8);
	TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, 1);
	for (row = 0; row < image->dim[1]; row++)
TIFFWriteScanline(tif, image->pix+(row*image->dim[0]*image->numbytes), row,2);

TIFFClose(tif); 
}

image_cc *GetImage(char *buf)
{
	long row; 
	TIFF *tif;
	char fname[255];
	image_cc *image;
	
	image = (image_cc *)malloc(sizeof(image_cc));

	if((tif = TIFFOpen(buf, "r"))==0)
		printf("\nGetImage: could not open %s\n",buf);

	TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &image->dim[1]);
	TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &image->dim[0]);
	TIFFGetField(tif, TIFFTAG_SAMPLESPERPIXEL, &image->numbytes);
	TIFFGetField(tif, TIFFTAG_BITSPERSAMPLE, &image->numbits);

	image->numbytes=1;
	sprintf(image->name,"%s",buf);
	printf("Image name: %s\n",image->name);
	printf("Image hight: %d\n", image->dim[1]);
	printf("Image width: %d\n", image->dim[0]);
	printf("Image depth: %d\n", image->dim[2]);
	printf("Image nbytes: %d\n", image->numbytes);

	image->pix =(unsigned char *)malloc(image->dim[1]*image->dim[0]*image->numbytes);

	for (row = 0; row < image->dim[1]; row++)
TIFFReadScanline(tif, image->pix+(row*image->dim[0]*image->numbytes), row,0);

TIFFClose(tif);
 return image;
}


void GetImageAlph(image_cc *image, char *buf)
     /* loads an image having an alpha channel but ignors it */ 
{
	long row; 
	TIFF *tif;
	char fname[255];
	unsigned char *pixbuf;
	int i;
	
	if((tif = TIFFOpen(buf, "r"))==0)
		printf("\nGetImage: could not open %s\n",buf);

	TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &image->dim[1]);
	TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &image->dim[0]);
	TIFFGetField(tif, TIFFTAG_SAMPLESPERPIXEL, &image->numbytes);
	TIFFGetField(tif, TIFFTAG_BITSPERSAMPLE, &image->numbits);

	image->numbytes=1;
	sprintf(image->name,"%s",buf);
	printf("Image name: %s\n",image->name);
	printf("Image hight: %d\n", image->dim[1]);
	printf("Image width: %d\n", image->dim[0]);

	pixbuf =(unsigned char *)malloc(image->dim[1]*image->dim[0]*2);
	image->pix =(unsigned char *)malloc(image->dim[1]*image->dim[0]);

	for (row = 0; row < image->dim[1]; row++)
TIFFReadScanline(tif, pixbuf+(row*image->dim[0]*2), row,0);

	for(i=0;i<image->dim[0]*image->dim[1];i++)
	  *(image->pix+i)=*(pixbuf+2*i);

TIFFClose(tif);
}

void GetRGBImage(image_cc *image, char *buf)
{
	long row; 
	TIFF *tif;

	if((tif = TIFFOpen(buf, "r"))==0)
		printf("\ncould not open %s\n",buf);
	sprintf(image->name,"%s",buf);
	TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &image->dim[1]);
	TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &image->dim[0]);
	TIFFGetField(tif, TIFFTAG_SAMPLESPERPIXEL, &image->numbytes);
	TIFFGetField(tif, TIFFTAG_BITSPERSAMPLE, &image->numbits);

	image->numbytes=3;
	printf("Image name: %s\n",image->name);
	printf("Bytes per pixel: %d\n", image->numbytes);
	printf("Image hight: %d\n", image->dim[1]);
	printf("Image width: %d\n", image->dim[0]);

	image->pix =(unsigned char *)malloc(image->dim[1]*image->dim[0]*image->numbytes);

	for (row = 0; row < image->dim[1]; row++)
TIFFReadScanline(tif, image->pix+(row*image->dim[0]*image->numbytes), row,0);

TIFFClose(tif);
}

image_cc *GetDDDImage(char *buf)
{
 	FILE *fp;
 	unsigned long i; 	       
        long offset;
        image_cc image, *im;
        
        im= (image_cc *)malloc(sizeof(image_cc));
        sprintf(buf,"%s.ddd",buf);
        printf("\nloading: %s\n",buf);
        fp=fopen(buf,"r");
        fscanf(fp,"%ld %d %d %d %d %lf %lf %lf",&offset,&im->dim[0],&im->dim[1],&im->dim[2],&im->numbytes,&im->res[0],&im->res[1],&im->res[2]);
       im->pix=(unsigned char *)malloc(im->dim[0]*im->dim[1]*im->numbytes);

	printf("\n...loading 3D-8bit image:\nx: %d / %.2lf    y: %d / %.2lf   z: %d / %.2lf  offset: %ld\n",im->dim[0],im->res[0],im->dim[1],im->res[1],im->dim[2],im->res[2],offset);

        fseek(fp,offset,SEEK_SET);
	for(i=0;i<im->dim[0]*im->dim[1]*im->numbytes;i++) 
		im->pix[i]=getc(fp);
	fclose(fp);
	return im;
}

image_cc *GetBtdImage(char *buf)
{
 	FILE *fp;
 	unsigned long i; 	       
        long offset;
        image_cc image, *im;
        
        im= (image_cc *)malloc(sizeof(image_cc));
        sprintf(buf,"%s.btd",buf);
        printf("\nloading: %s",buf);
        if((fp=fopen(buf,"r"))==0)
	  printf("\n\nCOULD NOT OPEN FILE %s \n\n",buf);
        fscanf(fp,"%ld %d %d %d %d %lf %lf %lf",&offset,&im->dim[0],&im->dim[1],&im->dim[2],&im->numbytes,&im->res[0],&im->res[1],&im->res[2]);
       im->pix=(unsigned char *)malloc(im->dim[0]*im->dim[1]*im->numbytes);
        fseek(fp,offset,SEEK_SET);
	for(i=0;i<im->dim[0]*im->dim[1]*im->numbytes;i++) 
		im->pix[i]=getc(fp);
	fclose(fp);
	return im;
}

image_cc *CopyDddImage(image_cc *image)
{
	unsigned long j;
	image_cc *copy;
	
	copy = (image_cc *)malloc(sizeof(image_cc));
	
	copy->dim[2]= image->dim[2];
	copy->dim[0] = image->dim[0];
	copy->dim[1] = image->dim[1];
	copy->numbytes=image->numbytes; 
	copy->res[0] = image->res[0];
	copy->res[1] = image->res[1];
	copy->res[2] =image->res[2]; 
	copy->pix=(unsigned char *)malloc(image->dim[2]*image->dim[1]*image->dim[0]);
	for(j=0;j<image->dim[2]*image->dim[1]*image->dim[0];j++) 
		*(copy->pix+j) = *(image->pix+j);
	return copy;
}

