/*---------------------------------------------------------------------
VERSION 1.0,COPYRIGHT 5. 1998 H.J. VOGEL
quantim IS LICENSED FREE OF CHARGE. THEREFORE THIS FILE,AND ALL THE ACCOMPANYING FILES WHICH IN THEIR ENTIRETY CONSTITUTE quantim, ARE PROVIDED "AS IS",WITHOUT WARRANTY OF ANY KIND,WHETHER EXPRESSED OR IMPLIED. YOU ARE RESPONSIBLE FOR
ASCERTAINING THE FITNES
---------------------------------------------------------------------------*/
#include "quantim4.h"


int UnimThresh(double *histo, int mode){
	/*Function: Threshold detection in a unimodal histogram (8bit) by triangulation according to Rosin(2001):Pattern Recognition,34,2083-2096. */
	/*1. argument: histogram of gray values, e.g. via Histo()*/
	/*2. argument: if mode=0 left tail, else right tail*/
	/*Output: threshold*/
	
	int count, xmax=255, xmin=0, xmode, xrosin;
	double  a1, a2, dummy=0, sum=0,hxmode, fac; 
	
	double mean[256]; /*moving mean of three elements*/
	double xs[256],ys[256]; /*coordinates along the auxilary line*/
	double dist[256]; /*normal distance to the auxilary line*/
	double percentile=0.0001; /* percentile that is cut off from the tailing*/

	for(count=0;count<256;count++){
		xs[count]=0;
		ys[count]=0;
		dist[count]=0;
		mean[count]=0;
		}
	
	//smoothing by moving mean
	for(count=1;count<255;count++) mean[count]=(histo[count+1]+histo[count]+histo[count-1])/3; 

	//find histogram mode
	for(count=1; count<255; count++)
	{
		if (mean[count]>dummy)
		{
			dummy=mean[count]; 
			xmode=count; 
		} 
	}
	
	if(mode){//right tail

		//calculate the part of the histogram that is removed from analysis
		for(count=0; count<xmode; count++) sum+=histo[count]; 
		
		//find the maximum bin after percentile removal
		dummy=0;
		for(count=254; count>1; count--) if((dummy+=histo[count])<percentile*(1-sum)) xmax=count; 
		
		
		//this scaling maps the y-range to the x-range , so that slope a1 and a2 become -1 and 1 respectively
		fac=(xmax-xmode)/mean[xmode];
		for(count=1;count<255;count++) mean[count]*=fac;
		
	
		hxmode=mean[xmode];
// // 		a1=hxmode/(xmode-xmax); // slope a1
// // 		a2=-1/a1; //normal slope of a1
	
		
		dummy=0;
		for(count=xmax;count>xmode;count--){
// 			xs[count]=(-a2*(double)count+a1*xmode-hxmode+mean[count])/(a1-a2);
// 			ys[count]=a1*xs[count]-a1*xmode+hxmode;
			xs[count]=(-(double)count-xmode-hxmode+mean[count])/(-2.0);
			ys[count]=-xs[count]+xmode+hxmode;
			dist[count]=(xs[count]-(double)count)*(xs[count]-(double)count)+(ys[count]-mean[count])*(ys[count]-mean[count]); //squared distance
			if (dist[count]>dummy){
					dummy=dist[count]; 
					xrosin=count; 
					} 
			
			}

	}
	else{//left tail

		//calculate the part of the histogram that is removed from analysis
		for(count=254; count>xmode; count--) sum+=histo[count]; 
		
		//find the maximum bin after percentile removal
		dummy=0;
		for(count=1; count<255; count++) if((dummy+=histo[count])<percentile*(1-sum)) xmin=count; 
		
		//this scaling maps the y-range to the x-range , so that slope a1 and a2 become 1 and -1 respectively
		fac=(xmode-xmin)/mean[xmode];
		for(count=1;count<255;count++) mean[count]*=fac;
		
		hxmode=mean[xmode];
// 		a1=hxmode/(xmode-xmin); // slope a1
// 		a2=-1/a1; //normal slope of a1

	
		dummy=0;
		for(count=xmin;count<xmode;count++){
// 			xs[count]=(-a2*(double)count+a1*xmode-hxmode+mean[count])/(a1-a2);
// 			ys[count]=a1*xs[count]-a1*xmode+hxmode;
			xs[count]=((double)count+xmode-hxmode+mean[count])/2.0;
			ys[count]=xs[count]-xmode+hxmode;
			dist[count]=(xs[count]-(double)count)*(xs[count]-(double)count)+(ys[count]-mean[count])*(ys[count]-mean[count]); //squared distance
			if (dist[count]>dummy){
					dummy=dist[count]; 
					xrosin=count; 
					} 
			
			}
	}	return xrosin;
}
int *KMeansThresh(double *cdf, int nr)
{
  /*Function: Iterative K-Means Threshold Selection Method according to Ridler & Calvard (1978): IEEE Transactions on Systems, Man, AND CYBERNETICS,8(8),630-632. */
  /*1. argument: cumulative histogram of gray values, e.g. via GreyCdf*/
  /*2. argument: number of classes*/
  /*Output: Arithmetic mean of both class means*/ 
  
  int max, min, old[nr-1], *upd, flag;
  double fdum=0.0, sum;
  double pdf[256], mean[nr];
  
  upd=(int*)calloc((nr-1),sizeof(int));
  pdf[0]=cdf[0];
  flag=0;
  for(int i=1;i<256;i++)
  {
    pdf[i]=cdf[i]-cdf[i-1];
    if(pdf[i]>0 && !flag){ min=i; flag=1;}
    if(pdf[i]>0) max=i;
  }
  
  //start: split range into equidistant intervals
  double dstep=(max-min)/nr;
  for(int i=0;i<nr-1;i++)
    old[i]=min+dstep*(i+1);

  
  //iterations until the arithmetic mean of all class means remains unchanged
  while(1)
  {
    sum=0.0;
    int j=0;
    int flag=1;
    for(int i=0;i<256;i++)
    {
      sum+=pdf[i]*i;
      if(i==old[j])
      {
	if(!j)mean[j]=sum/cdf[old[j]];
	else mean[j]=sum/(cdf[old[j]]-cdf[old[j-1]]);
	if(j<nr-2)j++;
	sum=0.0;
      }
    }
    mean[j+1]=sum/(1.0-cdf[old[j]]);
    for(int j=0; j<nr-1; j++)
    {
      upd[j]=int((mean[j]+mean[j+1])/2);
      //printf("%d\t%lf\t%lf\t%d\t%d\n",j,mean[j],mean[j+1],upd[j],old[j]);
      if(old[j]!=upd[j])
      {
	old[j]=upd[j];
	flag=0;
      }
    }
    if(flag) break;
  }
  return upd;
}
int *FuzzyCMeansThresh(double *cdf, int nr, double tau)
{
  /*Function: returns vector of thresholds according to Jawahar, Biswas, Ray (1997):Pattern Recognition
  /*1. argument: cumulative histogram of gray values, e.g. via GreyCdf()*/
  /*2. argument: number of classes*/
  /*3. argument: degree of fuzziness tau=[1,+/indef); tau=1 is k-means*/
  
  int max, min, flag, steps, idum, old, upd, clas, *t, iter;
  double pdf[256], memb[nr][256], memb_new[nr][256], cmean[nr], fuzzmom[nr], fuzzprob[nr], dist[nr], fdum, change,dstep, obj_old, obj_new;
  double epsilon=10e-4;
  
  //Basic Allocations
  t=(int*)calloc(nr-1,sizeof(int));
 
  //1. Initialization of membership function by equidistant intervals between max and min
  for(int j=0;j<nr;j++)for(int i=0;i<256;i++) memb[j][i]=0.;
  pdf[0]=cdf[0];
  flag=0;
  for(int i=1;i<256;i++)
  {
    pdf[i]=cdf[i]-cdf[i-1];
    if(pdf[i]>0 && !flag){ min=i; flag=1;}
    if(pdf[i]>0) max=i;
  }
  dstep=(max-min+epsilon)/nr;
  
  for(int i=min;i<max;i++) memb[int((i-min)/dstep)][i]=1.0;
  
  
  //Starting the iterations
  iter=0;
  obj_old=1e30;
  while(1)
  {
    //2. update class means
    for(int j=0;j<nr;j++)
    {
      fuzzmom[j]=fuzzprob[j]=0;
      for(int i=min;i<max;i++)
      {
	fdum=pdf[i]*pow(memb[j][i],tau);
	fuzzmom[j]+=i*fdum; //numerator in JBR97, eq. 9
	fuzzprob[j]+=fdum; //denominator in JBR97, eq. 9
      }
      cmean[j]=fuzzmom[j]/fuzzprob[j]; //JBR97, eq. 9
      //printf("%d\t%lf\n",j,cmean[j]);
    }
    
    //3. update membership functions
    change=obj_new=0;
    for(int i=min;i<max;i++)
    {
      //3(a)calculate euclidean distance between grey value and class mean for each class
      for(int j=0;j<nr;j++)
      	dist[j]=fabs(i-cmean[j]);
      
      //3(b) calculate membership function for each class
      for(int j=0;j<nr;j++)
      {
	fdum=0;
	for(int k=0;k<nr;k++)
	  fdum+=pow(dist[j]/dist[(j+k)%nr],2./(tau-1.0)); //denominator in JBR97 eq. 10a for multiclass formulation
	memb_new[j][i]=1.0/fdum; //JBR97 eq. 10a
	change+=(memb_new[j][i]-memb[j][i])*(memb_new[j][i]-memb[j][i]);
	obj_new+=fuzzprob[j]*dist[j]*dist[j]; //JBR97 eq. 8
      }
    }
    iter++;
    //printf("%d\t%e\t%e\t%e\n",iter, obj_new, obj_old, change);
    if(change<epsilon) // stopping criteria not in terms of energy function but in total change of membership functions like in JBR97
      break;
    else 
    {
      obj_old=obj_new;
      //if(change>0)
	for(int j=0;j<nr;j++)
	  for(int i=min;i<max;i++)
	    memb[j][i]=memb_new[j][i];
    }
  }
  
  //4. Hardening scheme after iterations
  
  int j=0;
  for(int i=min;i<max-1;i++)
  {
    if(memb[j][i]>memb[j+1][i] && memb[j][i+1]<memb[j+1][i+1])
    {
      t[j]=i;
      j++;
    }
    if(j==nr-1) break;
  }
  return t;
}
int *FuzzyCMeansThresh(double *cdf, int nr, double tau, double percentile)
{
  /*Function: returns vector of thresholds according to Jawahar, Biswas, Ray (1997):Pattern Recognition
  /*1. argument: cumulative histogram of gray values, e.g. via GreyCdf()*/
  /*2. argument: number of classes*/
  /*3. argument: degree of fuzziness tau=[1,+/indef); tau=1 is k-means*/
  
  int max, min, flag, steps, idum, old, upd, clas, *t, iter,lbound,hbound;
  double pdf[256], memb[nr][256], memb_new[nr][256], cmean[nr], fuzzmom[nr], fuzzprob[nr], dist[nr], fdum, change,dstep, obj_old, obj_new;
  double epsilon=10e-4, normdiff;
  
  //Basic Allocations
  t=(int*)calloc(3*(nr-1),sizeof(int));
 
  //1. Initialization of membership function by equidistant intervals between max and min
  for(int j=0;j<nr;j++)for(int i=0;i<256;i++) memb[j][i]=0.;
  pdf[0]=cdf[0];
  flag=0;
  for(int i=1;i<256;i++)
  {
    pdf[i]=cdf[i]-cdf[i-1];
    if(pdf[i]>0 && !flag){ min=i; flag=1;}
    if(pdf[i]>0) max=i;
  }
  dstep=(max-min+epsilon)/nr;
  for(int i=min;i<max;i++) memb[int((i-min)/dstep)][i]=1.0;
  
  
  //Starting the iterations
  iter=0;
  obj_old=1e30;
  while(1)
  {
    //2. update class means
    for(int j=0;j<nr;j++)
    {
      fuzzmom[j]=fuzzprob[j]=0;
      for(int i=min;i<max;i++)
      {
	fdum=pdf[i]*pow(memb[j][i],tau);
	fuzzmom[j]+=i*fdum; //numerator in JBR97, eq. 9
	fuzzprob[j]+=fdum; //denominator in JBR97, eq. 9
      }
      cmean[j]=fuzzmom[j]/fuzzprob[j]; //JBR97, eq. 9
      //printf("%d\t%lf\n",j,cmean[j]);
    }
    
    //3. update membership functions
    change=obj_new=0;
    for(int i=min;i<max;i++)
    {
      //3(a)calculate euclidean distance between grey value and class mean for each class
      for(int j=0;j<nr;j++)
      	dist[j]=fabs(i-cmean[j]);
      
      //3(b) calculate membership function for each class
      for(int j=0;j<nr;j++)
      {
	fdum=0;
	for(int k=0;k<nr;k++)
	  fdum+=pow(dist[j]/dist[(j+k)%nr],2./(tau-1.0)); //denominator in JBR97 eq. 10a for multiclass formulation
	memb_new[j][i]=1.0/fdum; //JBR97 eq. 10a
	change+=(memb_new[j][i]-memb[j][i])*(memb_new[j][i]-memb[j][i]);
	obj_new+=fuzzprob[j]*dist[j]*dist[j]; //JBR97 eq. 8
      }
    }
    iter++;
    //printf("%d\t%e\t%e\t%e\n",iter, obj_new, obj_old, change);
    if(change<epsilon) // stopping criteria not in terms of energy function but in total change of membership functions like in JBR97
      break;
    else 
    {
      obj_old=obj_new;
      //if(change>0)
	for(int j=0;j<nr;j++)
	  for(int i=min;i<max;i++)
	    memb[j][i]=memb_new[j][i];
    }
  }
  
  //4. Hardening scheme after iterations
  int j=0;
  for(int i=min;i<max-1;i++)
  {
    if(memb[j][i]>memb[j+1][i] && memb[j][i+1]<memb[j+1][i+1])
    {
      t[j]=i;
      j++;
    }
    if(j==nr-1) break;
    
  }
  //5. find lower and upper thresholds for which the normed difference of memberships exceeds a certain percentile
  for(int j=0;j<nr-1;j++)
  {
    lbound=j?t[j-1]:min;
    hbound=(j<nr-2)?t[j+1]:max;
    flag=1;
    for(int i=t[j];i>lbound;i--)
    {
      normdiff=(memb[j][i]-memb[j+1][i])/(memb[j][i]+memb[j+1][i]);
      if(normdiff<percentile && flag)
      	t[1*(nr-1)+j]=i;
      else 
	flag=0;
    }
    flag=1;
    for(int i=t[j];i<hbound;i++)
    {
      normdiff=(memb[j+1][i]-memb[j][i])/(memb[j][i]+memb[j+1][i]);
      if(normdiff<percentile && flag)
      	t[2*(nr-1)+j]=i;
      else 
	flag=0;
    }
  }
  return t;
}
#define IGNORE 0.00001
int *MaxEntroThresh(double *cm0, int nr)
{
  
  /*Function: Multilevel thresholding by maximizing sum of partial entropies of each class histogram according to Kapur et al. (1985):Computer Vision, Graphics and Image Processing,29,273-285. */
  /*1. argument: cumulative histogram of gray values, e.g. via GreyCdf*/
  /*2. argument: number of classes*/
  /*Output: vector of nr-1 thresholds*/
  

  double max=0, h, psi/*sum of partial entropies of each clas*/;
  int i,j,k,*t,*b, low, high, flag;
  double ent[256] /*entropy of histogram between 0 and specific gray value*/;
  double lut0[256][256],lut1[256][256]; /* look-up tables */
  double tiny=1e-20;
  
  t=(int*)malloc(3*(nr-1)*sizeof(int));
    for (i=0;i<3*(nr-1);i++)t[i]=0;
    
  //moving class borders to screen the function space  
  b=(int*)malloc((nr+1)*sizeof(int));
  b[0]=0;
  b[nr]=255;
    
  lut0[0][0]=cm0[i]?cm0[i]:tiny;  
  ent[0]=cm0[0];
  for(i=1;i<256;i++)
  {
    lut0[i][0]=cm0[i]==cm0[i-1]?lut0[i-1][0]+tiny:lut0[i-1][0]+(cm0[i]-cm0[i-1]); //cumulative frequency - zeroth moment  (area)
    h=lut0[i][0]-lut0[i-1][0];
    lut1[i][0]=ent[i]=ent[i-1]+(h>0?(h)*log(h):tiny); /*h must not be 0*/
    //printf("%d\t%lf\t%lf\n",i,lut0[i][0],lut1[i][0]);
  }
  for(i=1;i<256;i++)
    for(j=i+1;j<256;j++)
    {
      lut0[j][i]=lut0[j][0]-lut0[i][0];
      lut1[j][i]=ent[j]-ent[i];
    }
  low=1;
  high=255;    
  for(i=1;i<256;i++)
  {
    if(cm0[i]<=IGNORE) low=i+1;
    if(cm0[i]<1.-IGNORE) high=i-1;
  }

  //FIRST LOOP TO FIND THRESHOLDS THAT MAXIMIZE SUM OF PARTIAL ENTROPIES
  for(i=1;i<nr;i++) b[i]=low+i-1;
  while(b[1]<=high-nr+1) //+1 for class one
  {
    //calculate sum of partial entropies
    psi=0;
    for(i=0;i<nr;i++)
      psi+=log(lut0[b[i+1]][b[i]])-lut1[b[i+1]][b[i]]/lut0[b[i+1]][b[i]];

    //if smallest value then store thresholds 
    if(psi>max)
    {
      max=psi;
      for(i=1;i<nr;i++) t[i-1]=b[i];
    }
    
//     for(i=1;i<nr;i++) printf("%d\t",b[i]);
//     printf("%lf\n",psi);
    
    //increment borders
    for(i=1;i<nr;i++) if(b[i]+1<=high-nr+i) flag=i; // mark inner loop b[i] where no overflow occurs. e.g.  five classes with six borders with fixed b[0]=0 and b[5]=256: (fix,1,0,1,1,fix) -> flag=2 because b[3] and b[4] have reached its border
    
    b[flag]++;
    for(i=flag+1;i<nr;i++) b[i]=b[i-1]+1; //this makes the function space triangular
  }
  //printf("\n\n%d\t%d\t%e\n",t[0],t[1],max);
 
  return t;
}

int *MaxEntroThresh(double *cm0, int nr, double percentile)
{
  
  /*Function: Multilevel thresholding by maximizing sum of partial entropies of each class histogram according to Kapur et al. (1985):Computer Vision, Graphics and Image Processing,29,273-285. */
  /*1. argument: cumulative histogram of gray values, e.g. via GreyCdf*/
  /*2. argument: number of classes*/
  /*3. argument: percentile that sets a fuzzy region around the optimal grey threshold*/
  /*Output: vector of 3*(nr-1) thresholds*/
  
  /*EXAMPLE: 3 CLASSES
   * ----------------
   * t[0] low-to-medium -> threshold
   * t[1] medium-to-high -> threshold
   * t[2] low-to-medium -> threshold - percentile
   * t[3] medium-to-high -> threshold - percentile
   * t[4] low-to-medium -> threshold + percentile
   * t[5] medium-to-high -> threshold + percentile
   */
  
  double max=0, h, psi/*sum of partial entropies of each clas*/;
  int i,j,k,*t,*b, low, high, flag;
  double ent[256] /*entropy of histogram between 0 and specific gray value*/;
  double lut0[256][256],lut1[256][256]; /* look-up tables */
  double tiny=1e-20;
  
  t=(int*)malloc(3*(nr-1)*sizeof(int));
    for (i=0;i<3*(nr-1);i++)t[i]=0;
    
  //moving class borders to screen the function space  
  b=(int*)malloc((nr+1)*sizeof(int));
  b[0]=0;
  b[nr]=255;
    
  lut0[0][0]=cm0[0]?cm0[0]:tiny;  
  ent[0]=cm0[0];
  for(i=1;i<256;i++)
  {
    lut0[i][0]=cm0[i]==cm0[i-1]?lut0[i-1][0]+tiny:lut0[i-1][0]+(cm0[i]-cm0[i-1]); //cumulative frequency - zeroth moment  (area)
    h=lut0[i][0]-lut0[i-1][0];
    lut1[i][0]=ent[i]=ent[i-1]+(h>0?(h)*log(h):tiny); /*h must not be 0*/
    //printf("%d\t%lf\t%lf\n",i,lut0[i][0],lut1[i][0]);
  }
  for(i=1;i<256;i++)
    for(j=i+1;j<256;j++)
    {
      lut0[j][i]=lut0[j][0]-lut0[i][0];
      lut1[j][i]=ent[j]-ent[i];
    }
  low=1;
  high=255;    
  for(i=1;i<256;i++)
  {
    if(cm0[i]<=IGNORE) low=i+1;
    if(cm0[i]<1.-IGNORE) high=i-1;
  }

  //FIRST LOOP TO FIND THRESHOLDS THAT MAXIMIZE SUM OF PARTIAL ENTROPIES
  for(i=1;i<nr;i++) b[i]=low+i-1;
  while(b[1]<=high-nr+1) //+1 for class one
  {
    //calculate sum of partial entropies
    psi=0;
    for(i=0;i<nr;i++)
      psi+=log(lut0[b[i+1]][b[i]])-lut1[b[i+1]][b[i]]/lut0[b[i+1]][b[i]];

    //if smallest value then store thresholds 
    if(psi>max)
    {
      max=psi;
      for(i=1;i<nr;i++) t[i-1]=b[i];
    }
    
//     for(i=1;i<nr;i++) printf("%d\t",b[i]);
//     printf("%lf\n",psi);
    
    //increment borders
    for(i=1;i<nr;i++) if(b[i]+1<=high-nr+i) flag=i; // mark inner loop b[i] where no overflow occurs. e.g.  five classes with six borders with fixed b[0]=0 and b[5]=256: (fix,1,0,1,1,fix) -> flag=2 because b[3] and b[4] have reached its border
    
    b[flag]++;
    for(i=flag+1;i<nr;i++) b[i]=b[i-1]+1; //this makes the function space triangular
  }
  //printf("\n\n%d\t%d\t%e\n",t[0],t[1],max);
 
  //SECOND LOOP TO FIND PERCENTILES THAT SERVE AS LOWER THRESHOLD FOR REGION GROWING IN BI-LEVEL SEGMENTATION
  for(i=1;i<nr;i++) b[i]=t[i-1]; //b[0]=0 and remains b[nr]=256 remains unaffected
  for(i=1;i<nr;i++)
  {
    for(j=b[i];j>low+i;j--)
    {
      psi=0;
      for(k=0;k<nr;k++)
      {
	if(i==k+1)psi+=log(lut0[j][b[k]])-lut1[j][b[k]]/lut0[j][b[k]];
	else if(i==k) psi+=log(lut0[b[k+1]][j])-lut1[b[k+1]][j]/lut0[b[k+1]][j];
	else psi+=log(lut0[b[k+1]][b[k]])-lut1[b[k+1]][b[k]]/lut0[b[k+1]][b[k]];
      }
      if(psi>max*percentile) t[1*(nr-1)-1+i]=j;
    }
    for(j=b[i];j<=high-nr+i;j++)
    {
      psi=0;
      for(k=0;k<nr;k++)
      {
	if(i==k+1)psi+=log(lut0[j][b[k]])-lut1[j][b[k]]/lut0[j][b[k]];
	else if(i==k) psi+=log(lut0[b[k+1]][j])-lut1[b[k+1]][j]/lut0[b[k+1]][j];
	else psi+=log(lut0[b[k+1]][b[k]])-lut1[b[k+1]][b[k]]/lut0[b[k+1]][b[k]];
      }
      if(psi>max*percentile) t[2*(nr-1)-1+i]=j;
    }
  }  
  return t;
}
int *MinErrThresh(double *cm0, int nr)
{
  /*Function: Multilevel thresholding by minimizing the overlap error of fitted Gaussians according to Kittler & Illingworth (1986): Pattern Recognition ,19(1),41-47. */
  /*1. argument: cumulative histogram of gray values, e.g. via GreyCdf*/
  /*2. argument: number of classes*/
  /*Output: vector of nr-1 thresholds*/  
  
  double min=1e30,merr /*minimum error, i.e. overlap between fitted Gaussians*/;
  int i,j,k,*t,*b, flag,low, high;
  double cm1[256] /*cumulative first moment*/, cm2 /*cumulative second moment*/ ;
  double lut0[256][256],loglut0[256][256], lut1[256][256], lut2[256][256]; /* look-up tables */
  double tiny=1e-20;
  
  //threshold triples to be returned
  t=(int*)malloc(3*(nr-1)*sizeof(int));
    for (i=0;i<3*(nr-1);i++)t[i]=0;
    
  //moving class borders to screen the function space  
  b=(int*)malloc((nr+1)*sizeof(int));
  b[0]=0;
  b[nr]=255;
    
  lut0[0][0]=cm0[0]?cm0[0]:tiny;
  cm1[0]=0.0;
  for(i=1;i<256;i++)
  {

    lut0[i][0]=cm0[i]==cm0[i-1]?lut0[i-1][0]+tiny:lut0[i-1][0]+(cm0[i]-cm0[i-1]); //cumulative frequency - zeroth moment  (area)
    loglut0[i][0]=log(lut0[i][0]); //log of zeroth moment
    cm1[i]=i*(lut0[i][0]-lut0[i-1][0])+cm1[i-1]; //cumulative first moment 
    

  }
  for(i=1;i<256;i++)
  {
    lut1[i][0]=cm1[i]/lut0[i][0]; //first moment (mean)
    cm2=0;
    for(j=0;j<=i;j++)
      cm2+=(j-lut1[i][0])*(j-lut1[i][0])*(lut0[j][0]-lut0[j-1][0]); // sum second moment for a given mean
    cm2/=lut0[i][0]; // second moment (variance)  
    lut2[i][0]= log(sqrt(cm2?cm2:tiny)); //log root of second moment (variance)
    //printf("%d\t%e\t%e\t%lf\t%lf\n",i,lut0[i][0],loglut0[i][0],lut1[i][0],lut2[i][0]);
  }
  
  for(i=1;i<256;i++)
    for(j=i+1;j<256;j++)
    {
      lut0[j][i]=lut0[j][0]-lut0[i][0];
      loglut0[j][i]=log(lut0[j][i]);
      lut1[j][i]=(cm1[j]-cm1[i])/lut0[j][i];
      cm2=0;
      for(k=i;k<j;k++)
      {
	cm2+=(k-lut1[j][i])*(k-lut1[j][i])*(lut0[k][0]-lut0[k-1][0]); 
      }
      cm2/=lut0[j][i];  
      lut2[j][i]= log(sqrt(cm2?cm2:tiny));
    }

  //reduce grey value range by IGNORE-percentile on both ends  
  low=1;
  high=255;
  for(i=1;i<256;i++)
  {
    if(cm0[i]<=IGNORE) low=i+1;
    if(cm0[i]<1.-IGNORE) high=i-1;
  }

  //FIRST LOOP TO FIND THRESHOLD-COMBINATION THAT MINIMIZES OVERLAP ERROR
  for(i=1;i<nr;i++) b[i]=low+i-1;
  while(b[1]<=high-nr+1) //+1 for class one
  {
    //calculate overlap error
    merr=0;
    for(i=0;i<nr;i++)
    {
      merr+=lut0[b[i+1]][b[i]]*(lut2[b[i+1]][b[i]]-loglut0[b[i+1]][b[i]]);
      
    }
    merr=1+2*merr;

    //if smallest value then store thresholds 
    if(merr<min)
    {
      min=merr;
      for(i=1;i<nr;i++) t[i-1]=b[i];
    }
    
//     for(i=1;i<nr;i++) printf("%d\t",b[i]);
//     printf("%lf\n",merr);
    
    //increment borders
    for(i=1;i<nr;i++) if(b[i]+1<=high-nr+i) flag=i; // mark inner loop b[i] where no overflow occurs. e.g.  five classes with six borders with fixed b[0]=0 and b[5]=256: (fix,1,0,1,1,fix) -> flag=2 because b[3] and b[4] have reached its border
    
    b[flag]++;
    for(i=flag+1;i<nr;i++) b[i]=b[i-1]+1; //this makes the function space triangular
  }
  
  return t;
}

int *MinErrThresh(double *cm0, int nr, double percentile)
{
  /*Function: Multilevel thresholding by minimizing the overlap error of fitted Gaussians according to Kittler & Illingworth (1986): Pattern Recognition ,19(1),41-47. */
  /*1. argument: cumulative histogram of gray values, e.g. via GreyCdf*/
  /*2. argument: number of classes*/
  /*3. argument: percentile that sets a fuzzy region around the optimal grey threshold*/
  /*Output: vector of 3*(nr-1) thresholds*/  
  
    /*EXAMPLE: 3 CLASSES
   * ----------------
   * t[0] low-to-medium -> threshold
   * t[1] medium-to-high -> threshold
   * t[2] low-to-medium -> threshold - percentile
   * t[3] medium-to-high -> threshold - percentile
   * t[4] low-to-medium -> threshold + percentile
   * t[5] medium-to-high -> threshold + percentile
   */
    
  double min=1e30,merr /*minimum error, i.e. overlap between fitted Gaussians*/;
  int i,j,k,*t,*b, flag,low, high;
  double cm1[256] /*cumulative first moment*/, cm2 /*cumulative second moment*/ ;
  double lut0[256][256],loglut0[256][256], lut1[256][256], lut2[256][256]; /* look-up tables */
  double tiny=1e-20;
  
  //threshold triples to be returned
  t=(int*)malloc(3*(nr-1)*sizeof(int));
    for (i=0;i<3*(nr-1);i++)t[i]=0;
    
  //moving class borders to screen the function space  
  b=(int*)malloc((nr+1)*sizeof(int));
  b[0]=0;
  b[nr]=255;
    
  lut0[0][0]=cm0[0]?cm0[0]:tiny;
  cm1[0]=0.0;
  for(i=1;i<256;i++)
  {

    lut0[i][0]=cm0[i]==cm0[i-1]?lut0[i-1][0]+tiny:lut0[i-1][0]+(cm0[i]-cm0[i-1]); //cumulative frequency - zeroth moment  (area)
    loglut0[i][0]=log(lut0[i][0]); //log of zeroth moment
    cm1[i]=i*(lut0[i][0]-lut0[i-1][0])+cm1[i-1]; //cumulative first moment 
    

  }
  for(i=1;i<256;i++)
  {
    lut1[i][0]=cm1[i]/lut0[i][0]; //first moment (mean)
    cm2=0;
    for(j=0;j<=i;j++)
      cm2+=(j-lut1[i][0])*(j-lut1[i][0])*(lut0[j][0]-lut0[j-1][0]); // sum second moment for a given mean
    cm2/=lut0[i][0]; // second moment (variance)  
    lut2[i][0]= log(sqrt(cm2?cm2:tiny)); //log root of second moment (variance)
    //printf("%d\t%e\t%e\t%lf\t%lf\n",i,lut0[i][0],loglut0[i][0],lut1[i][0],lut2[i][0]);
  }
  
  for(i=1;i<256;i++)
    for(j=i+1;j<256;j++)
    {
      lut0[j][i]=lut0[j][0]-lut0[i][0];
      loglut0[j][i]=log(lut0[j][i]);
      lut1[j][i]=(cm1[j]-cm1[i])/lut0[j][i];
      cm2=0;
      for(k=i;k<j;k++)
      {
	cm2+=(k-lut1[j][i])*(k-lut1[j][i])*(lut0[k][0]-lut0[k-1][0]); 
      }
      cm2/=lut0[j][i];  
      lut2[j][i]= log(sqrt(cm2?cm2:tiny));
    }

  //reduce grey value range by IGNORE-percentile on both ends  
  low=1;
  high=255;
  for(i=1;i<256;i++)
  {
    if(cm0[i]<=IGNORE) low=i+1;
    if(cm0[i]<1.-IGNORE) high=i-1;
  }

  //FIRST LOOP TO FIND THRESHOLD-COMBINATION THAT MINIMIZES OVERLAP ERROR
  for(i=1;i<nr;i++) b[i]=low+i-1;
  while(b[1]<=high-nr+1) //+1 for class one
  {
    //calculate overlap error
    merr=0;
    for(i=0;i<nr;i++)
    {
      merr+=lut0[b[i+1]][b[i]]*(lut2[b[i+1]][b[i]]-loglut0[b[i+1]][b[i]]);
      
    }
    merr=1+2*merr;

    //if smallest value then store thresholds 
    if(merr<min)
    {
      min=merr;
      for(i=1;i<nr;i++) t[i-1]=b[i];
    }
    
//     for(i=1;i<nr;i++) printf("%d\t",b[i]);
//     printf("%lf\n",merr);
    
    //increment borders
    for(i=1;i<nr;i++) if(b[i]+1<=high-nr+i) flag=i; // mark inner loop b[i] where no overflow occurs. e.g.  five classes with six borders with fixed b[0]=0 and b[5]=256: (fix,1,0,1,1,fix) -> flag=2 because b[3] and b[4] have reached its border
    
    b[flag]++;
    for(i=flag+1;i<nr;i++) b[i]=b[i-1]+1; //this makes the function space triangular
  }
  
  //SECOND LOOP TO FIND PERCENTILES THAT SERVE AS A FUZZY REGION
  for(i=1;i<nr;i++) b[i]=t[i-1]; //b[0]=0 and remains b[nr]=256 remains unaffected
  for(i=1;i<nr;i++)
  {
    for(j=b[i];j>low+i;j--)
    {
      merr=0;
      for(k=0;k<nr;k++)
      {
	if(i==k+1) merr+=lut0[j][b[k]]*(lut2[j][b[k]]-loglut0[j][b[k]]);
	else if(i==k) merr+=lut0[b[k+1]][j]*(lut2[b[k+1]][j]-loglut0[b[k+1]][j]);
	else merr+=lut0[b[k+1]][b[k]]*(lut2[b[k+1]][b[k]]-loglut0[b[k+1]][b[k]]);
      }
      merr=1+2*merr;
      if(merr<min/percentile) t[1*(nr-1)-1+i]=j;
    }
    for(j=b[i];j<=high-nr+i;j++)
    {
      merr=0;
      for(k=0;k<nr;k++)
      {
	if(i==k+1) merr+=lut0[j][b[k]]*(lut2[j][b[k]]-loglut0[j][b[k]]);
	else if(i==k) merr+=lut0[b[k+1]][j]*(lut2[b[k+1]][j]-loglut0[b[k+1]][j]);
	else merr+=lut0[b[k+1]][b[k]]*(lut2[b[k+1]][b[k]]-loglut0[b[k+1]][b[k]]);
      }
      merr=1+2*merr;
      if(merr<min/percentile) t[2*(nr-1)-1+i]=j;
    }
  }
  return t;
}
int *MaxVarThresh(double *cm0, int nr)
{
  /*Function: Multilevel thresholding by maximizing between-class variance according to Liao et al. (2001):Journal of Information science and Engineering,17,713-727. */
  /*1. argument: cumulative histogram of gray values, e.g. via GreyCdf*/
  /*2. argument: number of classes*/
  /*Output: vector of nr-1 thresholds*/ 
  

  double max=0,bcv /*between class variance*/;
  int i,j,k,*t,*b, flag,low, high;
  double lut0[256][256],lut1[256][256]; /* look-up tables */
  double tiny=1e-20;
  
  //threshold triples to be returned
  t=(int*)malloc(3*(nr-1)*sizeof(int));
    for (i=0;i<3*(nr-1);i++)t[i]=0;
    
  //moving class borders to screen the function space  
  b=(int*)malloc((nr+1)*sizeof(int));
  b[0]=0;
  b[nr]=255;
    
  lut0[0][0]=cm0[0]?cm0[0]:tiny;  
  lut1[0][0]=0.0;
  for(i=1;i<256;i++)
  {
    lut0[i][0]=cm0[i]==cm0[i-1]?lut0[i-1][0]+tiny:lut0[i-1][0]+(cm0[i]-cm0[i-1]); //cumulative frequency - zeroth moment  (area)
    lut1[i][0]=i*(lut0[i][0]-lut0[i-1][0])+lut1[i-1][0];
  }
  for(i=1;i<256;i++)
    for(j=i+1;j<256;j++)
    {
      lut0[j][i]=lut0[j][0]-lut0[i][0];
      lut1[j][i]=lut1[j][0]-lut1[i][0];
    }
  low=1;
  high=255;    
  for(i=1;i<256;i++)
  {
    if(cm0[i]<=IGNORE) low=i+1;
    if(cm0[i]<1.-IGNORE) high=i-1;
  }
  

  //FIRST LOOP TO FIND THRESHOLDS THAT MAXIMIZE BETWEEN-CLASS VARIANCE
  for(i=1;i<nr;i++) b[i]=low+i-1;
  while(b[1]<=high-nr+1) //+1 for class one
  {
    //calculate between class variance
    bcv=0;
    for(i=0;i<nr;i++)
      bcv+=lut1[b[i+1]][b[i]]*lut1[b[i+1]][b[i]]/lut0[b[i+1]][b[i]];

    //if smallest value then store thresholds 
    if(bcv>max)
    {
      max=bcv;
      for(i=1;i<nr;i++) t[i-1]=b[i];
    }
    
//     for(i=1;i<nr;i++) printf("%d\t",b[i]);
//     printf("%lf\n",bcv);
    
    //increment borders
    for(i=1;i<nr;i++) if(b[i]+1<=high-nr+i) flag=i; // mark inner loop b[i] where no overflow occurs. e.g.  five classes with six borders with fixed b[0]=0 and b[5]=256: (fix,1,0,1,1,fix) -> flag=2 because b[3] and b[4] have reached its border
    
    b[flag]++;
    for(i=flag+1;i<nr;i++) b[i]=b[i-1]+1; //this makes the function space triangular
  }  
//   printf("\n\n%d\t%d\t%e\n",t[0],t[1],max);


  return t;
}

int *MaxVarThresh(double *cm0, int nr, double percentile)
{
  /*Function: Multilevel thresholding by maximizing between-class variance according to Liao et al. (2001):Journal of Information science and Engineering,17,713-727. */
  /*1. argument: cumulative histogram of gray values, e.g. via GreyCdf*/
  /*2. argument: number of classes*/
  /*3. argument: percentile that sets a fuzzy region around the optimal grey threshold*/
  /*Output: vector of 3*(nr-1) thresholds*/  
 
  /*EXAMPLE: 3 CLASSES
  * ----------------
  * t[0] low-to-medium -> threshold
  * t[1] medium-to-high -> threshold
  * t[2] low-to-medium -> threshold - percentile
  * t[3] medium-to-high -> threshold - percentile
  * t[4] low-to-medium -> threshold + percentile
  * t[5] medium-to-high -> threshold + percentile
  */
    
  double max=0,bcv /*between class variance*/;
  int i,j,k,*t,*b, flag,low, high;
  double lut0[256][256],lut1[256][256]; /* look-up tables */
  double tiny=1e-20;
  
  //threshold triples to be returned
  t=(int*)malloc(3*(nr-1)*sizeof(int));
    for (i=0;i<3*(nr-1);i++)t[i]=0;
    
  //moving class borders to screen the function space  
  b=(int*)malloc((nr+1)*sizeof(int));
  b[0]=0;
  b[nr]=255;
    
  lut0[0][0]=cm0[0]?cm0[0]:tiny;  
  lut1[0][0]=0.0;
  for(i=1;i<256;i++)
  {
    lut0[i][0]=cm0[i]==cm0[i-1]?lut0[i-1][0]+tiny:lut0[i-1][0]+(cm0[i]-cm0[i-1]); //cumulative frequency - zeroth moment  (area)
    lut1[i][0]=i*(lut0[i][0]-lut0[i-1][0])+lut1[i-1][0];
  }
  for(i=1;i<256;i++)
    for(j=i+1;j<256;j++)
    {
      lut0[j][i]=lut0[j][0]-lut0[i][0];
      lut1[j][i]=lut1[j][0]-lut1[i][0];
    }
  low=1;
  high=255;    
  for(i=1;i<256;i++)
  {
    if(cm0[i]<=IGNORE) low=i+1;
    if(cm0[i]<1.-IGNORE) high=i-1;
  }
  

  //FIRST LOOP TO FIND THRESHOLDS THAT MAXIMIZE BETWEEN-CLASS VARIANCE
  for(i=1;i<nr;i++) b[i]=low+i-1;
  while(b[1]<=high-nr+1) //+1 for class one
  {
    //calculate between class variance
    bcv=0;
    for(i=0;i<nr;i++)
      bcv+=lut1[b[i+1]][b[i]]*lut1[b[i+1]][b[i]]/lut0[b[i+1]][b[i]];

    //if smallest value then store thresholds 
    if(bcv>max)
    {
      max=bcv;
      for(i=1;i<nr;i++) t[i-1]=b[i];
    }
    
//     for(i=1;i<nr;i++) printf("%d\t",b[i]);
//     printf("%lf\n",bcv);
    
    //increment borders
    for(i=1;i<nr;i++) if(b[i]+1<=high-nr+i) flag=i; // mark inner loop b[i] where no overflow occurs. e.g.  five classes with six borders with fixed b[0]=0 and b[5]=256: (fix,1,0,1,1,fix) -> flag=2 because b[3] and b[4] have reached its border
    
    b[flag]++;
    for(i=flag+1;i<nr;i++) b[i]=b[i-1]+1; //this makes the function space triangular
  }  
//   printf("\n\n%d\t%d\t%e\n",t[0],t[1],max);

  //SECOND LOOP TO FIND PERCENTILES THAT SERVE AS A FUZZY REGION
  for(i=1;i<nr;i++) b[i]=t[i-1]; //b[0]=0 and remains b[nr]=256 remains unaffected
  for(i=1;i<nr;i++)
  {
    for(j=b[i];j>low+i;j--)
    {
      bcv=0;
      for(k=0;k<nr;k++)
      {
	if(i==k+1) bcv+=lut1[j][b[k]]*lut1[j][b[k]]/lut0[j][b[k]];
	else if(i==k) bcv+=lut1[b[k+1]][j]*lut1[b[k+1]][j]/lut0[b[k+1]][j];
	else bcv+=lut1[b[k+1]][b[k]]*lut1[b[k+1]][b[k]]/lut0[b[k+1]][b[k]];
      }
      if(bcv>max*percentile) t[1*(nr-1)-1+i]=j;
    }
    for(j=b[i];j<=high-nr+i;j++)
    {
      bcv=0;
      for(k=0;k<nr;k++)
      {
	if(i==k+1) bcv+=lut1[j][b[k]]*lut1[j][b[k]]/lut0[j][b[k]];
	else if(i==k) bcv+=lut1[b[k+1]][j]*lut1[b[k+1]][j]/lut0[b[k+1]][j];
	else bcv+=lut1[b[k+1]][b[k]]*lut1[b[k+1]][b[k]]/lut0[b[k+1]][b[k]];
      }
      if(bcv>max*percentile) t[2*(nr-1)-1+i]=j;
    }
  }
  return t;
}


#define MAX(a,b) ((a>b)?(a):(b))
#define MIN(a,b) ((a<b)?(a):(b))

int *ShapeThresh(double *cdf, int nr)
{
  /*Function: 
  /*1. argument: cumulative histogram of gray values, e.g. via GreyCdf*/
  /*2. argument: number of classes*/
  /*Output: threshold vector detected in valleys (or at maximum curvature locations if nr > nr. of peaks after histogram smoothing)*/ 
  
  int  *t, iter, max_nr=0, t_nr=0, max[256], mask[256],flag=0, left, right, pos, wnd=3, kernel, maxcurv_loc[256], maxcurv_nr=0, highest, overflow, full;
  double pdf_old[256],pdf_new[256],angle[256],curv[256], gssn[3]={0.2261,0.5478,0.2261}, maxcurv[nr-1], maxcurv_val[256],fdum;
  
  
  //Basic Allocations
  t=(int*)calloc(nr-1,sizeof(int));
 
  
  pdf_old[0]=cdf[0];
  for(int i=1;i<256;i++)
  {
    pdf_old[i]=cdf[i]-cdf[i-1];
    pdf_new[i]=curv[i]=angle[i]=mask[i]=maxcurv_loc[i]=maxcurv_val[i]=0;
  }

  for(int i=1;i<255;i++)
  {
    if(pdf_old[i]>pdf_old[i-1] && pdf_old[i]>pdf_old[i+1])
    {
      max[max_nr]=i;
      max_nr++;
    }
  }

  if(max_nr<nr)
    flag=-1; // after valley search fill remaining thresholds with maximum curvature locations
  else if(max_nr==nr)
    flag=1; // apply valley search in interval
  else 
    flag=0; // enter the Gaussian smoothing loop to reduce the number of maxima

  
  
  //Starting the Gaussian smoothing iterations
  iter=0;
  while(!flag)
  {
    //smoothing done here
    for(int i=0;i<256;i++)
    {
      pdf_new[i]=0;
      for(int j=0;j<3;j++)
      {
	pos=MAX(0,MIN(i+j-1,255));
	pos=i+j-1;
	pdf_new[i]+=gssn[j]*pdf_old[pos];
      }
      //printf("%d\t%d\t%lf\n",iter,i,pdf_new[i]);
    }
    //printf("\n\n");
    max_nr=0;
    
    //find peaks
    for(int i=0;i<255;i++)
    {
      if(pdf_new[i]>pdf_new[i-1] && pdf_new[i]>pdf_new[i+1])
      {
	max[max_nr]=i;
	max_nr++;
      }
    }

    //check for stopping criteria
    if(max_nr<nr)
      flag=-1;
    else
    {
      for(int i=0;i<256;i++) pdf_old[i]=pdf_new[i];
      if(max_nr==nr) flag=1;
    }
    iter++;
  }
  
  
 
  //mask windows around peaks for subsequent maximum curvature detection
  for(int i=0;i<max_nr;i++)
    for(int j=-wnd;j<=wnd;j++)
    {
      pos=MAX(0,MIN(max[i]+j,255));
      mask[pos]=1;
    }
  
  if(flag) //find minima in valleys and if flag=-1 fill only the rest with curvature method (see below)
  {
    full=0;
    for(int i=1;i<255;i++)
    {
      for(int j=1;j<=wnd;j++)
      {
	left=MAX(i-j,0);
	right=MIN(i+j,255);
	angle[i]+=(pdf_old[right]-pdf_old[left])/(2*j);
      }
      angle[i]/=wnd;
    }
    
    for(int i=wnd;i<256-wnd;i++)
    {
      for(int j=1;j<=wnd;j++)
      {
	left=MAX(i-j,0);
	right=MIN(i+j,255);
	curv[i]+=fabs(angle[right]-angle[left]);
      }
      curv[i]/=wnd;
      //printf("%d\t%lf\t%e\t%e\t%d\n",i,pdf_old[i],curv[i], angle[i], mask[i]);
      if(i>max[0] && i<max[max_nr-1] && !full)
	if((pdf_old[i]<pdf_old[i-1] && pdf_old[i]<pdf_old[i+1]) || (pdf_old[i]<pdf_old[i-1] && !pdf_old[i] ))
	{
	  t[t_nr]=i;
	  maxcurv[t_nr]=curv[i];
	  t_nr++;
	  for(int j=-wnd;j<=wnd;j++)
	  {
	    pos=MAX(0,MIN(i+j,255));
	    mask[pos]=1;
	  }
	  if(t_nr==max_nr-1) 
	    full=1;
	}

    }
  }
  if(flag==-1) //find additional thresholds for maximum curvature
  {
    //find potential candidates of maximum curvature locations
    maxcurv_nr=0;
    for(int i=1;i<255;i++)
    {
      if(curv[i]>curv[i-1] && curv[i]>curv[i+1] && !mask[i])
      {
	maxcurv_loc[maxcurv_nr]=i;
	maxcurv_val[maxcurv_nr]=curv[i];
	maxcurv_nr++;
      }	
    }
    while(t_nr<nr-1) //detect the location with highest, second-highest, etc. curvature until all remaining thresholds are filled
    {
      fdum=0;
      for(int i=0;i<maxcurv_nr;i++)
      {
	if(maxcurv_val[i]>fdum)
	{
	  fdum=maxcurv_val[i];
	  highest=i;
	}
	//printf("%d\t%d\t%d\t%lf\t%lf\n",i,maxcurv_loc[i],highest,maxcurv_val[i],fdum);
      }
      //printf("\n\n");
      t[t_nr]=maxcurv_loc[highest];
      maxcurv[t_nr]=maxcurv_val[highest];
      maxcurv_val[highest]=0;
      t_nr++;
      //Tsai (p. 656) states that max. curvature points have to be separated by a spacing of at least wnd
      for(int j=-wnd;j<=wnd;j++)
      {
	pos=MAX(0,MIN(highest+j,255));
	mask[pos]=1;
      } 
      for(int i=0;i<maxcurv_nr;i++)
	if(mask[maxcurv_loc[i]]) maxcurv_val[i]=0;
    }
  }
  
  

  return t;
}

int *ShapeThresh(double *cdf, int nr, double percentile)
{
  /*Function: 
  /*1. argument: cumulative histogram of gray values, e.g. via GreyCdf*/
  /*2. argument: number of classes*/
  /*3. argument: percentile of gradient or curvature, respectively*/
  /*Output: threshold vector detected in valleys (or at maximum curvature locations if nr > nr. of peaks after histogram smoothing)*/ 
  
  int  *t, iter, max_nr=0, t_nr=0, max[256], mask[256],flag=0, left, right, pos, wnd=3, kernel, maxcurv_loc[256], maxcurv_nr=0, highest, overflow, full,type[nr-1];
  double pdf_old[256],pdf_new[256],angle[256],curv[256], gssn[3]={0.2261,0.5478,0.2261}, maxcurv[nr-1], maxcurv_val[256],fdum,fdum2,rel_curv, rel_angle,maxangle[nr-1][2];
  
  
  //Basic Allocations
  t=(int*)calloc(3*(nr-1),sizeof(int));
 
  
  pdf_old[0]=cdf[0];
  pdf_new[0]=curv[0]=angle[0]=mask[0]=maxcurv_loc[0]=maxcurv_val[0]=0;
  for(int i=1;i<256;i++)
  {
    pdf_old[i]=cdf[i]-cdf[i-1];
    pdf_new[i]=curv[i]=angle[i]=mask[i]=maxcurv_loc[i]=maxcurv_val[i]=0;
  }

  for(int i=1;i<255;i++)
  {
    if(pdf_old[i]>pdf_old[i-1] && pdf_old[i]>pdf_old[i+1])
    {
      max[max_nr]=i;
      max_nr++;
    }
  }

  if(max_nr<nr)
    flag=-1; // after valley search fill remaining thresholds with maximum curvature locations
  else if(max_nr==nr)
    flag=1; // apply valley search in interval
  else 
    flag=0; // enter the Gaussian smoothing loop to reduce the number of maxima

  
  
  //Starting the iterations
  iter=0;
  while(!flag)
  {
    //smoothing done here
    for(int i=0;i<256;i++)
    {
      pdf_new[i]=0;
      for(int j=0;j<3;j++)
      {
	pos=MAX(0,MIN(i+j-1,255));
	pos=i+j-1;
	pdf_new[i]+=gssn[j]*pdf_old[pos];
      }
      //printf("%d\t%d\t%lf\n",iter,i,pdf_new[i]);
    }
    //printf("\n\n");
    
    //find peaks
    max_nr=0;
    for(int i=wnd;i<256-wnd;i++)
    {
      if(pdf_new[i]>pdf_new[i-1] && pdf_new[i]>=pdf_new[i+1])
      {
	max[max_nr]=i;
	max_nr++;
      }
    }

    //check for stopping criteria
    if(max_nr<nr)
      flag=-1;
    else
    {
      for(int i=0;i<256;i++) pdf_old[i]=pdf_new[i];
      if(max_nr==nr) flag=1;
    }
    iter++;
  }
  
  
 
  //mask windows around peaks for subsequent maximum curvature detection
  for(int i=0;i<max_nr;i++)
    for(int j=-wnd;j<=wnd;j++)
    {
      pos=MAX(0,MIN(max[i]+j,255));
      mask[pos]=1;
    }
  
  if(flag) //find minima in valleys and if flag=-1 fill only the rest with curvature method (see below)
  {
    full=0;
    for(int i=1;i<255;i++)
    {
      for(int j=1;j<=wnd;j++)
      {
	left=MAX(i-j,0);
	right=MIN(i+j,255);
	angle[i]+=(pdf_old[right]-pdf_old[left])/(2*j);
      }
      angle[i]/=wnd;
    }
    
    //determine gradient maximum (right shoulder) and minimum (left shoulder) with peak interval 
    
    for(int j=0;j<max_nr-1;j++)
    {
      fdum=fdum2=0;
      for(int i=max[j];i<max[j+1];i++)
      {
	//negative gradients on left sholder
	if(angle[i]<angle[i-1] && angle[i]<angle[i+1])
	{
	  if(fdum<fabs(angle[i]))
	  {
	    fdum=fabs(angle[i]);
	    maxangle[j][0]=angle[i];
	  }
	}
	//positive gradient on right shoulder
	if(angle[i]>angle[i-1] && angle[i]>angle[i+1])
	{
	  if(fdum2<fabs(angle[i]))
	  {
	    fdum2=fabs(angle[i]);
	    maxangle[j][1]=angle[i];
	  }
	}
      }
      
    }
    
    for(int i=wnd;i<256-wnd;i++)
    {
      for(int j=1;j<=wnd;j++)
      {
	left=MAX(i-j,0);
	right=MIN(i+j,255);
	curv[i]+=fabs(angle[right]-angle[left]);
      }
      curv[i]/=wnd;
      //printf("%d\t%lf\t%e\t%e\t%d\n",i,pdf_old[i],curv[i], angle[i], mask[i]);
      if(i>max[0] && i<max[max_nr-1] && !full)
	if((pdf_old[i]<pdf_old[i-1] && pdf_old[i]<pdf_old[i+1]) || (pdf_old[i]<pdf_old[i-1] && !pdf_old[i] ))
	{
	  t[t_nr]=i;
	  type[t_nr]=1; //gradient-based
	  maxcurv[t_nr]=curv[i];
	  t_nr++;
	  for(int j=-wnd;j<=wnd;j++)
	  {
	    pos=MAX(0,MIN(i+j,255));
	    mask[pos]=1;
	  }
	  if(t_nr==max_nr-1) 
	    full=1;
	}

    }
  }
  if(flag==-1) //find additional thresholds for maximum curvature
  {
    //find potential candidates of maximum curvature locations
    maxcurv_nr=0;
    for(int i=1;i<255;i++)
    {
      if(curv[i]>curv[i-1] && curv[i]>curv[i+1] && !mask[i])
      {
	maxcurv_loc[maxcurv_nr]=i;
	maxcurv_val[maxcurv_nr]=curv[i];
	maxcurv_nr++;
      }	
    }
    while(t_nr<nr-1) //detect the location with highest, second-highest, etc. curvature until all remaining thresholds are filled
    {
      fdum=0;
      for(int i=0;i<maxcurv_nr;i++)
      {
	if(maxcurv_val[i]>fdum)
	{
	  fdum=maxcurv_val[i];
	  highest=i;
	}
	//printf("%d\t%d\t%d\t%lf\t%lf\n",i,maxcurv_loc[i],highest,maxcurv_val[i],fdum);
      }
      //printf("\n\n");
      t[t_nr]=maxcurv_loc[highest];
      type[t_nr]=-1; //curvature-based
      maxcurv[t_nr]=maxcurv_val[highest];
      maxcurv_val[highest]=0;
      t_nr++;
      //Tsai (p. 656) states that max. curvature points have to be separated by a spacing of at least wnd
      for(int j=-wnd;j<=wnd;j++)
      {
	pos=MAX(0,MIN(highest+j,255));
	mask[pos]=1;
      } 
      for(int i=0;i<maxcurv_nr;i++)
	if(mask[maxcurv_loc[i]]) maxcurv_val[i]=0;
    }
  }
  
  //find fuzzy ranges via percentile of gradient or curvature,respectively (reasonable values are 0.5-0.9)
  for(int j=0;j<nr-1;j++)
  {
    if(type[j]==1) //for valleys neighboring peaks define the search limit
    {
      left=max[j];
      right=max[j+1];
      flag=1;
      for(int i=t[j];i>left;i--)
      {
	rel_angle=fabs(angle[i]/maxangle[j][0]);
	if(rel_angle<percentile && flag)
	  t[1*(nr-1)+j]=i;
	else 
	  flag=0;
      }
      flag=1;
      for(int i=t[j];i<right;i++)
      {
	rel_angle=fabs(angle[i]/maxangle[j][1]);
	if(rel_angle<percentile && flag)
	  t[2*(nr-1)+j]=i;
	else 
	  flag=0;
      }      
    }
    else //for maximum curvature points there is no need for search limits, as there are always some adjacent zeros somewhere
    {
      left=1;
      right=255;      
    

      flag=1;
      for(int i=t[j];i>left;i--)
      {
	rel_curv=curv[i]/maxcurv[j];
	if(rel_curv>percentile && flag)
	  t[1*(nr-1)+j]=i;
	else 
	  flag=0;
      }
      flag=1;
      for(int i=t[j];i<right;i++)
      {
	rel_curv=curv[i]/maxcurv[j];
	if(rel_curv>percentile && flag)
	  t[2*(nr-1)+j]=i;
	else 
	  flag=0;
      }
    }
  }

  return t;
}