/*---------------------------------------------------------------------
VERSION 1.0,COPYRIGHT 5. 1998 H.J. VOGEL
quantim IS LICENSED FREE OF CHARGE. THEREFORE THIS FILE,AND ALL THE ACCOMPANYING FILES WHICH IN THEIR ENTIRETY CONSTITUTE quantim, ARE PROVIDED "AS IS",WITHOUT WARRANTY OF ANY KIND,WHETHER EXPRESSED OR IMPLIED. YOU ARE RESPONSIBLE FOR
ASCERTAINING THE FITNES


%--------------------------------- hj 30.05.03
\command{       % COMMAND
image\_c *gWaterShed(image\_c *image, int conmode);
}{              % DESCRIPTION
Calculates the watershed lines for a grey image. The different basins separated by the watershed lines are marked by different grey values, the watershed is 0. This may be applied to a distance map of a binary image to separate overlapping  grains. 

\medskip        %ARGUMENTS AND RETURN VALUE
\argu{*image}{Pointer to the grey image}
\argu{conmode}{Connectivity mode: either 4 or 8.}
\return{Pointer to the image containing the watershed and the basins}
}

---------------------------------------------------------------------------*/
#include "quantim4.h"

#define mask -2
#define init -1
#define inqueue -3
#define wshed 0

image_cc *gWaterShed(image_cc *image, int conmode)
{

   	image_cc *out;
 	int i,j,k,p, x,y, xx,yy, change, val, val2, edge, th, thresh, row, col, d[8],pos;
	char buffer[127];
	int *watshed, *fifoqueue, cur_label=0, flag=0, wcount, rcount, abscount;
	
	if(conmode!=4 && conmode!=8){
	  printf("\ngWaterShed: no valid connectivity mode (must be 4 or 8)\n");
	  return NULL;
	}
	printf("\n calculating watershed ...\n");


	row=image->dim[1]; 
	col=image->dim[0];
	fifoqueue = (int *)malloc(col*row*sizeof(int));
	watshed   = (int *)malloc(col*row*sizeof(int));

	d[0] = -col;          /* neighborhood pixels */
	d[1] = 1;
	d[2] = col;
	d[3] = -1;
	d[4] = 1 - col;
	d[5] = 1 + col;
	d[6] = col - 1;
	d[7] = -1 - col; 


	for(i=0;i<col*row;i++)
	  watshed[i]=init;

	wcount=rcount=abscount=0;
	/*** mark local maxima ***/
	for(th=255; th>0; th--){
	  for(y=1;y<row-1;y++)
	    for(x=1;x<col-1;x++)
	      if(rPixel(x,y,image)==th){
		pos=x+y*col;
		watshed[pos]=mask;
		for(j=0;j<conmode;j++)
		  if(watshed[pos+d[j]]>=0){
		    watshed[pos]=inqueue;		
		    *(fifoqueue+wcount%(col*row)) = pos;
		    wcount++; abscount++;
		    break;
		  }

	      }
	  while(abscount){
	    pos = *(fifoqueue+rcount%(col*row));
	    rcount++;
	    abscount--;
	    for(j=0;j<conmode;j++){
		  if((val=watshed[pos+d[j]])>0){
		    if(watshed[pos]==inqueue || (watshed[pos]==wshed && flag==1) )
		      watshed[pos] = val;
		    else if(watshed[pos] >0 && watshed[pos]!=val){  
		      watshed[pos]=wshed;
		      flag=0;
		    }
		  }
	          else if(val==wshed){
		    if(watshed[pos]==inqueue){
		      watshed[pos]=wshed;
		      flag=1;
		    }
		  }
		  else if(val==mask){
		    watshed[pos+d[j]]=inqueue;
		    *(fifoqueue+wcount%(col*row)) = pos+d[j];
		    wcount++;
		    abscount++;
		  }
	    }
	  }
	 
	  for(y=1;y<image->dim[1]-1;y++)
	    for(x=1;x<image->dim[0]-1;x++)
	      if(rPixel(x,y,image)==th){
		pos=x+y*col;
		if(watshed[pos]==mask){
		  cur_label++;
		  *(fifoqueue+wcount%(col*row)) =pos;
		  wcount++; abscount++;
		  watshed[pos]=cur_label;
		  while(abscount){
		    pos = *(fifoqueue+rcount%(col*row));
		    rcount++;
		    abscount--;
		    for(j=0;j<conmode;j++){
		      if(watshed[pos+d[j]]==mask){
			*(fifoqueue+wcount%(col*row))=pos+d[j];
			wcount++; abscount++;			
			watshed[pos+d[j]]=cur_label;
		      }
		    }
		  }		  

		}
	      }
	}/** loop over thresholds **/


	out=CopyImage(image);
	for(y=1;y<row-1;y++)
	  for(x=1;x<col-1;x++)
	      wPixel(x,y,out,*(watshed+x+y*col));
	return out;  
}

#undef mask 
#undef init 
#undef inqueue 
#undef wshed 


