/*---------------------------------------------------------------------
VERSION 1.0,COPYRIGHT 5. 1998 H.J. VOGEL

quantim IS LICENSED FREE OF CHARGE. THEREFORE THIS FILE,AND ALL THE ACCOMPANYING
FILES WHICH IN THEIR ENTIRETY CONSTITUTE quantim, ARE PROVIDED "AS IS",WITHOUT
WARRANTY OF ANY KIND,WHETHER EXPRESSED OR IMPLIED. YOU ARE RESPONSIBLE FOR
ASCERTAINING THE FITNES
---------------------------------------------------------------------------*/

/*
quantim4.h: header file for quantim4

history:
  
modifications: lots

*/

/*----------------------------------------------------------------------- definitions */
#ifndef QUANTIM_4
#define QUANTIM_4

#include <float.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <iostream>

#include "string.h"
#include "tiff.h"
#include "tiffio.h"

#define NONE            -1
#define MASK 100  /*maximum diameter (bits) of structuring elements*/
#define Pac_Bytedepth 4
#define SWAP(a,b) {tempr=(a);(a)=(b);(b)=tempr;}
#define MAXDILAT 100

typedef struct
       {
       char             name[127];
       unsigned char    *pix;
       unsigned int     ndim;
       unsigned int     numchannel;
       unsigned int     *dim;
       unsigned int     numbits;
       unsigned int     numbytes;
       double		*res;
       char           itype[5]; //rgb gry gry16 ddd ddd16 btd 
       } image_cc;
typedef struct
       {
       	char    *name;
       	double    Euler[MASK];
       	double     Vv[MASK];
  	double     Sv[MASK];
	double	Sv3d[MASK];
       } pac_data;

typedef struct {
	int radz,radx;
	unsigned long Element[MASK][MASK];
	} Sphere;
static union {
	unsigned char *p;
	unsigned short *q;
	double *d;
        float *f;
	int *r; // U. Weller 2.08.07
} point;

static long IDUM=-1; 
static int RPS;

static int neigh4x[4] = {0,-1,0,1};
static int neigh4y[4] = {1,0,-1,0};      /*4 Neighbours starting below the central pixel*/
static int neigh8x[8] = {-1,-1,0,1,1,1,0,-1};
static int neigh8y[8] = {0,-1,-1,-1,0,1,1,1};  /*8 Neighbours starting at left the 
central pixel*/
static unsigned long DIRECT[5] ={0,0,0,0,0}; /*ILx, ILy, ILz, ILxy, ILxyz*/
static int N26x[26] = {-1,0,1,-1,0,1,-1,0,1,-1,0,1,-1,1,-1,0,1,-1,0,1,-1,0,1,-1,0,1};
static int N26y[26] = {-1,-1,-1,0,0,0,1,1,1,-1,-1,-1,0,0,1,1,1,-1,-1,-1,0,0,0,1,1,1};   
static int N26z[26] = {-1,-1,-1,-1,-1,-1,-1,-1,-1,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1};
static int N6x[6] = {-1,0,1, 0,0, 0};
static int N6y[6] = { 0,0,0, 0,1,-1};   
static int N6z[6] = { 0,1,0,-1,0, 0};


/*------------ Graphics ------------------*/
static int PSWIDTH = 110;
static int PSHIGHT = 90;
extern char *PSXTITLE;
extern char *PSYTITLE;

#define FONT "Helvetica"
#define YTIT -23
#define XTIT -14

/*------------------------------------------------------------------------- functions */
// old stuff
void gErode(image_cc *image,int step, int mode);
// actual functions
void DeleteImage(image_cc *image);
void gBibin(int LOW,int HIGH,image_cc *image);
int gBilevel(int *th_high,int *th_low,image_cc *image);
void gBin(int LOW, int value1,int value2,image_cc *image);
void bObjects(image_cc *image,unsigned long *o,unsigned long *l);
int bContour(int x, int y, unsigned char mark, image_cc *image);
void SavePaletteImage(image_cc *image, char *buf);
void bErode(image_cc *image,int step, int mode);
void SetFrame(image_cc *image,int value);
long bDisector(image_cc *wnd1, image_cc *wnd2);
void bLogAnd(image_cc *wnd1, image_cc *wnd2);
double pDisector(image_cc *wnd1, image_cc *wnd2);
void SaveShiftImage(image_cc *image, char *buf,int maxx, int maxy,int vx, int vy);
void gMean(image_cc *image, int mode);
void bLogOr(image_cc *wnd1, image_cc *wnd2);
void bAddition(image_cc *wnd1, image_cc *wnd2, int gv1, int gv2);
double b2DEuler(image_cc *image);
void gBin2(int LOW, int value1,int value2,image_cc *image, image_cc *out);
void gBinTwoLevel(int LOW,int HIGH, image_cc *image, image_cc *out);
void gMedian(image_cc *image, int mode);
char *GetSphereElement(int rad, double rx, double ry, double rz);
void BtdErodeFilter(image_cc *im, int step, int mode);
image_cc *BtdErode(image_cc *im, int step, int mode);
image_cc *BtdOpen(image_cc *im,int step,int mode);
void BtdContinuity(image_cc *im,int mode);
void D2Continuity(image_cc *im,int mode);
void SetBtdShell(image_cc *im, int dx , int dy, int dz, int mode);
void DLine(int x1,int y1,int x2,int y2,image_cc *image, int val);
int bContourCent(int x, int y, int mark, image_cc *image, int *xx, int *yy);
image_cc *RGBtoGray(image_cc *image);
void StretchRGBHisto(image_cc *in, int Rlow, int Rhigh, int Glow, int Ghigh, int Blow, int Bhigh);
void StretchRGBBright(image_cc *in, int Low, int High);
float ran3(long *idum);
image_cc *GetRandDDDImage(int col,int row,int dep,double rx,double ry, double rz);
image_cc *gDiff(image_cc *wnd1,image_cc *wnd2);
void error(int ernum);
void DddMinMax(image_cc *im,int mode);
image_cc *DddBin(image_cc *im, int thresh);
void DddClas(image_cc *im, int Nclas, unsigned char *th, unsigned char *gval);
double BtdEuler(image_cc *im);
double *DddHisto(image_cc *im, int precision);
double *DddCircHisto(image_cc *im, int precision, int rad);
void DddEulerFunc(image_cc *im, int *num, double **xdat, double **ydat, int prec);
void psPlot(char *fname, int n, double *xdat, double *ydat, int mode);
void psDddCircHisto(image_cc *im, char *buf, int prec);
void qdefault(void);
image_cc *GetCorDDDImage(double *cdf, int col,int row,int dep,double rx,double ry, double rz,double cx,double cy, double cz, int mode);
void grf3(double ***sfa,double DX,double DY,double DZ,double LAMX,double LAMY,double LAMZ,double VAR,int N,int TYPE);
double *GetDddGreyCdf(image_cc *im);
void psMultiPlot(char *fname, int *ndat, int nplot, double **xdat, double **ydat, int mode);
void StretchHisto(image_cc *image, int Low, int High);
void StretchHisto(image_cc *image);
void StretchHisto(image_cc *image, double th);
void gLaplace(image_cc *image);
void gSobel(image_cc *image);
double *gHisto(image_cc *im, int precision);
void psHisto(image_cc *im, char *buf, int prec);
void psHisto(image_cc *im, char *buf);
void bThinning(image_cc *image);
image_cc *CopyImage(image_cc *image);
void bConCom(image_cc *image,int xv,int yv,int val);
void bHitMiss(image_cc *image,int Mx, int My, long MP, long MNP);
void bVolSurf(image_cc *image, double *vv, double *sv, double res, int step);
image_cc *DddResRed(image_cc *image, int mode);
double *GetDddAcov(image_cc *im, double *corl, int lag, int mode);
image_cc *gRedRes(image_cc *image, int mode); 
double BtdVolSurf(image_cc *image, double *vv, double *sv);
void BtdSkelet(image_cc *in, int mode, int deadends);
double BtdDiffusionZ(image_cc *in, FILE *dif, unsigned long max, unsigned long min, int fluxstep, double sens);
double BtdEuler26(image_cc *image, long int *h);
double BtdEuler6(image_cc *image, long int *h);
double BtdMeancurv(image_cc *image, long int *h);
long int *BtdQuant(image_cc *image);
double BtdSurfdens(image_cc *image, long int *h);
double BtdVoldens(long int *h);
double *GetDddAcov(image_cc *im, double *corl, int lag, int mode);
image_cc *GetCorImage(double *cdf, int col,int row,double rx,double ry, double cx,double cy, int mode, double **acov);
image_cc *GetCorImage(double *cdf, int col,int row,double rx,double ry, double cx,double cy, int mode);
double *GetGreyCdf(image_cc *im);
void MinMaxf(double *ar, int n, double *min, double *max);
long *gConfunc(image_cc *inim);
int bConCom2(image_cc *image,int xv,int yv);
void bConCom(image_cc *image,int xv,int yv,int val);
image_cc *gMean2(image_cc *image, int mode);
image_cc *gCircMask(image_cc *im, int xmid, int ymid, int rad, int val);
void bRemObjects(image_cc *image, int size);
image_cc *GetCorDDDImage2(double *cdf, int col, int row, int dep, double rx, double ry, double rz, double cx, double cy, double cz, int mode);
image_cc *Btd2Ddd(image_cc *image);
double *GetAcov(image_cc *im, double *corl, int lag, int mode);
void grf2ext(double*sfa,int NX,int NY,double DX,double DY,double LAMX,double LAMY,double VAR,int N,int TYPE, double **acov);
void grf2(double*sfa,int NX,int NY,double DX,double DY,double LAMX,double LAMY,double VAR,int N,int TYPE);
double *GetSemiVar(image_cc *im, int lag, int mode);
void psMultiPlotTit(char *fname,char *xtitle,char *ytitle,int *ndat, int nplot, double **xdat, double **ydat, int mode);
void psPlotTit(char *fname,char *xtitle,char *ytitle, int n, double *xdat, double *ydat, int mode);
image_cc *GetVoronoiTes(int Xdim, int Ydim , int Nump, int mode);
image_cc *GetVoronoiTes(int Xdim, int Ydim , int Nump);
image_cc *GetPercolClus(int width, int height, double lamx, double lamy, int mode);
long int *bQuant(image_cc *image);
double bEuler4(image_cc *image, long int *h);
double bEuler8(image_cc *image, long int *h);
double bLengthdens(image_cc *image, long int *h);
double bVoldens(image_cc *image, long int *h);
double bSurfdens(image_cc *image, long int *h);
double bAverageCurvature(image_cc *image, long int *h);
void DddBibin(int LOW,int HIGH,image_cc *im);
void gHistoMatch(double *cdf, image_cc *image);
char *GetCircElement(int rad, double rx, double ry);
long int *BtdQuantMask(image_cc *image, int xm, int ym, int zm, char *mask);
long int *bQuantMask(image_cc *image, int xm, int ym, char *mask);
long int *bQuantRecMask(image_cc *image, int xdim,int ydim, int xul, int yul);
double **dmatrix(int n, int m);
int    **imatrix(int n, int m);
void nextline(FILE *fp);
int nextinput(FILE*fn);
unsigned long bErodeMark(image_cc *image, int mark, int step);
image_cc *bGetDistMap(image_cc *image, int *n, int gval, int gstep);
void BtdDrawSphere(image_cc *image,  int xmid, int ymid, int zmid, int rad,  int val);
image_cc *BtdErodeMir(image_cc *im, int step, int mode);
image_cc *BtdGetDistMap(image_cc *image, int *n, int gval, int gstep);
int BtdGetDistOpenMap(image_cc *image, image_cc *distance, image_cc *opened,  int gval, int gstep);
void Dx(image_cc *image, char *buf);
void DddDrawCylinder(image_cc *image, int xmid, int ymid, int zmid, int rad, int len,  int val);
image_cc *DddWaterShed(image_cc *image, int conmode);
image_cc *gWaterShed(image_cc *image, int conmode);
void SetDddShell(image_cc *im, int thick, int val);
image_cc *ScaleRGBImage(image_cc *image, double sc);
unsigned char Red(image_cc *in, int x, int y);
unsigned char Green(image_cc *in, int x, int y);
unsigned char Blue(image_cc *in, int  x, int y);
void WRed(image_cc *in,int   x,int  y, unsigned char val);
void WGreen(image_cc *in, int  x,int  y, unsigned char val);
void WBlue(image_cc *in, int  x,int  y, unsigned char val);
void WRGBPixel(image_cc *in, int  x,int  y,unsigned char rval,unsigned char gval,unsigned char bval);
image_cc *BtdErodeMirCond(image_cc *im,image_cc *cond, int step, int mode);
image_cc *bErodeMirCond(image_cc *im, image_cc *cond, int step, int mode, int fast);
image_cc *bGetDistMap(image_cc *image, int *n, int gval, int gstep);
void bErodeMir(image_cc *im, int step, int mode);
image_cc *bErodeMirCirc(image_cc *im, int step, int mode);
int bGetDistOpenMap(image_cc *image, image_cc *distance, image_cc *opened, int gval, int gstep);
void bThinning8(image_cc *image);
void weights(double *Delta,double *weight);
void fourn(float *data, unsigned long *nn, int ndim, int isign);
double I0(double x);
double K0(double x);
void set_line(int i);
char *GetSphereElementDouble(double rad, double rx, double ry, double rz, int *vol);
image_cc *BtdErodeMirCondDouble(image_cc *im,image_cc *cond, double step, int mode); 
int BtdGetDistOpenMapDouble(image_cc *image, image_cc *distance, image_cc *opened,  int gval, int gstep, double step);
image_cc *BtdErodeMirDouble(image_cc *im, double step, int mode);
void ps3Dview(char *fname, image_cc *image, double min, double max);
image_cc *GetRandImage(int col,int row,double rx,double ry, double cx,double cy, int mode);
image_cc *LoadImage(char *buf);
void StoreImage(image_cc *im, char *buf);
void wPixel(int x, int y, image_cc *im, int value);
void wPixel(int x, int y, image_cc *im, int *value);
void wPixel(int x, int y, int z, image_cc *im, int value);
void wPixel(int x, int y, image_cc *im, int vr, int vg,int vb);
void wPixel(unsigned long i, image_cc *im, int value);
void wPixel(int x, int y, int z, image_cc *im, double value);
void wPixel(int x, int y, image_cc *im, double value);
void wPIxel(unsigned long i, image_cc *im, double value);
int rPixel(int x, int y, image_cc *im);
int rPixel(int x, int y, image_cc *im, int *value);
int rPixel(int x, int y, int z, image_cc *im);
int rPixel(int x, int y, image_cc *im, int *vr, int *vg, int *vb);
int rPixel(unsigned long i, image_cc *im);
image_cc *InitImage(int x, int y, int val);
image_cc *InitImage(int x, int y, int val, char *type);
image_cc *InitImage(int x, int y, int z, int val);
image_cc *InitImage(int x, int y, int z, int val, char *type);
image_cc *InitImage(image_cc *image, int val);
image_cc *InitImage(image_cc *image);
image_cc *ImageSegment(image_cc *image, int ulx, int uly, int dx, int dy);
image_cc *ImageSegment(image_cc *image, int ulx, int uly, int ulz, int dx, int dy, int dz);
image_cc *GetPlane(image_cc *im,int plane, int mode);
void InvertImage(image_cc *im);
void FlipImage(image_cc *im, int mode);
void RotateImage(image_cc *im, int mode);
image_cc *TurnImage(image_cc *im,double grad);
int isborder(int x, int y, image_cc *im);
int isborder(int x, int y, int z, image_cc *im);
void Mean(image_cc *image, int mode);
void Median(image_cc *image, int mode);
unsigned long CountLines(FILE *fp);
void Gauss(image_cc *image, int mode,double sigma);
image_cc *bGetFullDistMap(image_cc *image, int *nblack, int *nwhite ,int gstruc, int gval, int gstep);
image_cc *gLaplaceRescale(image_cc *image);
image_cc *gLaplace(image_cc *image, int mode);
image_cc *gSobelRescale(image_cc *image);
image_cc *gSobel(image_cc *image, int mode);
void GetMinMax(image_cc *im, int *max, int *min);
//******new stuff
image_cc *Diff(image_cc *wnd1,image_cc *wnd2);
image_cc *gry16to8(image_cc *im);
image_cc *ddd16to8(image_cc *im);
image_cc *bit16to8(image_cc *im);
double *Histo(image_cc *im);
image_cc *Mask(image_cc *im, int thresh, int size);
void MinMax(image_cc *im, double size, int mode);
void Luul(image_cc *im, double size);
void Ullu(image_cc *im, double size);
void g2MinMax(image_cc *im, double size, int mode);
void g3MinMax(image_cc *im, double size, int mode);
image_cc *ChangeResolution( image_cc *in, int newx, int newy );
//***************24/09/07************
void Bin(image_cc *im, int thresh);
void Bin(image_cc *im, int thresh, int lower, int upper);
void BinBilevel(image_cc *im, int lower, int upper);
void BinBand(image_cc *im, int lthresh, int uthresh);
int CalcBilevel(image_cc *im, int *th_high,int *th_low); //to be improved
image_cc *CreateDDDImage(char *buf, int z1, int z2,double rx,double ry,double rz);
//***************07/02/08************
image_cc *Ddd2Btd(image_cc *im);
char *GetCircElementDouble(double rad, double rx, double ry);
image_cc *CircMask(image_cc *im, int xmid, int ymid, int rad);
image_cc *GetDistMap(image_cc *im);//, double gstep);
image_cc *GetDistOpenCloseMap(image_cc *im, int maxdist);// double gstep, int maxdist);
image_cc *GetDistOpenCloseMap(image_cc *im);//, double gstep);
image_cc *ErodeCirc(image_cc *im, double rad, int mode);
image_cc *OpenCirc(image_cc *im, double rad, int mode);
image_cc *ErodeDist(image_cc *im, double dist, int mode);
//image_cc *OpenDist(image_cc *im, double rad, int mode);
image_cc *newb2GetDistMap(image_cc *im, double gstep);
image_cc *newb3GetDistMap(image_cc *im, double gstep);
image_cc *newb2GetOpenCloseMap(image_cc *im, double gstep, int maxdist);
image_cc *newb3GetOpenCloseMap(image_cc *im, double gstep, int maxdist);
image_cc *MeanVar(image_cc *image, int mode);
double rPIxel(int x, int y, image_cc *im);
double rPIxel(int x, int y, int z, image_cc *im);
double rPIxel(int i, image_cc *im);
double *Histo(image_cc *im);
double *GreyCdf(image_cc *im);
double *Acov(image_cc *im, double *corl, int lag, int mode);
double *SemiVar(image_cc *im, int lag, int mode);
image_cc *WaterShed(image_cc *image, int conmode);
image_cc *InitRandImage(int col,int row,double rx,double ry, double cx,double cy, int mode, double *cdf);
image_cc *InitRandImage(int col,int row,int dep,double rx,double ry, double rz,double cx,double cy, double cz, int mode, double *cdf);
double *GreyConFunc(image_cc *inim, int mode);
void HistoMatch(double *cdf, image_cc *image);
long int *m2Quant(image_cc *image);
long int *m2Quant(image_cc *image, image_cc *maskim);
long int *m3Quant(image_cc *image);
long int *m3Quant(image_cc *image, image_cc *maskim);
long int *m3QuantGrad(image_cc *image, image_cc *maskim);
double m2areadens(image_cc *image, long int *h);
double m2lengthdens(image_cc *image, long int *h);
double m2surfdens(image_cc *image, long int *h);
double m2euler4(image_cc *image, long int *h);
double m2euler8(image_cc *image, long int *h);
double m3voldens(long int *h);
double m3surfdens(image_cc *image, long int *h);
double m3meancurv(image_cc *image, long int *h);
double m3euler6(image_cc *image, long int *h);
double m3euler26(image_cc *image, long int *h);
image_cc *LogAnd(image_cc *wnd1, image_cc *wnd2);
image_cc *LogOr(image_cc *wnd1, image_cc *wnd2);
image_cc *Intersection(image_cc *wnd1, image_cc *wnd2, int gv1, int gv2);
image_cc *ErodeCircCond(image_cc *im, image_cc *cond, double rad, int mode);
image_cc *OpenDist(image_cc *im, image_cc *distmap, double rad, int mode);// double gstep, int mode);
char *GetCircElementDouble_new(double rad, double rx, double ry);
void SavePrinciplePlanes(image_cc *im, char *name);
void GreyClas(image_cc *image, int Nclas, int *th, int *gval);
image_cc *GetOpenMap(image_cc *distmap, int smax);// double gstep, double sstep, int smax);
int GetMaxGrey(image_cc *im);
int GetMinGrey(image_cc *im);
void MinkowskiOpenFunctions(image_cc *image, char *outfile);// double gstep, double pixstep, char *outfile);
void MinkowskiOpenFunctions(image_cc *image, image_cc *maskim, char *outfile);// double gstep, double pixstep, char *outfile);
void MinkowskiFunctions(image_cc *image, char *outfile);// double gstep, double pixstep, char *outfile);
image_cc *ChangeResolution( image_cc *in, int newx, int newy , int newz );

/*********************************************//**
        * \brief windowed mean
        *
        * Filters with an averaging kernel defined over a spherical kernel.
        * Boundaries are mirrored. Note that dimensions smaller than the
        * kernel radius will cause problems.
        ******************************************/

void wMean( image_cc *im, double size);
void wMean_f( image_cc *im, double size);
void wGauss_f( image_cc *im, double size, double sigma);
image_cc *Laplace(image_cc *image, int mode);
image_cc *Laplace(image_cc *image, int mode, image_cc *mask);
image_cc *Sobel(image_cc *image);
image_cc *Sobel(image_cc *image, image_cc *mask);
int UnimThresh(double *histo, int mode);
double ConLength(image_cc *image,int phase, int every, char *outfile);
/*********************************************//**
        * \brief pseudo Median absolute deviation
        *
        *  The mdeian absolute deviation is the median of the
        *  absolute differences towards the median.
        *  For reason of speed the median is replaced by
        *  the luul filter.
        *
        *  \param orig pointer to original image
        *  \param pMed pointer to image containing the luul filtered
        *         image. If NULL, this will be created.
        *  \param size radius of spherical window to use.
        *
        ******************************************/
void pseudoMad(image_cc *orig, image_cc **pMed, double size);
void pseudoMad(image_cc *orig, double size);

/*********************************************//**
        * \brief Rudin, Osher, Fatemi filter
        *
        *  This filter minimizes the total variation while keeping
        *  a maximal fidelity to the original image. The strength
        *  of fidelity is given by $\lambda$.
        *
        *
        *  \returns nothing worthwhile
        *  \param timestep timestep for each filtering. If negative, will be handled dynamically. -0.1 normally
        *         works well.
        *  \param lambda fidelity parameter. 5 is a good start.
        *  \param maxstep maximal number of filtering steps.
        *  \param intervall number of steps after which the image in memory will be updated.
        *  \param function kernel functions for filtering. 0: classical Total variation, 1: Entropy, 2: Enhanced entropy
        *
        ******************************************/
int rof_filter(image_cc *image, double timestep, double lambda, int maxstep, int intervall, int function);
void DiffGauss(image_cc *image, int mode,double sigma, double sigma2);
void UnsharpMask(image_cc *image, int size,double sigma);
image_cc *Cluster(image_cc *image, int *nr);
image_cc *Cluster(image_cc *image, image_cc *mask, int *nr);
double *Histo(image_cc *im,image_cc *mask);
double *GreyCdf(image_cc *im, image_cc *mask);
int *MaxVarThresh(double *cm0, int nr);
int *MaxVarThresh(double *cm0, int nr, double percentile);
int *MaxEntroThresh(double *cm0, int nr);
int *MaxEntroThresh(double *cm0, int nr, double percentile);
int *MinErrThresh(double *cm0, int nr);
int *MinErrThresh(double *cm0, int nr, double percentile);
int *KMeansThresh(double *cdf, int nr);
void TotVarFilter(image_cc *image, double timestep, double lambda, int maxstep, int interval, int function); //just a intuitive synonym of rof_filter
double *Histo16(image_cc *im);
double *Histo16(image_cc *im, image_cc *mask);
double *GreyCdf16(image_cc *im);
double *GreyCdf16(image_cc *im, image_cc *mask);
int *GradMaskThresh(image_cc *image, double alpha);
int *GradMaskThresh(image_cc *image, double alpha, image_cc *mask);
int *FuzzyCMeansThresh(double *cdf, int nr, double tau);
int *FuzzyCMeansThresh(double *cdf, int nr, double tau, double percentile);
int *ShapeThresh(double *cdf, int nr);
int *ShapeThresh(double *cdf, int nr, double percentile);
void *FastMedian(image_cc *im, int size);
void *AnisoDiffFilter(image_cc *in, int maxstep, double threshold, double sigma);
image_cc *MajorityFilter(image_cc *clas, int nr, int wnd, double maj, double rel);
image_cc *MajorityFilter(image_cc *clas, image_cc *roi, int nr, int wnd, double maj, double rel);
void RemoveObjects(image_cc *im, int size, int mode);
void RemoveObjects(image_cc *im, image_cc *roi, int size, int mode);
void StoreRaw(image_cc *im);
image_cc *LoadRaw(char *buf, int cols, int rows, int layers, int offset, int nbyte);
int isPercol(image_cc *image,int backbone, int mode);
//18.12.2014 SS
image_cc *LoadMetaImage(char *buf);
void StoreMetaImage(image_cc *im);
//04.01.2016
void StoreMetaImage(image_cc *im, char *buf);
void StoreRaw(image_cc *im, char *buf);
void StoreImage(image_cc *im);
#endif
